/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../formalyze/util/date_utils.dart';
import '../structure/ordered_map.dart';
import 'abstract_bean.dart';
import 'abstract_isl_resp_metadata.dart';
import 'enums/base_server_response_code.dart';
import 'isl_resp.dart';
import 'register/response_message.dart';

abstract class AbstractIslResp extends AbstractBean implements IslResp {
  String trxId;
  int instanceId;
  int idCompany;
  int respCode;
  String respDesc;
  ResponseMessage customMessage;
  OrderedMap<String> arrayOptions; // for CMS MessageArray
  Map<String, Object> data; // for CMS RMI data handling
  DateTime createdDateTime;

  AbstractIslResp() {
    createdDateTime = DateUtils.now();
  }

  AbstractIslResp.basic(this.trxId, this.respCode, this.respDesc);

  String getTrxId() {
    return trxId;
  }

  @override
  void setTrxId(String trxId) {
    this.trxId = trxId;
  }

  int getInstanceId() {
    return instanceId;
  }

  @override
  void setInstanceId(int instanceId) {
    this.instanceId = instanceId;
  }

  int getIdCompany() {
    return idCompany;
  }

  @override
  void setIdCompany(int idCompany) {
    this.idCompany = idCompany;
  }

  int getRespCode() {
    return respCode;
  }

  bool isSucessCode() {
    return respCode == 0 ||
        respCode == BaseServerResponseCode.SUCCESS.getCode();
  }

  @override
  void setRespCode(int respCode) {
    this.respCode = respCode;
  }

  String getRespDesc() {
    return respDesc;
  }

  @override
  void setRespDesc(String respDesc) {
    this.respDesc = respDesc;
  }

  DateTime getCreatedDateTime() {
    return createdDateTime;
  }

  @override
  void setCreatedDateTime(DateTime createdDateTime) {
    this.createdDateTime = createdDateTime;
  }

  ResponseMessage getCustomMessage() {
    return customMessage;
  }

  void setCustomMessage(ResponseMessage customMessage) {
    this.customMessage = customMessage;
  }

  OrderedMap<String> getArrayOptions() {
    return arrayOptions;
  }

  void setArrayOptions(OrderedMap<String> arrayOptions) {
    this.arrayOptions = arrayOptions;
  }

  Map<String, Object> getData() {
    return data;
  }

  void setData(Map<String, Object> data) {
    this.data = data;
  }

  @override
  Set<String> getFields() => AbstractIslRespMetaData.getFields();

  @override
  dynamic getFieldValue(String field) =>
      AbstractIslRespMetaData.getFieldValue(field, this);
}
