/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../formalyze/config/defaults.dart';
import '../../formalyze/generator/reader/json_reader.dart';
import '../../formalyze/util/utils.dart';
import '../../formalyze/validation/converter/c_boolean.dart';
import '../structure/ordered_map.dart';
import '../structure/string_object_map.dart';
import 'register/crm_form_data.dart';
import 'register/mpos_info_data.dart';

abstract class IReflect {
  String getXmlTag() => null;

  Set<String> getFields();

  Set<String> getTransientFields() => {};

  Map<String, String> getPropertyMap() => {};

  Map<String, String> getEntryMap() => getPropertyMap();

  Map<String, dynamic> toMap() {
    Map<String, dynamic> map = {};

    for (String field in getFields()) {
      final dynamic value = getFieldValue(field);

      if (!Utils.isBlank(value)) {
        map[field] = value;
      }
    }

    return map;
  }

  String toJson() => JsonReader().toJson(this);

  dynamic getFieldValue(String field);

  void setFieldValue(String field, Object value) {}

  // Utility methods are listed below

  static Map<String, String> transformMap(Map map) => _transformMap(map, {});

  static StringObjectMap transformMap_(Map map) =>
      _transformMap(map, StringObjectMap());

  static OrderedMap<String> transformMap__(Map map) =>
      _transformMap(map, OrderedMap<String>());

  static Map<String, bool> transformMap___(Map map) =>
      _transformMap(map, <String, bool>{},
          converter: (value) => CBoolean.parseNoErr(value));

  static List<T> transformToList<T>(Map map, T Function(Map map) transformer) {
    if (map == null) {
      return null;
    }

    List<T> formDetails = [];

    for (Map entry in map.values) {
      formDetails.add(transformer(entry));
    }

    return formDetails;
  }

  static T _transformMap<T extends Map>(Map source, T destination,
      {dynamic Function(String value) converter}) {
    if (source == null) {
      return null;
    }

    final bool noConverter = converter == null;

    for (Map entry in source.values) {
      String value = entry[Defaults.VALUE];
      destination[entry[Defaults.KEY]] = noConverter ? value : converter(value);
    }

    return destination;
  }

  static T newInstance<T>(String clazz) {
    if (Utils.match(
        clazz, ['CrmFormData', 'com.mt.form.bean.register.CrmFormData'])) {
      return CrmFormData() as T;
    }

    if (Utils.match(
        clazz, ['MposInfoData', 'com.mt.form.bean.register.MposInfoData'])) {
      return MposInfoData() as T;
    }

    if (Utils.match(
        clazz, ['StringObjectMap', 'com.mt.form.structure.StringObjectMap'])) {
      return StringObjectMap() as T;
    }

    // TODO Add any beans here (if missing / needed)

    return null;
  }
}
