/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../formalyze/error/illegal_argument_exception.dart';
import '../i_reflect.dart';
import '../register/crm_form_data_metadata.dart';
import 'base_mpos_state_metadata.dart';
import 'i_mpos_state.dart';

class BaseMposState extends IReflect implements IMposState {
  static final Map<int, IMposState> ENUMS_MAP = {};

  // Match with IN_PROGRESS 100
  static final BaseMposState PENDING_ISL = BaseMposState(0, "PENDING");
  static final BaseMposState SUCCEEDED = BaseMposState(1, "SUCCEEDED");
  static final BaseMposState FAILED = BaseMposState(2, "FAILED");
  static final BaseMposState ABORTED = BaseMposState(3, "ABORTED");

  // Match with 102
  static final BaseMposState DOCUMENT_PENDING =
      BaseMposState(4, "DOCUMENT - PENDING");

  // Internal state
  static final BaseMposState PENDING_MPOS = BaseMposState(5, "PENDING - MPOS");
  static final BaseMposState DUPLICATE = BaseMposState(6, "DUPLICATE");
  static final BaseMposState WRONG_MSISDN = BaseMposState(7, "WRONG_MSISDN");
  static final BaseMposState APPROVED_SUCCESS =
      BaseMposState(8, "APPROVED_SUCCESS");
  static final BaseMposState SENDING = BaseMposState(9, "SENDING");
  static final BaseMposState OPERATOR_RECEIVED =
      BaseMposState(10, "OPERATOR_RECEIVED");
  static final BaseMposState WAITING_FOR_PRIMARY_MSISDN_SUCCESS =
      BaseMposState(11, "WAITING_FOR_PRIMARY_MSISDN_SUCCESS");
  static final BaseMposState DUPLICATE_CREDENTIAL =
      BaseMposState(12, "DUPLICATE_CREDENTIAL");
  static final BaseMposState INVALID_PIN_CODE =
      BaseMposState(13, "INVALID_PIN_CODE");
  static final BaseMposState EXPIRED_PIN_CODE =
      BaseMposState(14, "EXPIRED_PIN_CODE");
  static final BaseMposState IMEI_NOT_AUTHORIZED =
      BaseMposState(15, "IMEI_NOT_AUTHORIZED");
  static final BaseMposState IMEI_NOT_MATCHED =
      BaseMposState(16, "IMEI_NOT_MATCHED");

  // User can't edit and resend the transaction.
  static final BaseMposState PERMANENTLY_ABORTED =
      BaseMposState(17, "PERMANENTLY_ABORTED");

  // State 405
  static final BaseMposState REJECTED = BaseMposState(18, "REJECTED");

  static final BaseMposState UNKNOWN = BaseMposState(19, "UNKNOWN");

  // State 215
  static final BaseMposState SUCCESS_DELAYED =
      BaseMposState(20, "SUCCESS_DELAYED");

  final int value;
  final String displayName;

  BaseMposState(this.value, this.displayName) {
    ENUMS_MAP[value] = this;
  }

  BaseMposState.empty() : this(-1, null);

  @override
  int getValue() {
    return value;
  }

  @override
  String getDisplayName() {
    return displayName;
  }

  static IMposState fromInteger(int value) {
    IMposState mposState = ENUMS_MAP[value];
    if (mposState == null) {
      throw IllegalArgumentException("Unknown transaction state value: $value");
    }
    return mposState;
  }

  @override
  bool operator ==(Object obj) {
    if (identical(this, obj)) {
      return true;
    } else if (obj == null || !(obj is BaseMposState)) {
      return false;
    }
    // ignore: test_types_in_equals
    return value == (obj as BaseMposState).value;
  }

  @override
  String toString() => displayName;

  @override
  int get hashCode => value;

  @override
  String getXmlTag() => CrmFormDataMetaData.state;

  @override
  Set<String> getFields() => BaseMposStateMetaData.getFields();

  @override
  dynamic getFieldValue(String field) =>
      BaseMposStateMetaData.getFieldValue(field, this);
}
