/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../formalyze/error/illegal_argument_exception.dart';
import '../../../formalyze/validation/converter/c_boolean.dart';
import '../../structure/string_object_map.dart';
import '../abstract_bean.dart';
import '../mpos/base_mpos_state.dart';
import '../mpos/i_mpos_state.dart';
import 'add_reg_data_metadata.dart';

class AddRegData extends AbstractBean {
  static const String ID_KEY = "SYS_SID";
  static const String MSISDN_KEY = "msisdn";
  static const String ICCID_KEY = "iccid";
  static const String EXT_ISSMSSENT = "EXT_ISSMSSENT";

  String trxId;
  int trxStatusId;
  String trxStatusDescription;
  int trxTypeId;
  String trxEventId;
  int trxEventTypeId;
  String primaryMsisdn;
  StringObjectMap map;

  // Transaction State in mPOS only
  IMposState state = BaseMposState.PENDING_MPOS;
  String stateInfo;

  // retried time that mPOS trying to send data to backEnd
  int retries = 0;
  DateTime startRegistrationTime;
  DateTime endRegistrationTime;
  DateTime sendRegistrationTime;
  DateTime receivedRegistrationTime;
  DateTime createdDateTime;
  DateTime updatedDateTime;
  int smsRetries = 0;

  int getSmsRetries() {
    return smsRetries;
  }

  void setSmsRetries(int smsRetries) {
    this.smsRetries = smsRetries;
  }

  bool isSmsSent() {
    return map.containsKey(EXT_ISSMSSENT) &&
        CBoolean.parse(map.getAsString(EXT_ISSMSSENT));
  }

  void setSmsSent(bool smsSent) {
    map.put(EXT_ISSMSSENT, smsSent);
  }

  AddRegData() {
    map = StringObjectMap();
  }

  int getSysId() {
    if (!map.containsKey(ID_KEY)) {
      throw IllegalArgumentException(
          "The SYS_ID has not been found in the map !");
    }
    return int.parse(map.getAsString(ID_KEY));
  }

  String getMsisdn() {
    return map.getAsString(MSISDN_KEY);
  }

  void setMsisdn(String msisdn) {
    map.put(MSISDN_KEY, msisdn);
  }

  String getIccid() {
    return map.getAsString(ICCID_KEY);
  }

  void setIccid(String iccid) {
    map.put(ICCID_KEY, iccid);
  }

  String getTrxId() {
    return trxId;
  }

  void setTrxId(String trxId) {
    this.trxId = trxId;
  }

  int getTrxStatusId() {
    return trxStatusId;
  }

  void setTrxStatusId(int trxStatusId) {
    this.trxStatusId = trxStatusId;
  }

  String getTrxStatusDescription() {
    return trxStatusDescription;
  }

  void setTrxStatusDescription(String trxStatusDescription) {
    this.trxStatusDescription = trxStatusDescription;
  }

  int getTrxTypeId() {
    return trxTypeId;
  }

  void setTrxTypeId(int trxTypeId) {
    this.trxTypeId = trxTypeId;
  }

  StringObjectMap getMap() {
    return map;
  }

  void setMap(StringObjectMap map) {
    this.map = map;
  }

  String getTrxEventId() {
    return trxEventId;
  }

  void setTrxEventId(String trxEventId) {
    this.trxEventId = trxEventId;
  }

  int getTrxEventTypeId() {
    return trxEventTypeId;
  }

  void setTrxEventTypeId(int trxEventTypeId) {
    this.trxEventTypeId = trxEventTypeId;
  }

  IMposState getState() {
    return state;
  }

  void setState(IMposState state) {
    this.state = state;
  }

  String getStateInfo() {
    return stateInfo;
  }

  void setStateInfo(String stateInfo) {
    this.stateInfo = stateInfo;
  }

  int getRetries() {
    return retries;
  }

  void setRetries(int retries) {
    this.retries = retries;
  }

  String getPrimaryMsisdn() {
    return primaryMsisdn;
  }

  void setPrimaryMsisdn(String primaryMsisdn) {
    this.primaryMsisdn = primaryMsisdn;
  }

  DateTime getStartRegistrationTime() {
    return startRegistrationTime;
  }

  void setStartRegistrationTime(DateTime startRegistrationTime) {
    this.startRegistrationTime = startRegistrationTime;
  }

  DateTime getEndRegistrationTime() {
    return endRegistrationTime;
  }

  void setEndRegistrationTime(DateTime endRegistrationTime) {
    this.endRegistrationTime = endRegistrationTime;
  }

  DateTime getSendRegistrationTime() {
    return sendRegistrationTime;
  }

  void setSendRegistrationTime(DateTime sendRegistrationTime) {
    this.sendRegistrationTime = sendRegistrationTime;
  }

  DateTime getReceivedRegistrationTime() {
    return receivedRegistrationTime;
  }

  void setReceivedRegistrationTime(DateTime receivedRegistrationTime) {
    this.receivedRegistrationTime = receivedRegistrationTime;
  }

  DateTime getCreatedDateTime() {
    return createdDateTime;
  }

  void setCreatedDateTime(DateTime createdDateTime) {
    this.createdDateTime = createdDateTime;
  }

  DateTime getUpdatedDateTime() {
    return updatedDateTime;
  }

  void setUpdatedDateTime(DateTime updatedDateTime) {
    this.updatedDateTime = updatedDateTime;
  }

  @override
  int get hashCode => 31 * 1 + (trxId == null ? 0 : trxId.hashCode);

  @override
  bool operator ==(Object obj) {
    if (identical(this, obj)) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (runtimeType != obj.runtimeType) {
      return false;
    }
    // ignore: test_types_in_equals
    AddRegData other = obj as AddRegData;
    if (trxId == null) {
      if (other.trxId != null) {
        return false;
      }
    } else if (trxId != other.trxId) {
      return false;
    }
    return true;
  }

  @override
  Set<String> getFields() => AddRegDataMetaData.getFields();

  @override
  dynamic getFieldValue(String field) =>
      AddRegDataMetaData.getFieldValue(field, this);
}
