/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../formalyze/error/illegal_argument_exception.dart';
import '../../../formalyze/util/date_utils.dart';
import '../../../formalyze/util/utils.dart';
import '../../structure/string_map.dart';
import '../../structure/string_object_map.dart';
import '../abstract_bean.dart';
import '../i_security_token.dart';
import '../isl_req.dart';
import '../isl_resp.dart';
import '../master_data_version_bean.dart';
import '../mpos/base_mpos_state.dart';
import '../mpos/i_mpos_state.dart';
import 'add_reg_data.dart';
import 'crm_form_data_metadata.dart';
import 'mpos_info_data.dart';
import 'reason.dart';

class CrmFormData extends AbstractBean
    implements IslReq, IslResp, ISecurityToken {
  // XXX these fields cannot be removed from O&M configuration screens +
  // FIRSTNAME and LASTNAME (used in the search process...)
  static const String ID_KEY = "SYS_SID";
  static const String MSISDN_KEY = "msisdn";
  static const String ICCID_KEY = "iccid";
  static const String KITNO_KEY = "kitNumber";
  static const String PUKNO_KEY = "pukNumber";
  static const String DEALER_KEY = "DEALER_NAME";
  static const String FIRSTNAME_KEY = "FIRSTNAME";
  static const String LASTNAME_KEY = "LASTNAME";
  static const String SUBSCRIBER_KEY = "SUBSCRIBER";
  static const String SECURITY_TOKEN = "securityToken";
  static const String TRX_ID = "trxId";
  static const String TRX_STATUS_ID = "trxStatusId";

  static const String PREFERRED_LANGUAGE_KEY = "preferredlanguage";
  static const String DEFAULT_LANGUAGE = "EN";
  static const String REJECTED_REASON = "rejectedreason";
  static const String REJECTED_COMMENT = "rejectedcomment";
  static const String SYS_FORM_ID = "SYS_FORM_ID";

  static const String DATE_TIME_CONVERSION_FORMAT = "yyyy-MM-dd'T'HH:mm:ss";

  String trxId;
  int trxStatusId;
  String trxStatusDescription;
  int trxTypeId; // transaction typeId ex: MTNG 1-GSM, 2-MOMO, 3-MNP
  int instanceId;
  int companyId;
  MasterDataVersionBean formVersion;
  int registrationTypeId; // 1 = Individual, 2 = Corporate
  String username; // XXX could be mapped as a Long if it was a FK to
  // SSM_DISTRIBUTOR
  String securityToken;

  // For timing.
  DateTime startRegistrationTime; // When CMS form start open.
  DateTime endRegistrationTime; // When CMS form saved.
  DateTime sendRegistrationTime; // When CMS data sending to backend.
  DateTime receivedRegistrationTime; // When backend receive CMS Data.

  DateTime createdDateTime; // When save CMS data into DB.
  DateTime updatedDateTime; // When CMS has been updated in DB.

  // mPOS related fields
  MposInfoData mPosInfoData;

  // last transaction event attributes
  String actionDealer;
  DateTime actionDate;
  String actionName;
  String trxEventId;
  int trxEventTypeId;
  int actionTrxStatusId;

  // reasons for reject / block / unblock
  List<Reason> reasons;

  // to put the locale used by the logged in dealer
  String locale;

  // the map containing all the CMS fields filled in by the dealer
  StringObjectMap map;

  // the map containing all the CMS fields that went missing during the process
  StringMap missingReasons;

  // This is additional registration data for one subscribe.
  List<AddRegData> addRegDatas;

  // Transaction State in mPOS only
  IMposState state = BaseMposState.PENDING_MPOS;

  String stateInfo;

  // retried time that mPOS trying to send data to backEnd
  int retries = 0;
  String statusSummary;

  // id of transaction in mPOS Local SQLite Database
  int localId = 0;

  // Flag to indicate whiteList logic
  bool enableWhiteList = true;
  bool checkOccupied = false;
  int smsCount = 0;
  int respCode;
  String respDesc;

  CrmFormData() : this.map = StringObjectMap();

  CrmFormData.clone(CrmFormData crmFormData) {
    this.trxId = crmFormData.getTrxId();
    this.trxStatusId = crmFormData.getTrxStatusId();
    this.trxTypeId = crmFormData.getTrxTypeId();
    this.instanceId = crmFormData.getInstanceId();
    this.companyId = crmFormData.getCompanyId();
    this.registrationTypeId = crmFormData.getRegistrationTypeId();
    this.username = crmFormData.getUsername();
    this.securityToken = crmFormData.getSecurityToken();
    this.startRegistrationTime = crmFormData.getStartRegistrationTime();
    this.endRegistrationTime = crmFormData.getEndRegistrationTime();
    this.sendRegistrationTime = crmFormData.getSendRegistrationTime();
    this.receivedRegistrationTime = crmFormData.getReceivedRegistrationTime();
    this.createdDateTime = crmFormData.getCreatedDateTime();
    this.updatedDateTime = crmFormData.getUpdatedDateTime();
    this.mPosInfoData = crmFormData.getmPosInfoData();
    this.formVersion = crmFormData.getFormVersion();
    this.locale = crmFormData.getLocale();
    this.map = StringObjectMap.fromMap(crmFormData.getMap());
    this.checkOccupied = crmFormData.isCheckOccupied();
  }

  static CrmFormData copy(CrmFormData tx) {
    // create a CrmFormData and copy its values, except the map
    CrmFormData txCopy = CrmFormData.clone(tx);
    // copy additional remaining values
    txCopy.trxStatusDescription =
        tx.trxStatusDescription == null ? null : tx.trxStatusDescription;
    txCopy.actionDealer = tx.actionDealer == null ? null : tx.actionDealer;
    txCopy.actionDate = tx.actionDate == null ? null : tx.actionDate;
    txCopy.actionName = tx.actionName == null ? null : tx.actionName;
    txCopy.trxEventId = tx.trxEventId == null ? null : tx.trxEventId;
    txCopy.trxEventTypeId =
        tx.trxEventTypeId == null ? null : tx.trxEventTypeId;
    txCopy.actionTrxStatusId =
        tx.actionTrxStatusId == null ? null : tx.actionTrxStatusId;
    txCopy.reasons = Utils.isBlank(tx.reasons) ? null : List.from(tx.reasons);
    txCopy.missingReasons =
        Utils.isBlank(tx.missingReasons) ? null : tx.getMissingReasons();
    txCopy.addRegDatas =
        Utils.isBlank(tx.addRegDatas) ? null : List.from(tx.addRegDatas);
    txCopy.state = tx.state;
    txCopy.stateInfo = tx.stateInfo == null ? null : tx.stateInfo;
    txCopy.retries = tx.retries;
    txCopy.statusSummary = tx.statusSummary == null ? null : tx.statusSummary;
    txCopy.localId = tx.localId;
    txCopy.enableWhiteList = tx.enableWhiteList;
    txCopy.smsCount = tx.smsCount;
    txCopy.respCode = tx.respCode;
    txCopy.respDesc = tx.respDesc;
    return txCopy;
  }

  void putFieldsInMap() {
    map.put(TRX_STATUS_ID, trxStatusId);
    // Add another status for display
    map.put("status", trxStatusId.toString());
    map.put("trxStatusDescription", trxStatusDescription);
    map.put("actionDealer", actionDealer);
    map.put("actionDate", actionDate);
    map.put("actionName", actionName);
    // Add another actionName as Id, for reference in form to show or hide
    // reject reason and comment.
    map.put("actionNameId", actionName);
    // Set additional required data and put into the map. (For printing)
    map.put("username", username);
    map.put("respCode", respCode);
    map.put("respDesc", respDesc);
    if (createdDateTime != null) {
      map.put("createdDateTime",
          DateUtils.format_(DATE_TIME_CONVERSION_FORMAT, createdDateTime));
    }

    List<String> reasonLst = [];
    List<String> commentLst = [];
    if (reasons != null) {
      for (Reason r in reasons) {
        reasonLst.add(r.getCrmMsgArrayOptionId());
        commentLst.add(r.getComment());
      }
    }
    setRejectedReason(Utils.toString___(reasonLst));
    setRejectedComment(Utils.toString___(commentLst));
  }

  void extractFieldsFromMap() {
    // Add another status for display
    trxStatusId = map.getAsString("status") != null
        ? int.parse(map.getAsString("status"))
        : null;
    trxStatusDescription = map.getAsString("trxStatusDescription");
    actionDealer = map.getAsString("actionDealer");
    // actionDate = map.getAsString("actionDate");
    actionName = map.getAsString("actionName");
    // Set additional required data and put into the map. (For printing)
    username = map.getAsString("username");
    map.put("respCode", respCode);
    map.put("respDesc", respDesc);
    try {
      createdDateTime = DateUtils.getFormatter(DATE_TIME_CONVERSION_FORMAT)
          .parse(map.getAsString("createdDateTime"));
    } catch (e) {
      // Don't care.
    }
  }

  String getStateInfo() {
    return stateInfo;
  }

  void setStateInfo(String stateInfo) {
    this.stateInfo = stateInfo;
  }

  int getLocalId() {
    return localId;
  }

  void setLocalId(int localId) {
    this.localId = localId;
  }

  String getStatusSummary() {
    return statusSummary;
  }

  void setStatusSummary(String statusSummary) {
    this.statusSummary = statusSummary;
  }

  int getRetries() {
    return retries;
  }

  void setRetries(int retries) {
    this.retries = retries;
  }

  IMposState getState() {
    return state;
  }

  void setState(IMposState state) {
    this.state = state;
  }

  String getRejectedReason() {
    return map.getAsString(REJECTED_REASON);
  }

  void setRejectedReason(String rejectedRasons) {
    map.put(REJECTED_REASON, rejectedRasons);
  }

  String getRejectedComment() {
    return map.getAsString(REJECTED_COMMENT);
  }

  void setRejectedComment(String rejectedComment) {
    map.put(REJECTED_COMMENT, rejectedComment);
  }

  @override
  String getTrxId() {
    return trxId;
  }

  @override
  void setTrxId(String trxId) {
    this.trxId = trxId;
  }

  int getTrxStatusId() {
    return trxStatusId;
  }

  void setTrxStatusId(int trxStatusId) {
    this.trxStatusId = trxStatusId;
  }

  String getTrxStatusDescription() {
    return trxStatusDescription;
  }

  void setTrxStatusDescription(String trxStatusDescription) {
    this.trxStatusDescription = trxStatusDescription;
  }

  int getTrxTypeId() {
    return trxTypeId;
  }

  void setTrxTypeId(int trxTypeId) {
    this.trxTypeId = trxTypeId;
  }

  @override
  int getInstanceId() {
    return instanceId;
  }

  @override
  void setInstanceId(int instanceId) {
    this.instanceId = instanceId;
  }

  int getRegistrationTypeId() {
    return registrationTypeId;
  }

  void setRegistrationTypeId(int registrationTypeId) {
    this.registrationTypeId = registrationTypeId;
  }

  String getUsername() {
    return username;
  }

  void setUsername(String username) {
    this.username = username;
  }

  @override
  String getSecurityToken() {
    return securityToken;
  }

  void setSecurityToken(String securityToken) {
    this.securityToken = securityToken;
  }

  StringObjectMap getMap() {
    return map;
  }

  void setMap(StringObjectMap map) {
    this.map = map;
  }

  StringMap getMissingReasons() {
    return missingReasons;
  }

  void setMissingReasons(StringMap missingReasons) {
    this.missingReasons = missingReasons;
  }

  int getCompanyId() {
    return companyId;
  }

  void setCompanyId(int companyId) {
    this.companyId = companyId;
  }

  int getFormId() {
    if (formVersion != null) {
      return formVersion.getFormId();
    }

    if (map != null && map[SYS_FORM_ID] != null) {
      return int.parse(map.getAsString(SYS_FORM_ID));
    }

    return null;
  }

  void setFormId(int formId) {
    if (formVersion == null) {
      formVersion = MasterDataVersionBean();
    }

    formVersion.setFormId(formId);
  }

  MasterDataVersionBean getFormVersion() {
    return formVersion;
  }

  void setFormVersion(MasterDataVersionBean formVersion) {
    this.formVersion = formVersion;
  }

  DateTime getStartRegistrationTime() {
    return startRegistrationTime;
  }

  void setStartRegistrationTime(DateTime startRegistrationTime) {
    this.startRegistrationTime = startRegistrationTime;
  }

  DateTime getEndRegistrationTime() {
    return endRegistrationTime;
  }

  void setEndRegistrationTime(DateTime endRegistrationTime) {
    this.endRegistrationTime = endRegistrationTime;
  }

  DateTime getSendRegistrationTime() {
    return sendRegistrationTime;
  }

  void setSendRegistrationTime(DateTime sendRegistrationTime) {
    this.sendRegistrationTime = sendRegistrationTime;
  }

  DateTime getReceivedRegistrationTime() {
    return receivedRegistrationTime;
  }

  void setReceivedRegistrationTime(DateTime receivedRegistrationTime) {
    this.receivedRegistrationTime = receivedRegistrationTime;
  }

  @override
  DateTime getCreatedDateTime() {
    return createdDateTime;
  }

  @override
  void setCreatedDateTime(DateTime createdDateTime) {
    this.createdDateTime = createdDateTime;
  }

  DateTime getUpdatedDateTime() {
    return updatedDateTime;
  }

  void setUpdatedDateTime(DateTime updatedDateTime) {
    this.updatedDateTime = updatedDateTime;
  }

  String getActionDealer() {
    return actionDealer;
  }

  void setActionDealer(String actionDealer) {
    this.actionDealer = actionDealer;
  }

  DateTime getActionDate() {
    return actionDate;
  }

  void setActionDate(DateTime actionDate) {
    this.actionDate = actionDate;
  }

  String getActionName() {
    return actionName;
  }

  void setActionName(String actionName) {
    this.actionName = actionName;
  }

  String getTrxEventId() {
    return trxEventId;
  }

  void setTrxEventId(String trxEventId) {
    this.trxEventId = trxEventId;
  }

  int getActionTrxStatusId() {
    return actionTrxStatusId;
  }

  void setActionTrxStatusId(int actionTrxStatusId) {
    this.actionTrxStatusId = actionTrxStatusId;
  }

  int getTrxEventTypeId() {
    return trxEventTypeId;
  }

  void setTrxEventTypeId(int trxEventTypeId) {
    this.trxEventTypeId = trxEventTypeId;
  }

  List<Reason> getReasons() {
    return reasons;
  }

  void setReasons(List<Reason> reasons) {
    this.reasons = reasons;
  }

  @override
  String getLocale() {
    return locale;
  }

  void setLocale(String locale) {
    this.locale = locale;
  }

  MposInfoData getmPosInfoData() {
    return mPosInfoData;
  }

  void setmPosInfoData(MposInfoData mPosInfoData) {
    this.mPosInfoData = mPosInfoData;
  }

  List<AddRegData> getAddRegDatas() {
    return addRegDatas;
  }

  void setAddRegDatas(List<AddRegData> addRegDatas) {
    this.addRegDatas = addRegDatas;
  }

  int getSysId() {
    if (!map.containsKey(ID_KEY)) {
      throw IllegalArgumentException(
          "The SYS_ID has not been found in the map !");
    }
    if (map.getAsString(ID_KEY) != null) {
      return int.parse(map.getAsString(ID_KEY));
    }
    return null;
  }

  String getMsisdn() {
    return map.getAsString(MSISDN_KEY);
  }

  void setMsisdn(String msisdn) {
    map.put(MSISDN_KEY, msisdn);
  }

  String getIccid() {
    return map.getAsString(ICCID_KEY);
  }

  void setIccid(String iccid) {
    map.put(ICCID_KEY, iccid);
  }

  String getKitNumber() {
    return map.getAsString(KITNO_KEY);
  }

  void setKitNumber(String kitno) {
    map.put(KITNO_KEY, kitno);
  }

  String getPukNumber() {
    return map.getAsString(PUKNO_KEY);
  }

  void setPukNumber(String pukno) {
    map.put(PUKNO_KEY, pukno);
  }

  String getPreferredLanguage() {
    if (!map.containsKey(PREFERRED_LANGUAGE_KEY)) {
      // no need to throw an Exception in this case (the default value
      // will be in English...)
      return DEFAULT_LANGUAGE;
    }
    return map.getAsString(PREFERRED_LANGUAGE_KEY);
  }

  /// Check if the transaction is succeeded.
  ///
  /// @return True if the transaction is succeeded, or false otherwise.
  bool isSucceeded() {
    return state == BaseMposState.SUCCEEDED;
  }

  /// Check if the transaction is aborted.
  ///
  /// @return True if the transaction is aborted, or false otherwise.
  bool isAborted() {
    return state == BaseMposState.ABORTED;
  }

  /// Check if the transaction is peding_doc.
  ///
  /// @return True if the transaction is peding_doc, or false otherwise.
  bool isDocumentPending() {
    return state == BaseMposState.DOCUMENT_PENDING;
  }

  /// Check if the transaction is wrong_msisdn.
  ///
  /// @return True if the transaction is wrong_msisdn, or false otherwise.
  bool isWrongMsisdn() {
    return state == BaseMposState.WRONG_MSISDN;
  }

  bool isDuplicate() {
    return state == BaseMposState.DUPLICATE;
  }

  bool isRejected() {
    return state == BaseMposState.REJECTED;
  }

  @override
  int getIdCompany() {
    return companyId;
  }

  @override
  void setIdCompany(int idCompany) {
    companyId = idCompany;
  }

  int getRespCode() {
    return respCode;
  }

  @override
  void setRespCode(int respCode) {
    this.respCode = respCode;
  }

  String getRespDesc() {
    return respDesc;
  }

  @override
  void setRespDesc(String respDesc) {
    this.respDesc = respDesc;
  }

  bool isEnableWhiteList() {
    return enableWhiteList;
  }

  void setEnableWhiteList(bool enableWhiteList) {
    this.enableWhiteList = enableWhiteList;
  }

  bool isCheckOccupied() {
    return checkOccupied;
  }

  void setCheckOccupied(bool checkOccupied) {
    this.checkOccupied = checkOccupied;
  }

  int getSmsCount() {
    return smsCount;
  }

  void setSmsCount(int smsCount) {
    this.smsCount = smsCount;
  }

  @override
  int get hashCode {
    const int prime = 31;
    int result = 1;
    result = prime * result + (trxId == null ? 0 : trxId.hashCode);
    return result;
  }

  @override
  bool operator ==(Object obj) {
    if (identical(this, obj)) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (runtimeType != obj.runtimeType) {
      return false;
    }
    // ignore: test_types_in_equals
    CrmFormData other = obj as CrmFormData;
    if (trxId == null) {
      if (other.trxId != null) {
        return false;
      }
    } else if (trxId != other.trxId) {
      return false;
    }
    return true;
  }

  @override
  Set<String> getFields() => CrmFormDataMetaData.getFields();

  @override
  dynamic getFieldValue(String field) =>
      CrmFormDataMetaData.getFieldValue(field, this);

  @override
  void setFieldValue(String field, Object value) =>
      CrmFormDataMetaData.setFieldValue(field, value, this);
}
