/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../structure/ordered_map.dart';
import 'abstract_bean.dart';
import 'credential/common_credential_bean.dart';
import 'credential/form_detail_bean.dart';
import 'credential/registered_fp.dart';
import 'credential/version_bean.dart';
import 'enums/base_server_response_code.dart';
import 'enums/i_server_response_code.dart';
import 'register/response_message.dart';
import 'search/customer_information.dart';
import 'search/external/search_registration_information_resp.dart';
import 'server_response_metadata.dart';

class ServerResponse extends AbstractBean {
  int responseCode; // Response code for the request result
  String responseDesc; // Response description for the request result

  String statusInformation; // Status information
  String statusSummary; // statusSummary from ESB

  ResponseMessage customMessage;

  int abillityRespCode; // abillityRespCode for the request result
  String abillityRespMsg; // abillityRespMsg for the request result

  String securityToken; // Security token generated by the registration module

  int offlinePeriod; // For allow mpos login offline mode x hour config in isl_core

  String msisdn;
  String iccid;

  String promotionalMessageName;
  String promotionalMessageText;

  String eaiBackendUrl;
  String responseReasons; // reasons for reject or block
  String responseReasonComments; // reason comments for reject or block
  int loginLimit; // invalid login limit to block user

  // bean used to store the registration informations (search by web-service)
  // Support multiple-form.
  SearchRegistrationInformationResp searchRegistrationInformationResp;

  // Customer information.
  CustomerInformation customerInformation;

  // TODO Improve to support multiple-level modules.
  String permissions; // something like CREATE, UPDATE, etc.
  String printedRegistrationUrl; // when printing a registration

  // Dealer report
  // format = dd-MM-yyyy:nbTotalRegistrations#dd-MM-yyyy:nbTotalRegistrations#...
  String nbTotalRegistrations;

  // format = dd-MM-yyyy:nbSuccessRegistrations#dd-MM-yyyy:nbSuccessRegistrations#...
  String nbSuccessRegistrations;

  // format = dd-MM-yyyy:nbPendingRegistrations#dd-MM-yyyy:nbPendingRegistrations#...
  String nbPendingRegistrations;

  // format = dd-MM-yyyy:nbFailedRegistrations#dd-MM-yyyy:nbFailedRegistrations#...
  String nbFailedRegistrations;

  List<VersionBean> versionDetails;
  List<FormDetailBean> formDetails;

  // in order to allow user to update their hint in Change password page.
  String hintQuestion;
  String hintAnswer;

  // return when RESP_CODE_INVALID_PIN_CODE = 601
  int remainRetryLoginAttempt;

  // For or enable/disable flow.
  Map<String, bool> enableFlows;

  OrderedMap<String> arrayOptions; // for CMS MessageArray
  Map<String, Object> data; // for CMS RMI data handling

  // For Authentication service result such as login.
  CommonCredentialBean commonCredentialBean = CommonCredentialBean();

  List<RegisteredFP> fingerprints;

  // iSM specific
  String msisdnList;
  int reservationId;
  Map<String, String> resultMap;

  String smsActivationDestNumber;
  String smsActivationMsgFormat;
  String smsActivationSentCount;
  String smsActivationMaxOutbox;

  String otpPasswordChangeEnable;
  String license;

  String remainingStorageWarningValue;

  ServerResponse();

  ServerResponse.code(IServerResponseCode serverRespCode, String respDesc)
      : this.response(serverRespCode.getCodeInt(),
            respDesc != null ? respDesc : serverRespCode.getName());

  ServerResponse.response(this.responseCode, this.responseDesc);

  static ServerResponse success() {
    return ServerResponse.code(BaseServerResponseCode.SUCCESS, null);
  }

  static ServerResponse error() {
    return error_(null);
  }

  static ServerResponse error_(String description) {
    return ServerResponse.code(BaseServerResponseCode.INT_ERR, description);
  }

  static ServerResponse failure() {
    return ServerResponse.code(BaseServerResponseCode.FAILED, null);
  }

  static bool isSuccessful(ServerResponse response) {
    return response != null &&
        BaseServerResponseCode.fromCode(response.getResponseCode())
            .matchWith([BaseServerResponseCode.SUCCESS]);
  }

  static bool isAccepted(ServerResponse response) {
    return response != null &&
        response.getResponseCode() >= 500 &&
        response.getResponseCode() <= 599;
  }

  static bool isSecurityTokenExpired(ServerResponse response) {
    return response != null && response.getResponseCode() == 409;
  }

  int getResponseCode() {
    return responseCode;
  }

  void setResponseCode(int responseCode) {
    this.responseCode = responseCode;
  }

  String getResponseDesc() {
    return responseDesc;
  }

  void setResponseDesc(String responseDesc) {
    this.responseDesc = responseDesc;
  }

  int getAbillityRespCode() {
    return abillityRespCode;
  }

  void setAbillityRespCode(int abillityRespCode) {
    this.abillityRespCode = abillityRespCode;
  }

  String getAbillityRespMsg() {
    return abillityRespMsg;
  }

  void setAbillityRespMsg(String abillityRespMsg) {
    this.abillityRespMsg = abillityRespMsg;
  }

  String getSecurityToken() {
    return securityToken;
  }

  void setSecurityToken(String securityToken) {
    this.securityToken = securityToken;
  }

  int getOfflinePeriod() {
    return offlinePeriod;
  }

  void setOfflinePeriod(int offlinePeriod) {
    this.offlinePeriod = offlinePeriod;
  }

  String getStatusInformation() {
    return statusInformation;
  }

  void setStatusInformation(String statusInformation) {
    this.statusInformation = statusInformation;
  }

  String getEaiBackendUrl() {
    return eaiBackendUrl;
  }

  void setEaiBackendUrl(String eaiBackendUrl) {
    this.eaiBackendUrl = eaiBackendUrl;
  }

  String getStatusSummary() {
    return statusSummary;
  }

  void setStatusSummary(String statusSummary) {
    this.statusSummary = statusSummary;
  }

  String getMsisdn() {
    return msisdn;
  }

  void setMsisdn(String msisdn) {
    this.msisdn = msisdn;
  }

  String getIccid() {
    return iccid;
  }

  void setIccid(String iccid) {
    this.iccid = iccid;
  }

  String getPromotionalMessageName() {
    return promotionalMessageName;
  }

  void setPromotionalMessageName(String promotionalMessageName) {
    this.promotionalMessageName = promotionalMessageName;
  }

  String getPromotionalMessageText() {
    return promotionalMessageText;
  }

  void setPromotionalMessageText(String promotionalMessageText) {
    this.promotionalMessageText = promotionalMessageText;
  }

  String getResponseReasons() {
    return responseReasons;
  }

  void setResponseReasons(String responseReasons) {
    this.responseReasons = responseReasons;
  }

  String getResponseReasonComments() {
    return responseReasonComments;
  }

  void setResponseReasonComments(String responseReasonComments) {
    this.responseReasonComments = responseReasonComments;
  }

  int getLoginLimit() {
    return loginLimit;
  }

  void setLoginLimit(int loginLimit) {
    this.loginLimit = loginLimit;
  }

  SearchRegistrationInformationResp getSearchRegistrationInformationResp() {
    return searchRegistrationInformationResp;
  }

  void setSearchRegistrationInformationResp(
      SearchRegistrationInformationResp searchRegistrationInformationResp) {
    this.searchRegistrationInformationResp = searchRegistrationInformationResp;
  }

  String getPermissions() {
    return permissions;
  }

  void setPermissions(String permissions) {
    this.permissions = permissions;
  }

  String getPrintedRegistrationUrl() {
    return printedRegistrationUrl;
  }

  void setPrintedRegistrationUrl(String printedRegistrationUrl) {
    this.printedRegistrationUrl = printedRegistrationUrl;
  }

  String getNbTotalRegistrations() {
    return nbTotalRegistrations;
  }

  void setNbTotalRegistrations(String nbTotalRegistrations) {
    this.nbTotalRegistrations = nbTotalRegistrations;
  }

  String getNbSuccessRegistrations() {
    return nbSuccessRegistrations;
  }

  void setNbSuccessRegistrations(String nbSuccessRegistrations) {
    this.nbSuccessRegistrations = nbSuccessRegistrations;
  }

  String getNbPendingRegistrations() {
    return nbPendingRegistrations;
  }

  void setNbPendingRegistrations(String nbPendingRegistrations) {
    this.nbPendingRegistrations = nbPendingRegistrations;
  }

  String getNbFailedRegistrations() {
    return nbFailedRegistrations;
  }

  void setNbFailedRegistrations(String nbFailedRegistrations) {
    this.nbFailedRegistrations = nbFailedRegistrations;
  }

  String getHintQuestion() {
    return hintQuestion;
  }

  void setHintQuestion(String hintQuestion) {
    this.hintQuestion = hintQuestion;
  }

  String getHintAnswer() {
    return hintAnswer;
  }

  void setHintAnswer(String hintAnswer) {
    this.hintAnswer = hintAnswer;
  }

  List<FormDetailBean> getFormDetails() {
    return formDetails;
  }

  void setFormDetails(List<FormDetailBean> formDetails) {
    this.formDetails = formDetails;
  }

  List<VersionBean> getVersionDetails() {
    return versionDetails;
  }

  void setVersionDetails(List<VersionBean> versionDetails) {
    this.versionDetails = versionDetails;
  }

  CustomerInformation getCustomerInformation() {
    return customerInformation;
  }

  void setCustomerInformation(CustomerInformation customerInformation) {
    this.customerInformation = customerInformation;
  }

  Map<String, bool> getEnableFlows() {
    return enableFlows;
  }

  void setEnableFlows(Map<String, bool> enableFlows) {
    this.enableFlows = enableFlows;
  }

  OrderedMap<String> getArrayOptions() {
    return arrayOptions;
  }

  void setArrayOptions(OrderedMap<String> arrayOptions) {
    this.arrayOptions = arrayOptions;
  }

  Map<String, Object> getData() {
    return data;
  }

  void setData(Map<String, Object> data) {
    this.data = data;
  }

  int getRemainRetryLoginAttempt() {
    return remainRetryLoginAttempt;
  }

  void setRemainRetryLoginAttempt(int remainRetryLoginAttempt) {
    this.remainRetryLoginAttempt = remainRetryLoginAttempt;
  }

  CommonCredentialBean getCommonCredentialBean() {
    return commonCredentialBean;
  }

  void setCommonCredentialBean(CommonCredentialBean commonCredentialBean) {
    this.commonCredentialBean = commonCredentialBean;
  }

  List<RegisteredFP> getFingerprints() {
    return fingerprints;
  }

  void setFingerprints(List<RegisteredFP> fingerprints) {
    this.fingerprints = fingerprints;
  }

  int getReservationId() {
    return reservationId;
  }

  void setReservationId(int reservationId) {
    this.reservationId = reservationId;
  }

  String getMsisdnList() {
    return msisdnList;
  }

  void setMsisdnList(String msisdnList) {
    this.msisdnList = msisdnList;
  }

  ResponseMessage getCustomMessage() {
    return customMessage;
  }

  void setCustomMessage(ResponseMessage customMessage) {
    this.customMessage = customMessage;
  }

  Map<String, String> getResultMap() {
    return resultMap;
  }

  void setResultMap(Map<String, String> resultMap) {
    this.resultMap = resultMap;
  }

  String getSmsActivationDestNumber() {
    return smsActivationDestNumber;
  }

  void setSmsActivationDestNumber(String smsActivationDestNumber) {
    this.smsActivationDestNumber = smsActivationDestNumber;
  }

  String getSmsActivationMsgFormat() {
    return smsActivationMsgFormat;
  }

  void setSmsActivationMsgFormat(String smsActivationMsgFormat) {
    this.smsActivationMsgFormat = smsActivationMsgFormat;
  }

  String getSmsActivationSentCount() {
    return smsActivationSentCount;
  }

  void setSmsActivationSentCount(String smsActivationSentCount) {
    this.smsActivationSentCount = smsActivationSentCount;
  }

  String getSmsActivationMaxOutbox() {
    return smsActivationMaxOutbox;
  }

  void setSmsActivationMaxOutbox(String smsActivationMaxOutbox) {
    this.smsActivationMaxOutbox = smsActivationMaxOutbox;
  }

  String getOtpPasswordChangeEnable() {
    return otpPasswordChangeEnable;
  }

  void setOtpPasswordChangeEnable(String otpPasswordChangeEnable) {
    this.otpPasswordChangeEnable = otpPasswordChangeEnable;
  }

  String getLicense() {
    return license;
  }

  void setLicense(String license) {
    this.license = license;
  }

  String getRemainingStorageWarningValue() {
    return remainingStorageWarningValue;
  }

  void setRemainingStorageWarningValue(String remainingStorageWarningValue) {
    this.remainingStorageWarningValue = remainingStorageWarningValue;
  }

  @override
  Set<String> getFields() => ServerResponseMetaData.getFields();

  @override
  dynamic getFieldValue(String field) =>
      ServerResponseMetaData.getFieldValue(field, this);
}
