/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:collection' show LinkedHashMap;

import '../../formalyze/util/utils.dart';

// Case Insensitive String -> T Map
class OrderedMap<T> implements Map<String, T> {
  final Map<String, T> internalMap = LinkedHashMap();

  OrderedMap() {
    // nothing to do here
  }

  OrderedMap.fromMap(OrderedMap<T> map) {
    if (!Utils.isBlank(map)) {
      for (String key in map.keys) {
        put(key, map[key]);
      }
    }
  }

  @override
  T operator [](Object key) {
    final String sKey = Utils.castToString(key);
    return Utils.isBlank(sKey) ? null : internalMap[trimAndToCase(sKey)];
  }

  @override
  void operator []=(String key, T value) {
    put(key, value);
  }

  @override
  void addAll(Map<String, T> other) {
    internalMap.addAll(other);
  }

  @override
  void addEntries(Iterable<MapEntry<String, T>> newEntries) {
    internalMap.addEntries(newEntries);
  }

  @override
  Map<RK, RV> cast<RK, RV>() {
    return internalMap.cast();
  }

  @override
  void clear() {
    internalMap.clear();
  }

  @override
  bool containsKey(Object key) {
    final String sKey = Utils.castToString(key);
    return !Utils.isBlank(sKey) && internalMap.containsKey(trimAndToCase(sKey));
  }

  @override
  bool containsValue(Object value) {
    return internalMap.containsValue(value);
  }

  @override
  Iterable<MapEntry<String, T>> get entries => internalMap.entries;

  @override
  void forEach(void Function(String key, T value) action) {
    internalMap.forEach(action);
  }

  @override
  bool get isEmpty => internalMap.isEmpty;

  @override
  bool get isNotEmpty => internalMap.isNotEmpty;

  @override
  Iterable<String> get keys => internalMap.keys;

  @override
  int get length => internalMap.length;

  @override
  Map<K2, V2> map<K2, V2>(
      MapEntry<K2, V2> Function(String key, T value) convert) {
    return internalMap.map(convert);
  }

  @override
  T putIfAbsent(String key, T Function() ifAbsent) {
    return internalMap.putIfAbsent(key, ifAbsent);
  }

  @override
  T remove(Object key) {
    final String sKey = Utils.castToString(key);
    return Utils.isBlank(sKey) ? null : internalMap.remove(trimAndToCase(sKey));
  }

  @override
  void removeWhere(bool Function(String key, T value) test) {
    internalMap.removeWhere(test);
  }

  @override
  T update(String key, T Function(T value) update, {T Function() ifAbsent}) {
    return internalMap.update(key, update, ifAbsent: ifAbsent);
  }

  @override
  void updateAll(T Function(String key, T value) update) {
    internalMap.updateAll(update);
  }

  @override
  Iterable<T> get values => internalMap.values;

  void put(String key, T value) {
    if (!Utils.isBlank(key)) {
      internalMap[trimAndToCase(key)] = value;
    }
  }

  String trimAndToCase(String key) {
    return Utils.trimAndToLower(key);
  }

  String toString() => internalMap.toString();
}
