/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../formalyze/util/utils.dart';
import 'my_map.dart';

// Case Insensitive String -> T Map
class TMap<T> extends MyMap<String, T> {
  TMap() {
    // nothing to do here
  }

  TMap.fromMap(TMap<T> map) {
    if (!Utils.isBlank(map)) {
      for (String key in map.keys) {
        put(key, map[key]);
      }
    }
  }

  @override
  T operator [](Object key) {
    final String sKey = Utils.castToString(key);
    return Utils.isBlank(sKey) ? null : internalMap[trimAndToCase(sKey)];
  }

  @override
  void operator []=(String key, T value) {
    put(key, value);
  }

  @override
  bool containsKey(Object key) {
    final String sKey = Utils.castToString(key);
    return !Utils.isBlank(sKey) && super.containsKey(trimAndToCase(sKey));
  }

  @override
  T remove(Object key) {
    final String sKey = Utils.castToString(key);
    return Utils.isBlank(sKey) ? null : super.remove(trimAndToCase(sKey));
  }

  void put(String key, T value) {
    if (!Utils.isBlank(key)) {
      internalMap[trimAndToCase(key)] = value;
    }
  }

  String getAsString(String key) {
    Object value = internalMap[trimAndToCase(key)];
    String sValue;

    if (value is String) {
      sValue = value;
    } else if (value != null) {
      sValue = value.toString();
    }

    return sValue;
  }

  String trimAndToCase(String key) {
    return Utils.trimAndToLower(key);
  }
}
