/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../formalyze/util/bean_utils.dart';
import '../../formalyze/util/date_utils.dart';
import '../../formalyze/validation/converter/c_binary.dart';
import '../bean/i_reflect.dart';
import '../structure/string_object_map.dart';

abstract class ToStringUtils {
  const ToStringUtils._();

  /// Make toString with reflection technique for all fields including parent.
  ///
  /// @param excludeNullValueField if true, null value fields are excluded
  /// @return toString in format as below.
  ///
  ///         <pre>
  /// Person[name=John Doe,age=33,smoker=false]
  ///         </pre>
  static String toStringAllFields(IReflect object) {
    if (object == null) {
      return null;
    }

    StringBuffer stringBuffer =
        StringBuffer('${BeanUtils.getName(object, false)}[');

    for (String field in object.getFields()) {
      final dynamic value = object.getFieldValue(field);

      if (value != null) {
        dynamic toString;

        if (CBinary.checkType(value)) {
          toString = 'byte[${value.length}]';
        } else if (value is DateTime) {
          toString = DateUtils.formatDefault(value);
        } else if (value is List) {
          toString = listToString(value);
        } else if (value is Map) {
          toString = mapToString(value);
        } else if (value is IReflect) {
          toString = toStringAllFields(value);
        } else {
          toString = value;
        }

        stringBuffer.write('$field=$toString,');
      }
    }

    return bufferToString(stringBuffer, ']');
  }

  static String listToString(List list) {
    StringBuffer stringBuffer = StringBuffer('[');

    for (Object item in list) {
      stringBuffer.write('${toStringAllFields(item)},');
    }

    return bufferToString(stringBuffer, ']');
  }

  static String mapToString(Map map) {
    final bool isStringObjectMap = map is StringObjectMap;
    StringBuffer stringBuffer =
        StringBuffer("${isStringObjectMap ? 'StringObjectMap[' : '{'}");
    final String space = isStringObjectMap ? ' ' : '';

    for (String key in map.keys) {
      stringBuffer.write('$key$space=$space${map[key]},');
    }

    return bufferToString(stringBuffer, "${isStringObjectMap ? ']' : '\}'}");
  }

  static String bufferToString(StringBuffer stringBuffer, String endCh) =>
      '${stringBuffer.toString().substring(0, stringBuffer.length - 1)}$endCh';
}
