/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../enums/dir.dart';
import '../enums/orientation.dart';

class Defaults {
  const Defaults._();

  static const bool USE_I18N_NUMPAD = false;
  static const String NL = "\n";
  static const String EOL = "\r\n";

  static const String FORM_TITLE = "Form";

  // IDs for different components
  static const String ID_ROOT = "root";
  static const String ID_POPUP_ROOT = "popupRoot";
  static const String ID_POPUP = "popup";
  static const String ID_LAYOUT = "layout_";
  static const String ID_CLEAR = "btnClear_";
  static const String ID_GRAB = "btnGrab_";
  static const String ID_PREVIEW = "btnPreview_";
  static const String ID_UPLOAD = "btnUpload_";
  static const String ID_DOWNLOAD = "btnDownload_";

  // Random string literals
  static const String MT_NS = "http://www.mobile-technologies.com/cms";
  static const String UNKNOWN = "unknown";
  static const String NULL = "null";
  static const String FIELDS = "fields";
  static const String FIELD = "field";
  static const String ID = "id";
  static const String KEY = "key";
  static const String VALUE = "value";
  static const String LOCALE = "locale";
  static const String LAYOUT = "layout";
  static const String VIEW = "view";
  static const String NA = "N/A";
  static const String FORM_ID = "formId";
  static const String FORM_PK = "formPK";
  static const String MIN = "min";
  static const String MAX = "max";
  static const String WIDTH = "width";
  static const String HEIGHT = "height";
  static const String CHARS = "chars";
  static const String OPERATOR = "op";
  static const String SENSITIVE = "sensitive";
  static const String REGEX = "regex";
  static const String SHOW_RAW = "showRaw";
  static const String LANGUAGE = "language";
  static const String MODE = "mode";
  static const String OFF = "off";
  static const String ON = "on";
  static const String MESSAGES = "messages";
  static const String FORM_MODEL = "form.model";
  static const String FORM_UI = "form.ui";
  static const String FORM_I18N = "form.i18n";
  static const String UI = "ui";
  static const String MODEL = "model";
  static const String I18N = "i18n";
  static const String LEFT = "left";
  static const String RIGHT = "right";
  static const String OPTIONS = "options";
  static const String ACCEPT = "accept";
  static const String MASK = "mask";
  static const String EXT = "ext";
  static const String FILE = "file";

  // bundle messages used at several occasions
  static const String MSG_INVALID_DATA = "action.form.data.invalid";
  static const String MSG_REJECT_NO_REASON = "action.reject.no.reason";
  static const String MSG_BLANK_FILE = "fileupload.file.blank";
  static const String MSG_WRONG_FILE_TYPE = "fileupload.filetype.error";
  static const String MSG_IMAGE_UNREADABLE = "fileupload.image.unreadable";
  static const String MSG_TABLE_MAX_REACHED = "table.max.reached";

  static const String MSG_ALLOW_CAPTURE_OR_UPLOAD =
      "Either capture or upload should be allowed!";
  static const String MSG_MISSING_ARTIFACT =
      "Either Model or UI or I18n is not provided.";

  static const int MISSING = -1;
  static const int MISSING_L = -1;
  static const double MISSING_D = -1.0;

  static const bool INCLUDED = true;
  static const bool ENABLED = true;
  static const bool USE_MASKED_VALUE = false;
  static const bool EXTRA = false;
  static const bool BLANK_ITEM = true;
  static const bool CAMERA_SWITCH = false;
  static const bool FLASH = true;

  static const Orientation SUB_CONTAINER = Orientation.HORIZONTAL;
  static const Dir DIR = Dir.LTR;

  static final int INPUT_GROUP_COLUMNS = SUB_CONTAINER.columns;

  static const String YYYY_MM_DD = "yyyy-MM-dd";
  static const String DD_MM_YYYY = "dd/MM/yyyy";
  static const String DATE_FORMAT = "dd MMM yyyy";
  static const String TIME_FORMAT = "HH:mm";
  static const String TIME_FULL_FORMAT = "$TIME_FORMAT:ss";
  static const String DATE_TIME_FORMAT = "$DATE_FORMAT $TIME_FORMAT";

  // Standard DateTime Format used for conversion only.
  static const String DATE_CONVERSION_FORMAT = DATE_FORMAT;
  static const String TIME_CONVERSION_FORMAT = TIME_FORMAT;
  static const String DATE_TIME_CONVERSION_FORMAT =
      "EEE MMM dd $TIME_FULL_FORMAT yyyy";
  static const String DATE_TIME_SOAP_FORMAT = "$YYYY_MM_DD'T'$TIME_FULL_FORMAT";

  // file types
  static const String CSV = "csv";
  static const String XML = "xml";
  static const String PDF = "pdf";
  static const String XLSX = "xlsx";

  // MIME types
  static const String MIME_JPEG = "image/jpeg";
  static const String MIME_WSQ =
      "application/wsq"; // custom, doesn't really exist
  static const String MIME_CSV = "text/csv";
  static const String MIME_XML = "text/xml";
  static const String CHARSET = "charset=utf-8";
  static const String MIME_XML_UTF8 = "$MIME_XML;$CHARSET";
  static const String MIME_JSON = "application/json";
  static const String MIME_JSON_UTF8 = "$MIME_JSON;$CHARSET";
  static const String MIME_PDF = "application/pdf";
  static const String MIME_XLSX =
      "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
  static const String MIME_OCTET = "application/octet-stream";

  // File extensions
  static const String EXT_CAMERA = "jpg, jpeg, png, bmp";
  static const String EXT_FINGERPRINT = "wsq";
  static const String EXT_TABLE = CSV;

  static const String EXT_JPG = ".jpg";
  static const String EXT_PNG = ".png";
  static const String EXT_WEBP = ".webp";
  static const String EXT_WSQ = ".wsq";
  static const String EXT_TMP = ".tmp";

  // Icons common to components
  static const String ICON_CLEAR = "clear.png";
  static const String ICON_SHUTTER = "shutter.png";
  static const String ICON_SCAN = "scan.png";
  static const String ICON_UPLOAD = "upload.png";
  static const String ICON_IMPORT = "csv-import.png";
  static const String ICON_CLOSE = "close.png";

  // Icon packs common to components
  static const String ICONP_IMPRESSION = "impression";
  static const String ICONP_THUMBNAIL = "0xe7cf";
  static const String ICONP_CAMERA = "0xe62e";
  static const String ICONP_CLEAR = "0xebeb"; // Icons.clear_sharp

  // FontAwesome icon common to components
  static const String ICON_FA_CLEAR = "trash-o";
  static const String ICON_FA_UPLOAD = "upload";

  // for Android Camera
  static const int MAX_DIMENSION = 1024;
  static const int PHOTO_QUALITY = 90;
}
