/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../form/bean/server_response.dart';
import '../structure/ui/method/method.dart';
import '../ui/discrete/action/handler/rmi/rmi_cycle.dart';
import '../ui/discrete/form_controller.dart';
import '../ui/discrete/view_state.dart';

/// Contract for pre and post RMI invocation.
abstract class IRmi {
  /// Initially we should check if it's okay to proceed with the invocation.
  bool isReady(FormController controller);

  /// Before invoking the remote method, we prepare the the data that is configured in the XML. This method provides
  /// the flexibility to further do modifications in the created arguments. Such as translating message before sending
  /// printing.
  ///
  /// @param data
  ///        current CMS data.
  /// @param all
  ///        filter properties for change detection?
  /// @param method
  ///        method configuration element.
  /// @param controller
  ///        current controller.
  void prepare(Map<String, Object> data, bool all, Method method,
      FormController controller);

  /// Called after a remote method is called disregarding if it was a success or failure.
  ///
  /// @param success
  ///        determine this operation is success or failed from mapping.
  /// @param method
  ///        method configuration element.
  /// @param sRes
  ///        response data from calling.
  /// @param controller
  ///        current controller.
  /// @param action
  ///        Reference object to RMIAction.
  /// @return 'True' when need to render/enable attributes of configured views are toggled, otherwise return as 'False'
  ///         for skipping.
  bool doPost(bool success, Method method, ServerResponse sRes, RMICycle cycle,
      FormController controller);

  /// Called if any error during process
  ///
  /// @param e
  ///        exception raised by process
  /// @param controller
  ///        current controller
  void onError(
      Exception e, Method method, ViewState vSrc, FormController controller);
}
