/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../util/enum.dart';
import '../util/enum_utils.dart';
import '../util/utils.dart';

class Code extends Enum<Code> {
  static const SYS = Code("SYS_", "SYS");
  static const EXTRAS = Code("EXT_", "EXTRAS");

  static const LABEL = Code("lbl_", "LABEL");
  static const MSG = Code("msg_", "MSG");

  static const HTTP = Code("http", "HTTP");
  static const HTTPS = Code("https", "HTTPS");

  static const ALL = Code("*", "ALL");
  static const UNDERSCORE = Code("_", "UNDERSCORE");
  static const CUSTOM = Code("@", "CUSTOM");
  static const GLOBAL = Code("#", "GLOBAL");
  static const RMI = Code("~", "RMI");

  static const EFFECT = Code("@", "EFFECT");
  static const INCLUDE = Code("+", "INCLUDE");
  static const EXCLUDE = Code("-", "EXCLUDE");

  // mostly for ValueResolver
  static const RESOLVE = Code("?", "RESOLVE");
  static const PERMISSION = Code(".", "PERMISSION");
  static const EXTRA = Code("+", "EXTRA");
  static const CONFIG = Code("#", "CONFIG");
  static const FIELD_VALUE =
      Code("_", "FIELD_VALUE"); // gets the value configured in the field tag
  static const VIEW_VALUE =
      Code("@", "VIEW_VALUE"); // gets the value of an input-component
  static const VIEW = Code("^", "VIEW"); // gets any view in the form
  static const GET_VALUE = Code("%",
      "GET_VALUE"); // invokes a method in the external class to get the value
  static const SCRIPT =
      Code("\$", "SCRIPT"); // evaluates a script and returns the value
  static const SKIP_START = Code("[", "SKIP_START"); // Skips value resolution
  static const SKIP_END = Code("]", "SKIP_END");

  // for 'method' tag
  static const COMPLEX_TYPE = Code(".", "COMPLEX_TYPE");
  static const SKIP_PROPERTY = Code("-this", "SKIP_PROPERTY");

  static const NOT = Code("!", "NOT");

  final String code;

  const Code(this.code, String name) : super(name);

  String prefix(String str) {
    return Utils.prefix(str, code);
  }

  String remove(String str) {
    return starts(str) ? Utils.trim(str).substring(code.length).trim() : str;
  }

  bool starts(String str) {
    return Utils.startsWith_(str, [code]);
  }

  bool ends(String str) {
    return Utils.endsWith(str, [code]);
  }

  bool match(String str) {
    return Utils.match(code, [str]);
  }

  static Code toCode(String code, bool throwOnMismatch) {
    return EnumUtils.toEnum_(code, _ENUMS, throwOnMismatch);
  }

  static const List<Code> _ENUMS = [
    SYS,
    EXTRAS,
    LABEL,
    MSG,
    HTTP,
    HTTPS,
    ALL,
    UNDERSCORE,
    CUSTOM,
    GLOBAL,
    RMI,
    EFFECT,
    INCLUDE,
    EXCLUDE,
    RESOLVE,
    PERMISSION,
    EXTRA,
    CONFIG,
    FIELD_VALUE,
    VIEW_VALUE,
    VIEW,
    GET_VALUE,
    SCRIPT,
    SKIP_START,
    SKIP_END,
    COMPLEX_TYPE,
    SKIP_PROPERTY,
    NOT
  ];
}
