/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../util/enum.dart';
import '../util/enum_utils.dart';
import '../validation/constraint/pattern/c_pattern.dart';

// TODO Check for constraint type and field type compatibility?
class ConstraintType extends Enum<ConstraintType> {
  static final MANDATORY = ConstraintType("MANDATORY");
  static final GROUPMANDATORY = ConstraintType("GROUPMANDATORY");
  static final LENGTH = ConstraintType("LENGTH");
  static final RANGE = ConstraintType("RANGE");
  static final CHECKCASE = ConstraintType("CHECKCASE");
  static final EXCLUDE = ConstraintType("EXCLUDE");
  static final NOSPACE = ConstraintType("NOSPACE");

  static final LINK = ConstraintType.regex(
      "^(?:https?|ftp|file|ws|wss://)?[-" +
          CPattern.ALPHANUMERIC +
          "+&@#/%?=~_|!:,.;]*[-" +
          CPattern.ALPHANUMERIC +
          "+&@#/%=~_|]",
      "LINK");
  static final WORD = ConstraintType("WORD");

  static final DATE = ConstraintType("DATE");
  static final COMPARE = ConstraintType("COMPARE");
  static final COUNT = ConstraintType("COUNT");
  static final REPETITION = ConstraintType("REPETITION");
  static final DISTINCT = ConstraintType("DISTINCT");
  static final TEXT = ConstraintType("TEXT");

  static final FACE = ConstraintType.binary(true, "FACE");
  static final IMAGE = ConstraintType.binary(true, "IMAGE");
  static final SMARTSCAN = ConstraintType.binary(true, "SMARTSCAN");
  static final IMAGEMATCHER = ConstraintType.binary(true, "IMAGEMATCHER");
  static final FINGERPRINT = ConstraintType.binary(true, "FINGERPRINT");
  static final FILE = ConstraintType.binary(true, "FILE");

  static final DUPLICATE = ConstraintType("DUPLICATE");
  static final TABLEDUPLICATE = ConstraintType("TABLEDUPLICATE");

  static final SCRIPT = ConstraintType("SCRIPT");

  static final ALPHANUMERIC = ConstraintType("ALPHANUMERIC");
  static final PATTERN = ConstraintType("PATTERN");

  static final EMAIL = ConstraintType.regex(
      "^[_" +
          CPattern.ALPHANUMERIC +
          "-]+(\\.[_" +
          CPattern.ALPHANUMERIC +
          "-]+)*@[" +
          CPattern.ALPHANUMERIC +
          "-]+(\\.[" +
          CPattern.ALPHANUMERIC +
          "-]+)*(\\.[A-Za-z]{2,})\$",
      "EMAIL");
  static final HEXADECIMAL =
      ConstraintType.regex("[a-fA-F0-9\\s]*", "HEXADECIMAL");
  static final IPADDRESS = ConstraintType.regex(
      "\\b((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\\.|\$)){4}\\b", "IPADDRESS");
  static final PHONENUMBER = ConstraintType.regex("[0-9]*", "PHONENUMBER");

  static final USERNAME = ConstraintType.regex(
      "^[" +
          CPattern.ALPHANUMERIC +
          "_]+((\\.-?|-\\.?)[" +
          CPattern.ALPHANUMERIC +
          "_]+)*\$",
      "USERNAME");
  static final PASSWORD = ConstraintType.regex(
      "((?=.*\\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#\$%]).{6,20})", "PASSWORD");

  final bool _binary;
  final String _regex; // only for pattern related constraints
  final String _msgId;

  ConstraintType(String name) : this.binary(false, name);

  ConstraintType.binary(bool binary, String name)
      : this.full(binary, null, name);

  ConstraintType.regex(String regex, String name)
      : this.full(false, regex, name);

  ConstraintType.full(this._binary, this._regex, String name)
      : this._msgId = "validation.${name.toLowerCase()}.invalid",
        super(name);

  bool binary() {
    return this._binary;
  }

  String regex() {
    return this._regex;
  }

  String msgId() {
    return this._msgId;
  }

  @override
  String toString() => name().toLowerCase();

  static ConstraintType toType(String type, bool throwOnMismatch) {
    return EnumUtils.toEnum_(type, _ENUMS, throwOnMismatch);
  }

  static final List<ConstraintType> _ENUMS = [
    MANDATORY,
    GROUPMANDATORY,
    LENGTH,
    RANGE,
    CHECKCASE,
    EXCLUDE,
    NOSPACE,
    LINK,
    WORD,
    DATE,
    COMPARE,
    COUNT,
    REPETITION,
    DISTINCT,
    TEXT,
    FACE,
    IMAGE,
    SMARTSCAN,
    IMAGEMATCHER,
    FINGERPRINT,
    FILE,
    DUPLICATE,
    TABLEDUPLICATE,
    SCRIPT,
    ALPHANUMERIC,
    PATTERN,
    EMAIL,
    HEXADECIMAL,
    IPADDRESS,
    PHONENUMBER,
    USERNAME,
    PASSWORD
  ];
}
