/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../util/enum.dart';
import '../util/enum_utils.dart';
import "view_type.dart";

class FieldType extends Enum<FieldType> {
  static final BYTE = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "BYTE");
  static final SHORT = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "SHORT");
  static final NUMBER = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "NUMBER");
  static final LONG = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "LONG");
  static final FLOAT = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "FLOAT");
  static final DECIMAL = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "DECIMAL");

  static final CHARACTER = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "CHARACTER");
  static final STRING = FieldType({
    ViewType.LABEL,
    ViewType.INPUTTEXT,
    ViewType.INPUTAREA,
    ViewType.PASSWORD,
    ViewType.VIRTUAL,
    ViewType.BARCODE,
    ViewType.CARD,
    ViewType.SINGLECHOICE,
    ViewType.MULTIPLECHOICE,
    ViewType.SELECTONE,
    ViewType.SELECTMANY,
    ViewType.CAPTCHA
  }, "STRING");

  static final BOOLEAN = FieldType({
    ViewType.INPUTTEXT,
    ViewType.VIRTUAL,
    ViewType.TICKBOX,
    ViewType.SINGLECHOICE,
    ViewType.SELECTONE
  }, "BOOLEAN");

  static final DATE = FieldType(
      {ViewType.INPUTTEXT, ViewType.VIRTUAL, ViewType.DATEPICKER}, "DATE");
  static final TIME = FieldType(
      {ViewType.INPUTTEXT, ViewType.VIRTUAL, ViewType.TIMEPICKER}, "TIME");
  static final DATETIME = FieldType(
      {ViewType.INPUTTEXT, ViewType.VIRTUAL, ViewType.DATETIMEPICKER},
      "DATETIME");

  static final BINARY = FieldType({
    ViewType.VIRTUAL,
    ViewType.CAMERA,
    ViewType.FILEPICKER,
    ViewType.FINGERPRINT,
    ViewType.SIGNATURE,
    ViewType.OCR,
    ViewType.PASSPORT,
    ViewType.SCANNER
  }, "BINARY");

  static final LIST = FieldType({ViewType.VIRTUAL, ViewType.TABLE}, "LIST");

  final String _errMsg;
  final Set<ViewType> _supportedViews;

  FieldType(this._supportedViews, String name)
      : this._errMsg = "conversion.${name.toLowerCase()}.failure",
        super(name);

  String errMsg() {
    return this._errMsg;
  }

  bool supports(ViewType type) {
    return _supportedViews.contains(type);
  }

  static ViewType getDefaultView(bool show, FieldType fieldType) {
    return BINARY == fieldType
        ? ViewType.CAMERA
        : show
            ? ViewType.INPUTTEXT
            : ViewType.VIRTUAL;
  }

  static FieldType toType(String type, bool throwOnMismatch) {
    return EnumUtils.toEnum_(type, _ENUMS, throwOnMismatch);
  }

  static final List<FieldType> _ENUMS = [
    BYTE,
    SHORT,
    NUMBER,
    LONG,
    FLOAT,
    DECIMAL,
    CHARACTER,
    STRING,
    BOOLEAN,
    DATE,
    TIME,
    DATETIME,
    BINARY,
    LIST
  ];
}
