/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../util/enum.dart';
import '../util/enum_utils.dart';

class Mode extends Enum<Mode> {
  // To have the form for a fresh transaction.
  static const CREATE = Mode("CREATE");

  // To preview the form before saving it.
  static const PRE_SAVE_VIEW = Mode("PRE_SAVE_VIEW");

  // To start form initially in preview mode.
  static const PREVIEW = Mode("PREVIEW");

  // To have the form for persisted transaction with all fields disabled.
  static const POST_SAVE_VIEW = Mode("POST_SAVE_VIEW");

  // To have the form for persisted transaction with updatable fields enabled.
  static const UPDATE = Mode("UPDATE");

  // To have the form for persisted transaction with all fields disabled and a tickbox attached to each input
  // component so that the fields with invalid data could be rejected.
  static const REJECT = Mode("REJECT");

  // To have the form for persisted and rejected transaction with only the rejected fields enabled so that the user
  // could revise and re-enter the data which is acceptable.
  static const REVISE = Mode("REVISE");

  const Mode(String name) : super(name);

  static Mode toMode(String mode, bool throwOnMismatch) {
    return EnumUtils.toEnum_(mode, _ENUMS, throwOnMismatch);
  }

  static const List<Mode> _ENUMS = [
    CREATE,
    PRE_SAVE_VIEW,
    PREVIEW,
    POST_SAVE_VIEW,
    UPDATE,
    REJECT,
    REVISE
  ];
}
