/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../util/enum.dart';
import '../util/enum_utils.dart';
import '../util/utils.dart';

class ValueType extends Enum<ValueType> {
  // General values
  static const NULL = ValueType("NULL");
  static const PLATFORM = ValueType("PLATFORM");
  static const PIN = ValueType("PIN");
  static const IPv4 = ValueType("IPv4");
  static const IPv6 = ValueType("IPv6");
  static const MAC = ValueType("MAC");
  static const CAPTCHA = ValueType("CAPTCHA");

  // Date-time related
  static const TODAY = ValueType("TODAY"); // ignores time
  static const NOW = ValueType("NOW"); // date and time
  static const TIMESTAMP = ValueType("TIMESTAMP"); // long value in millis

  // Business related
  static const DEALER_CODE = ValueType("DEALER_CODE");
  static const TRX_ID = ValueType("TRX_ID");
  static const FORM_TRX_ID = ValueType("FORM_TRX_ID");

  // Form related
  static const MODE = ValueType("MODE");
  static const PRODUCTION_MODE = ValueType("PRODUCTION_MODE");
  static const BUILTIN_SCRIPTS = ValueType("BUILTIN_SCRIPTS");

  static const MODEL_ID = ValueType("MODEL_ID");
  static const MODEL_VERSION = ValueType("MODEL_VERSION");

  static const UI_ID = ValueType("UI_ID");
  static const UI_VERSION = ValueType("UI_VERSION");

  static const I18N_ID = ValueType("I18N_ID");
  static const I18N_VERSION = ValueType("I18N_VERSION");

  static const I18N_LOCALE = ValueType("I18N_LOCALE");

  // For data related
  static const FORM_IS_VALID = ValueType("FORM_IS_VALID");
  static const FORM_DATA = ValueType("FORM_DATA");

  // Web related

  // Android related
  static const ANDROID_VERSION = ValueType("ANDROID_VERSION");
  static const ANDROID_RELEASE = ValueType("ANDROID_RELEASE");
  static const MPOS_INFO_DATA = ValueType("MPOS_INFO_DATA");

  static const APP_NAME = ValueType("APP_NAME");
  static const APP_VERSION = ValueType("APP_VERSION");

  static const LATITUDE = ValueType("LATITUDE");
  static const LONGITUDE = ValueType("LONGITUDE");

  static const IMEI = ValueType("IMEI");
  static const IMSI = ValueType("IMSI");
  static const ICCID = ValueType("ICCID");

  static const CELL_ID_RAW = ValueType("CELL_ID_RAW");
  static const CELL_ID = ValueType("CELL_ID");
  static const CELL_LAC = ValueType("CELL_LAC");
  static const CELL_PSC = ValueType("CELL_PSC");
  static const CELL_PROCESSOR = ValueType("CELL_PROCESSOR");
  static const CELL_SERIAL = ValueType("CELL_SERIAL");
  static const CELL_BRAND = ValueType("CELL_BRAND");
  static const CELL_MODEL = ValueType("CELL_MODEL");
  static const CELL_MANUFACTURER = ValueType("CELL_MANUFACTURER");
  static const CELL_MODEL_WITH_MANUFACTURER =
      ValueType("CELL_MODEL_WITH_MANUFACTURER");
  static const CELL_NUMBER = ValueType("CELL_NUMBER");
  static const CELL_MCC = ValueType("CELL_MCC");
  static const CELL_MNC = ValueType("CELL_MNC");

  static const CAMERA_CAN_AUTO_FOCUS = ValueType("CAMERA_CAN_AUTO_FOCUS");
  static const CAMERA_RESOLUTIONS = ValueType("CAMERA_RESOLUTIONS");

  static const NETWORK_AVAILABLE = ValueType("NETWORK_AVAILABLE");
  static const NETWORK_TYPE = ValueType("NETWORK_TYPE");

  static const TEMP_PATH = ValueType("TEMP_PATH");

  static const DEVICE_SMALLEST_DP = ValueType("DEVICE_SMALLEST_DP");

  // Mock data
  static const MOCK_PIC = ValueType("MOCK_PIC");
  static const MOCK_FACE = ValueType("MOCK_FACE");
  static const MOCK_DOC = ValueType("MOCK_DOC");
  static const MOCK_SIGN = ValueType("MOCK_SIGN");
  static const MOCK_FINGER = ValueType("MOCK_FINGER");

  const ValueType(String name) : super(name);

  bool isMock() {
    return Utils.startsWith_(runtimeType.toString(), ["mock"]);
  }

  static ValueType toType(String type, bool throwOnMismatch) {
    return EnumUtils.toEnum_(type, _ENUMS, throwOnMismatch);
  }

  static const List<ValueType> _ENUMS = [
    NULL,
    PLATFORM,
    PIN,
    IPv4,
    IPv6,
    MAC,
    CAPTCHA,
    TODAY,
    NOW,
    TIMESTAMP,
    DEALER_CODE,
    TRX_ID,
    FORM_TRX_ID,
    MODE,
    PRODUCTION_MODE,
    BUILTIN_SCRIPTS,
    MODEL_ID,
    MODEL_VERSION,
    UI_ID,
    UI_VERSION,
    I18N_ID,
    I18N_VERSION,
    I18N_LOCALE,
    FORM_IS_VALID,
    FORM_DATA,
    ANDROID_VERSION,
    ANDROID_RELEASE,
    MPOS_INFO_DATA,
    APP_NAME,
    APP_VERSION,
    LATITUDE,
    LONGITUDE,
    IMEI,
    IMSI,
    ICCID,
    CELL_ID_RAW,
    CELL_ID,
    CELL_LAC,
    CELL_PSC,
    CELL_PROCESSOR,
    CELL_SERIAL,
    CELL_BRAND,
    CELL_MODEL,
    CELL_MANUFACTURER,
    CELL_MODEL_WITH_MANUFACTURER,
    CELL_NUMBER,
    CELL_MCC,
    CELL_MNC,
    CAMERA_CAN_AUTO_FOCUS,
    CAMERA_RESOLUTIONS,
    NETWORK_AVAILABLE,
    NETWORK_TYPE,
    TEMP_PATH,
    DEVICE_SMALLEST_DP,
    MOCK_PIC,
    MOCK_FACE,
    MOCK_DOC,
    MOCK_SIGN,
    MOCK_FINGER
  ];
}
