/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../util/enum.dart';
import '../util/enum_utils.dart';
import '../util/utils.dart';
import 'field_type.dart';

class ViewType extends Enum<ViewType> {
  // List of all UI containers
  static const LINEARLAYOUT = ViewType(true, "lnr", "LINEARLAYOUT");
  static const GRIDLAYOUT = ViewType(true, "grid", "GRIDLAYOUT");

  static const TABPANEL = ViewType(true, "tbPnl", "TABPANEL");
  static const TAB = ViewType(true, "tab", "TAB");

  static const POPUP = ViewType(true, "pop", "POPUP");

  // List of all input UI components
  static const LABEL =
      ViewType.full(true, false, false, false, Fields.STRING, "lbl", "LABEL");

  static const INPUTTEXT =
      ViewType.input(false, false, Fields.STRING, "txt", "INPUTTEXT");
  static const INPUTAREA =
      ViewType.input(false, false, Fields.STRING, "ia", "INPUTAREA");
  static const PASSWORD =
      ViewType.input(false, false, Fields.STRING, "pwd", "PASSWORD");
  static const VIRTUAL =
      ViewType.input(false, false, Fields.STRING, "vir", "VIRTUAL");

  static const BARCODE =
      ViewType.input(false, false, Fields.STRING, "bar", "BARCODE");
  static const CARD =
      ViewType.input(false, false, Fields.STRING, "card", "CARD");

  static const TICKBOX =
      ViewType.input(false, true, Fields.BOOLEAN, "tick", "TICKBOX");

  static const SINGLECHOICE =
      ViewType.input(true, true, Fields.STRING, "sinCh", "SINGLECHOICE");
  static const MULTIPLECHOICE =
      ViewType.input(true, false, Fields.STRING, "mulCh", "MULTIPLECHOICE");

  static const SELECTONE =
      ViewType.input(true, true, Fields.STRING, "selOne", "SELECTONE");
  static const SELECTMANY =
      ViewType.input(true, false, Fields.STRING, "selMan", "SELECTMANY");

  static const CAPTCHA =
      ViewType.input(false, false, Fields.STRING, "cap", "CAPTCHA");

  static const DATEPICKER =
      ViewType.input(false, false, Fields.DATE, "dtPkr", "DATEPICKER");
  static const TIMEPICKER =
      ViewType.input(false, false, Fields.TIME, "tmPkr", "TIMEPICKER");
  static const DATETIMEPICKER = ViewType.input(
      false, false, Fields.DATETIME, "dtTmPkr", "DATETIMEPICKER");

  static const FILEPICKER =
      ViewType.input(false, false, Fields.BINARY, "flPkr", "FILEPICKER");
  static const CAMERA =
      ViewType.input(false, false, Fields.BINARY, "cam", "CAMERA");
  static const FINGERPRINT =
      ViewType.input(false, false, Fields.BINARY, "fp", "FINGERPRINT");
  static const SIGNATURE =
      ViewType.input(false, false, Fields.BINARY, "sig", "SIGNATURE");
  static const OCR = ViewType.input(false, false, Fields.BINARY, "ocr", "OCR");
  static const PASSPORT =
      ViewType.input(false, false, Fields.BINARY, "pass", "PASSPORT");
  static const SCANNER =
      ViewType.input(false, false, Fields.BINARY, "scan", "SCANNER");

  static const TABLE =
      ViewType.full(true, true, false, false, Fields.LIST, "tbl", "TABLE");
  static const COLUMN = ViewType(false, "col", "COLUMN");
  static const IDCOLUMN = ViewType(false, "idCol", "IDCOLUMN");
  static const TICKCOLUMN = ViewType(false, "tkCol", "TICKCOLUMN");
  static const RADIOCOLUMN = ViewType(false, "radCol", "RADIOCOLUMN");
  static const IMAGECOLUMN = ViewType(false, "imgCol", "IMAGECOLUMN");
  static const FPCOLUMN = ViewType(false, "fpCol", "FPCOLUMN");
  static const ACTIONCOLUMN = ViewType(false, "actCol", "ACTIONCOLUMN");

  // List of all other UI components
  static const SPACER = ViewType(false, "spc", "SPACER");
  static const IMAGE = ViewType(false, "img", "IMAGE");
  static const DOCVIEWER = ViewType(false, "docVwr", "DOCVIEWER");

  static const LINK = ViewType(false, "lnk", "LINK");
  static const BUTTON = ViewType(false, "btn", "BUTTON");
  static const SUBMIT = ViewType(false, "sub", "SUBMIT");

  final bool _canInput;
  final bool _isContainer;
  final bool _canArray;
  final bool _canDepend;
  final String _fieldType;
  final String _shortName;

  const ViewType(bool isContainer, String shortName, String name)
      : this.full(false, isContainer, false, false, null, shortName, name);

  const ViewType.input(bool canArray, bool canDepend, String fieldType,
      String shortName, String name)
      : this.full(true, false, canArray, canDepend, fieldType, shortName, name);

  const ViewType.full(this._canInput, this._isContainer, this._canArray,
      this._canDepend, this._fieldType, this._shortName, String name)
      : super(name);

  bool canInput() {
    return this._canInput;
  }

  bool isContainer() {
    return this._isContainer;
  }

  bool canArray() {
    return this._canArray;
  }

  bool canDepend() {
    return this._canDepend;
  }

  bool isBinary() {
    return canInput() && FieldType.BINARY == fieldType();
  }

  FieldType fieldType() {
    return FieldType.toType(_fieldType, false);
  }

  String shortName() {
    return this._shortName;
  }

  bool isColumn() {
    return Utils.endsWith(toString(), ["column"]);
  }

  static ViewType toViewType(String viewType, bool throwOnMismatch) {
    return EnumUtils.toEnum_(viewType, _ENUMS, throwOnMismatch);
  }

  static const List<ViewType> _ENUMS = [
    LINEARLAYOUT,
    GRIDLAYOUT,
    TABPANEL,
    TAB,
    POPUP,
    LABEL,
    INPUTTEXT,
    INPUTAREA,
    PASSWORD,
    VIRTUAL,
    BARCODE,
    CARD,
    TICKBOX,
    SINGLECHOICE,
    MULTIPLECHOICE,
    SELECTONE,
    CAPTCHA,
    DATEPICKER,
    TIMEPICKER,
    DATETIMEPICKER,
    FILEPICKER,
    CAMERA,
    FINGERPRINT,
    SIGNATURE,
    OCR,
    PASSPORT,
    SCANNER,
    TABLE,
    COLUMN,
    IDCOLUMN,
    TICKCOLUMN,
    RADIOCOLUMN,
    IMAGECOLUMN,
    FPCOLUMN,
    ACTIONCOLUMN,
    SPACER,
    IMAGE,
    DOCVIEWER,
    LINK,
    BUTTON,
    SUBMIT
  ];
}

class Fields {
  static const String STRING = "STRING";
  static const String BOOLEAN = "BOOLEAN";
  static const String BINARY = "BINARY";
  static const String DATE = "DATE";
  static const String TIME = "TIME";
  static const String DATETIME = "DATETIME";
  static const String LIST = "LIST";
}
