/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../form/bean/master_data_version_bean.dart';
import '../../form/structure/t_map.dart';
import '../config/config.dart';
import '../config/defaults.dart';
import '../structure/root_tag.dart';
import '../util/log/logger.dart';
import '../util/utils.dart';
import 'form.dart';

class FormCache {
  // only successfully parsed and rendered forms are cached
  static final TMap<Form> FORMS = Config.CACHE_FORM ? TMap() : null;

  const FormCache._();

  static bool exists(dynamic id) {
    final String sId = id is String ? id : id.toString();
    return Config.CACHE_FORM && !isMissing(sId) && FORMS.containsKey(sId);
  }

  static Form get(int id) {
    return get_(id.toString());
  }

  static Form get_(String id) {
    Form form = Config.CACHE_FORM && !isMissing(id) ? FORMS[id] : null;

    if (form != null) {
      Logger.info("Found cached '$form' against '$id'");
    }

    return form;
  }

  static void set(int id, Form form) {
    set_(id.toString(), form);
  }

  static void set_(String id, Form form) {
    if (Config.CACHE_FORM && !isMissing(id) && !exists(id) && form != null) {
      FORMS.put(id, form);
      Logger.info("Successfully cached '$form' against '$id'");
    }
  }

  static void remove(int id) {
    remove_(id.toString());
  }

  static void remove_(String id) {
    if (Config.CACHE_FORM) {
      FORMS.remove(id);
    }
  }

  static void clear() {
    Utils.clear([FORMS]);
  }

  static bool isStale(int id, MasterDataVersionBean ver) {
    Form form = get(id);
    return form == null || isStale_(form, ver);
  }

  static bool isStale_(Form form, MasterDataVersionBean ver) {
    // For this logic to work, we must ensure that the version specified in each of the form artifact is in
    // harmony with the versions persisted in DB.
    return ver != null &&
        (isStale__(ver.getModelFormVersion(), form.getModel()) ||
            isStale__(ver.getWfUiFormVersion(), form.getUi()) ||
            isStale__(ver.getI18nFormVersion(), form.getI18n()));
  }

  static bool isStale__(int version, RootTag artifact) {
    return Utils.toDefault(version, Defaults.MISSING_L) > artifact.getVersion();
  }

  static bool isMissing(String id) {
    return Utils.isBlank(id) || Defaults.MISSING.toString() == id;
  }
}
