/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:convert' show JsonEncoder, JsonDecoder;
import 'dart:typed_data' show Uint8List;

import '../../../form/bean/i_reflect.dart';
import '../../config/defaults.dart';
import '../../error/illegal_argument_exception.dart';
import '../../structure/i18n/i18n.dart';
import '../../structure/model/model.dart';
import '../../structure/root_tag.dart';
import '../../structure/ui/ui.dart';
import '../../util/bean/card/card_data.dart';
import '../../util/log/logger.dart';
import '../../util/utils.dart';
import '../form.dart';
import '../raw_form.dart';

class JsonReader {
  static const String EXT = ".json";
  static const String BLANK_JSON = "{}";

  const JsonReader();

  dynamic decode(String json) {
    return Utils.isBlank(json) ? null : JsonDecoder().convert(json);
  }

  T fromJson<T>(String json, Type clazz) {
    if (clazz == Model) {
      return Model.json(decode(json)) as T;
    } else if (clazz == UI) {
      return UI.json(decode(json)) as T;
    } else if (clazz == I18n) {
      return I18n.json(decode(json)) as T;
    } else if (clazz == CardData) {
      return CardData.json(decode(json)) as T;
    }

    // TODO Support if clazz is StringObject or Map

    throw IllegalArgumentException("$clazz isn't supported!");
  }

  T fromJson_<T>(dynamic json, Type clazz) {
    return null; // FIXME
  }

  String toJson(Object object) {
    return object == null
        ? BLANK_JSON
        : JsonEncoder().convert(object is IReflect ? object.toMap() : object);
  }

  bool isBlank(String json) {
    return Utils.isBlank(json) ||
        Utils.match(json, [Defaults.NULL, BLANK_JSON, "[]"]);
  }

  List<List<String>> toTableList(String json) {
    final Iterable elements = decode(json);
    return Utils.isBlank(elements) ? null : List.from(elements);
  }

  Form readDecoded(Uint8List model, Uint8List ui, Uint8List i18n) {
    return read(RawForm.getDecodedForm(model, ui, i18n));
  }

  Form read(RawForm form) {
    Model model = fromJson(form.getModel(), Model);
    UI ui = fromJson(form.getUi(), UI);
    I18n i18n = fromJson(form.getI18n(), I18n);

    return Form(model, ui, i18n);
  }

  Form read_(Stream sModel, Stream sUi, Stream sI18n) {
    // FIXME
    try {
      // Reader rModel = InputStreamReader(sModel, Utils.UTF_8);
      // Reader rUi = InputStreamReader(sUi, Utils.UTF_8);
      // Reader rI18n = InputStreamReader(sI18n, Utils.UTF_8);
      return null;
      // return Form(fromJson(rModel, Model),
      //     fromJson(rUi, UI), fromJson(rI18n, I18n));
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace); // should never happen
    }

    throw IllegalArgumentException(Defaults.MSG_MISSING_ARTIFACT);
  }

  Model readModel(String json) {
    return readRootTag(json, Model);
  }

  UI readUI(String json) {
    return readRootTag(json, UI);
  }

  I18n readI18n(String json) {
    return readRootTag(json, I18n);
  }

  T readRootTag<T extends RootTag>(String json, Type clazz) {
    return toMap(fromJson(json, clazz));
  }

  T toMap<T extends RootTag>(T bean) {
    if (bean != null) {
      bean.toMap();
    }

    return bean;
  }
}
