/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../util/utils.dart';
import '../../validation/converter/c_date.dart';

class XmlReader {
  static String toOneLiner(String xml) {
    return Utils.isBlank(xml)
        ? ''
        : xml.replaceAll('\n', '').replaceAll(RegExp('>[ ]*<'), '><');
  }

  static Map<String, dynamic> toMap(String xml) {
    Map<String, dynamic> map = {};
    int currentIndex = 0, startBracketIndex, closeBracketIndex;

    while ((startBracketIndex = xml.indexOf('<', currentIndex)) > -1) {
      closeBracketIndex = xml.indexOf('>', startBracketIndex) + 1;
      List<String> tag =
          getTag(xml.substring(startBracketIndex, closeBracketIndex));

      if (tag == null) {
        // if tag is to be ignored, move forward
        currentIndex = closeBracketIndex;
      } else {
        final String namespace = tag[0];
        final String tagName = tag[1];
        final String closingTagName =
            namespace == '' ? tagName : '$namespace:$tagName';

        // TODO Should we use here lastIndexOf? same tag names can occur
        int endTagIndex = xml.indexOf('</$closingTagName>', closeBracketIndex);
        String innerXml = xml.substring(closeBracketIndex, endTagIndex);
        Map<String, dynamic> innerMap = toMap(innerXml);

        if (innerMap.isEmpty) {
          map[tagName] = innerXml.trim(); // innerXml is the tag value
        } else {
          // handle list of elements by appending size of the map
          String entryName =
              map.containsKey(tagName) ? '$tagName${map.length}' : tagName;
          map[entryName] = innerMap;
        }

        currentIndex = endTagIndex + closingTagName.length + 3; // 3 for </>
      }
    }

    return map;
  }

  /// 0th index is the namespace, 1st is the tag name
  static List<String> getTag(String tag) {
    // tag is self-enclosing, to be ignored
    if (tag.startsWith('<!--') || tag.startsWith('<?') || tag.endsWith('/>')) {
      return null;
    }

    int spaceIndex = tag.indexOf(' '); // expect space if attributes are present
    String name =
        tag.substring(1, spaceIndex == -1 ? tag.length - 1 : spaceIndex).trim();
    return name.contains(':') ? name.split(':') : ['', name];
  }

  static DateTime toDate(String date) => CDate.parseWithMany(date, [
        Defaults.DATE_TIME_SOAP_FORMAT,
        Defaults.DATE_TIME_CONVERSION_FORMAT,
        Defaults.YYYY_MM_DD,
        Defaults.DD_MM_YYYY,
        Defaults.DATE_FORMAT
      ]);
}
