/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../form/structure/string_object_map.dart';
import '../ui/discrete/action/data/value_resolver.dart';
import '../ui/discrete/renderer/features.dart';
import '../ui/discrete/renderer/ui_renderer.dart';
import '../util/form_utils.dart';
import '../util/structure/unique_no_blank_string_map.dart';
import 'model/constraint.dart';
import 'required_id_tag.dart';

class ArgsTag extends RequiredIdTag {
  // @ElementMap(entry = "arg", key = Defaults.ID, attribute = true, inline = true, required = false,
  //         keyType = String, valueType = String)
  UniqueNoBlankStringMap args;

  ArgsTag() {
    // default constructor required for de/serialization
  }

  ArgsTag.full(String id, UniqueNoBlankStringMap args) : super.id(id) {
    this.args = args;
  }

  ArgsTag.json(Map json) : super.json(json) {
    if (json.containsKey('args')) {
      this.args = UniqueNoBlankStringMap();

      for (String key in (json['args'] as Map).keys) {
        this.args[key] = json['args'][key];
      }
    }
  }

  void copyAttributes(Constraint constraint) {
    UniqueNoBlankStringMap args =
        constraint.getArgs(); // don't use 'args' directly

    for (String key in args.keys) {
      getArgs().put(key, args[key]); // don't use 'args' directly
    }
  }

  StringObjectMap getArgs_(UIRenderer renderer) {
    return getArgs__(Features.getResolver(renderer));
  }

  StringObjectMap getArgs__(ValueResolver resolver) {
    UniqueNoBlankStringMap config = getArgs();
    StringObjectMap args = StringObjectMap();

    for (String key in config.keys) {
      args.put(key, resolver.resolve(config[key]));
    }

    return args;
  }

  Object getArg___(String arg, UIRenderer renderer) {
    return FormUtils.resolve_(getArgs()[arg], renderer);
  }

  UniqueNoBlankStringMap getArgs() {
    if (args == null) {
      args = UniqueNoBlankStringMap();
    }

    return args;
  }
}
