/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../ui/discrete/renderer/ui_renderer.dart';
import '../../util/script/script_evaluator.dart';
import '../../util/structure/unique_string_map.dart';
import '../../util/utils.dart';
import '../../validation/converter/c_boolean.dart';
import '../../validation/converter/c_string.dart';
import '../required_id_tag.dart';
import 'locale.dart';

// @Root
class Message extends RequiredIdTag {
  // @Attribute(required = false)
  String refMsgId;

  // @Attribute(required = false)
  bool script;

  // @ElementMap(entry = "text", key = Defaults.LOCALE, attribute = true, inline = true, keyType = String,
  //         valueType = String, required = false)
  UniqueStringMap value; // blank values are allowed

  Message() {
    // default constructor required for de/serialization
  }

  Message.json(Map json) : super.json(json) {
    this.refMsgId = json['refMsgId'];
    this.script = json['script'];

    if (json.containsKey('value')) {
      this.value = UniqueStringMap();

      for (String key in (json['value'] as Map).keys) {
        this.value[key] = json['value'][key];
      }
    }
  }

  String get(Locale locale, UIRenderer renderer) {
    final String localeId = locale.getId();

    if (!Utils.got(localeId, value)) {
      return null;
    }

    final String text = value[localeId];
    return Utils.toBlank(!Utils.isBlank(text) && isScript()
        ? CString.convert(ScriptEvaluator.eval(text, null, renderer))
        : text);
  }

  String getRefMsgId() {
    return refMsgId;
  }

  bool isScript() {
    return Utils.toDefault(script, CBoolean.NO);
  }

  UniqueStringMap getValue() {
    return value;
  }

  @override
  String toString() => "Message [id=${getId()}, value=$value]";
}
