/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../ui/discrete/renderer/ui_renderer.dart';
import '../../util/script/condition_evaluator.dart';
import '../../util/utils.dart';
import '../../validation/converter/c_boolean.dart';
import '../required_id_tag.dart';

class Option extends RequiredIdTag {
  // @Attribute(required = false)
  String msg;

  // @Attribute(required = false)
  String inclusion;

  Option() {
    // default constructor required for de/serialization
  }

  Option.id(String id, String msg) : super.id(id) {
    this.msg = msg;
  }

  Option.json(Map json) : super.json(json) {
    this.msg = json['msg'];
    this.inclusion = json['inclusion'];
  }

  @override
  bool allowSpace() {
    return true; // for cases like country, nationality etc.
  }

  String getMsg() {
    // When we convert XML to JSON, if the value is null it is ignored by JSON parser which we don't want.
    return Utils.toBlank(msg);
  }

  // if message id is blank for an option, we use the key itself.
  String getMsgId() {
    return Utils.toDefault(getMsg(), getId());
  }

  String getInclusionIf() {
    return CBoolean.parseNoErr(inclusion) == null ? inclusion : null;
  }

  bool isIncluded(UIRenderer renderer) {
    return ConditionEvaluator.eval_(inclusion, Defaults.INCLUDED, renderer);
  }

  @override
  String toString() => "[Option=${getId()}, msg=${getMsg()}]";
}
