/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../enums/code.dart';
import '../error/illegal_argument_exception.dart';
import '../ui/discrete/component/behavior/i_map.dart';
import '../util/bean_utils.dart';
import '../util/utils.dart';

abstract class IdTag implements IMap {
  IdTag() {
    // nothing to do here
  }

  void validate(bool allowSpace) {
    final String id = resolveId();

    if (Code.UNDERSCORE.starts(id)) {
      throw IllegalArgumentException(
          "${BeanUtils.getName(this, false)} '$id' starts with underscore.");
    }

    if (!allowSpace && Utils.hasSpace(id)) {
      throw IllegalArgumentException(
          "${BeanUtils.getName(this, false)} '$id' contains space(s).");
    }
  }

  @override
  void toMap() {
    // nothing to do here
  }

  @override
  void toList() {
    // nothing to do here
  }

  String getId();

  String resolveId() {
    return getId();
  }

  // Should we allow space in the tag's id? Normally, we don't.
  bool allowSpace() {
    return false;
  }

  @override
  int get hashCode {
    final String id = resolveId();
    return 31 + (Utils.isBlank(id) ? 0 : id.hashCode);
  }

  @override
  bool operator ==(obj) {
    if (identical(this, obj)) {
      return true;
    }

    if (obj == null || this.runtimeType != obj.runtimeType) {
      return false;
    }

    IdTag other = obj;

    final String id = resolveId();
    final String otherId = other.resolveId();

    if (id == null) {
      if (otherId != null) {
        return false;
      }
    } else if (!Utils.equalsIgnoreCase(id, otherId)) {
      return false;
    }

    return true;
  }
}
