/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../form/structure/t_map.dart';
import '../../../error/value_required_exception.dart';
import '../../../util/map_utils.dart';
import '../../../util/utils.dart';
import '../../required_id_tag.dart';
import 'property.dart';

class ComplexType extends RequiredIdTag {
  // @Attribute(required = true)
  String bean;

  // @ElementList(required = true, inline = true)
  List<Property> properties;

  // @Transient
  TMap<Property> propertyMap; // case-insensitive properties

  ComplexType() {
    // default constructor required for de/serialization
  }

  ComplexType.json(Map json) : super.json(json) {
    this.bean = json['bean'];

    if (Utils.isBlank(this.bean)) {
      throw ValueRequiredException("ComplexType", "bean");
    }

    if (json.containsKey('properties')) {
      this.properties = [];

      for (Map map in json['properties']) {
        this.properties.add(Property.json(map));
      }
    } else {
      throw ValueRequiredException("ComplexType", "properties");
    }
  }

  @override
  void toMap() {
    this.propertyMap = MapUtils.toMap(properties, false);
  }

  @override
  void toList() {
    this.properties = MapUtils.toList(propertyMap, true);
  }

  String getBean() {
    return bean;
  }

  TMap<Property> getProperties() {
    return propertyMap;
  }

  @override
  String toString() => "ComplexType [id=${getId()}, bean=$bean]";
}
