/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../form/structure/t_map.dart';
import '../../../error/value_required_exception.dart';
import '../../../ui/discrete/component/behavior/i_map.dart';
import '../../../util/map_utils.dart';
import '../../../util/utils.dart';
import 'complex_type.dart';
import 'property.dart';

class Data implements IMap {
  // @Attribute(required = false)
  String bean; // required for CMI only.

  // @ElementList(required = true, inline = true)
  List<Property> properties;

  // @Transient
  TMap<Property> propertyMap; // case-insensitive properties

  // @ElementList(required = false)
  List<ComplexType> complexTypes;

  // @Transient
  TMap<ComplexType> complexTypeMap;

  Data() {
    // default constructor required for de/serialization
  }

  Data.json(Map json) {
    this.bean = json['bean'];

    if (json.containsKey('properties')) {
      this.properties = [];

      for (Map map in json['properties']) {
        this.properties.add(Property.json(map));
      }
    } else {
      throw ValueRequiredException("Data", "properties");
    }

    if (json.containsKey('complexTypes')) {
      this.complexTypes = [];

      for (Map map in json['complexTypes']) {
        this.complexTypes.add(ComplexType.json(map));
      }
    }
  }

  @override
  void toMap() {
    this.propertyMap = MapUtils.toMap(properties, false);
    this.complexTypeMap = MapUtils.toMap(complexTypes, false);
  }

  @override
  void toList() {
    this.properties = MapUtils.toList(propertyMap, true);
    this.complexTypes = MapUtils.toList(complexTypeMap, true);
  }

  String getBean() {
    return bean;
  }

  TMap<Property> getProperties() {
    return propertyMap;
  }

  ComplexType getComplexType(String id) {
    return Utils.get_(id, complexTypeMap);
  }

  @override
  String toString() => "Data [bean=$bean]";
}
