/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../enums/code.dart';
import '../../../enums/value_type.dart';
import '../../../error/value_required_exception.dart';
import '../../../util/utils.dart';
import '../../required_id_tag.dart';

class Property extends RequiredIdTag {
  // @Attribute(required = true)
  String value;

  // @Attribute(required = false)
  bool
      everChanging; // decides if this property will be involved in method's change detection

  // @Attribute(required = false)
  bool
      mandatory; // decides if this property should never be null during data preparation

  Property() {
    // default constructor required for de/serialization
  }

  Property.json(Map json) : super.json(json) {
    this.value = json['value'];

    if (Utils.isBlank(this.value)) {
      throw ValueRequiredException("Property", "value");
    }

    this.everChanging = json['everChanging'];
    this.mandatory = json['mandatory'];
  }

  String getValue() {
    return value;
  }

  bool isEverChanging() {
    return Utils.toFalse(everChanging);
  }

  bool isMandatory() {
    if (mandatory == null && Code.RESOLVE.starts(value)) {
      return ValueType.FORM_DATA ==
          ValueType.toType(Utils.skipFirst(value.trim()), false);
    }

    return Utils.toFalse(mandatory);
  }

  @override
  String toString() => "Property [id=${getId()}, value=$value]";
}
