/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../error/illegal_argument_exception.dart';
import '../../../error/value_required_exception.dart';
import '../../../util/map_utils.dart';
import 'failure.dart';
import 'success.dart';

class Response {
  // @Element(required = true)
  Success success;

  // @Element(required = true)
  Failure failure;

  Response() {
    // default constructor required for de/serialization
  }

  Response.json(Map json) {
    this.success =
        json.containsKey('success') ? Success.json(json['success']) : null;

    if (this.success == null) {
      throw ValueRequiredException("Response", "success");
    }

    this.failure =
        json.containsKey('failure') ? Failure.json(json['failure']) : null;

    if (this.failure == null) {
      throw ValueRequiredException("Response", "failure");
    }
  }

  void validate(String methodId) {
    try {
      MapUtils.checkDuplicates(success.getCodes(), failure.getCodes());
    } catch (e) {
      throw IllegalArgumentException(
          "$e is success and failure for method $methodId");
    }
  }

  Success getSuccess() {
    return success;
  }

  Failure getFailure() {
    return failure;
  }
}
