/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../error/value_required_exception.dart';
import '../../../util/structure/unique_string_map.dart';
import '../../../util/utils.dart';

class ResponseTag {
  // @Attribute(required = false)
  String msg; // default message id

  // @Attribute(required = false)
  String popup; // alert if popup is undefined

  // @ElementMap(entry = "code", key = Defaults.ID, attribute = true, inline = true, keyType = String,
  //         valueType = String, required = true)
  UniqueStringMap codes; // blank values are allowed

  // @Transient
  bool codesExpanded = false;

  ResponseTag() {
    // default constructor required for de/serialization
  }

  ResponseTag.json(Map json) {
    this.msg = json['msg'];
    this.popup = json['popup'];

    if (json.containsKey('codes')) {
      this.codes = UniqueStringMap();

      for (String key in (json['codes'] as Map).keys) {
        this.codes[key] = json['codes'][key];
      }
    } else {
      throw ValueRequiredException("ResponseTag", "codes");
    }
  }

  String getMsg() {
    return msg;
  }

  String getPopup() {
    return popup;
  }

  UniqueStringMap getCodes() {
    if (!codesExpanded) {
      UniqueStringMap expanded = UniqueStringMap();

      for (String code in codes.keys) {
        final String msgId = codes[code];
        List<String> csv =
            Utils.splitByCommaAsList(code); // comma separated codes

        for (String id in csv) {
          expanded.put(id, msgId);
        }
      }

      codes = expanded;
      codesExpanded = true;
    }

    return codes;
  }
}
