/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/string_object_map.dart';
import '../../contract/contract_utils.dart';
import '../../contract/i_page_handler.dart';
import '../../error/value_required_exception.dart';
import '../../ui/discrete/action/data/value_resolver.dart';
import '../../util/form_utils.dart';
import '../../util/utils.dart';
import '../args_tag.dart';

class Page extends ArgsTag {
  static const String ARG_RESULT_FOR = "_resultFor";

  // @Attribute(required = true)
  String name;

  // @Attribute(required = false)
  bool kill;

  // @Attribute(required = false)
  String resultFor; // resulting value is for which view?

  // @Attribute(required = false)
  String handler; // IPageHandler interface

  Page() {
    // default constructor required for de/serialization
  }

  Page.json(Map json) : super.json(json) {
    this.name = json['name'];

    if (Utils.isBlank(this.name)) {
      throw ValueRequiredException("Page", "name");
    }

    this.kill = json['kill'];
    this.resultFor = json['resultFor'];
    this.handler = json['handler'];
  }

  String getName() {
    return name;
  }

  bool shouldKill() {
    return Utils.toFalse(kill);
  }

  String getResultFor() {
    return resultFor;
  }

  IPageHandler getHandler() {
    return ContractUtils.getHandler(handler);
  }

  @override
  StringObjectMap getArgs__(ValueResolver resolver) {
    if (!Utils.isBlank(resultFor)) {
      FormUtils.getView__(resultFor, true, true,
          resolver.getRenderer()); // Checks if the view exists and can input
    }

    StringObjectMap args = super.getArgs__(resolver);
    args.put(ARG_RESULT_FOR, resultFor);
    return args;
  }

  @override
  String toString() =>
      "Page [id=${getId()}, name=$name, resultFor=$resultFor, args=$args]";
}
