/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/orientation.dart';
import '../../error/value_required_exception.dart';
import '../../util/utils.dart';
import '../required_id_tag.dart';
import 'view.dart';

class Popup extends RequiredIdTag {
  // @Attribute(required = false)
  String title;

  // @Attribute(required = false)
  Orientation orientation;

  // @Attribute(required = false)
  String focus;

  // @Attribute(required = false)
  String width;

  // @Attribute(required = false)
  String height;

  // @Attribute(required = false)
  String style;

  // @Attribute(required = false)
  bool closeable;

  // @ElementList(required = true)
  List<View> layout;

  Popup() {
    // default constructor required for de/serialization
  }

  Popup.json(Map json) : super.json(json) {
    this.title = json['title'];
    this.orientation = Orientation.toOrientation(json['orientation'], true);
    this.focus = json['focus'];
    this.width = json['width'];
    this.height = json['height'];
    this.style = json['style'];
    this.closeable = json['closeable'];

    if (json.containsKey('layout')) {
      this.layout = [];

      for (Map map in json['layout']) {
        this.layout.add(View.json(map));
      }
    } else {
      throw ValueRequiredException("Popup", "layout");
    }
  }

  String getTitle() {
    return Utils.isBlank(title) ? Utils.suffix(getId(), ".title") : title;
  }

  Orientation getOrientation() {
    return orientation;
  }

  String getFocus() {
    return focus;
  }

  String getWidth() {
    return width;
  }

  String getHeight() {
    return height;
  }

  String getStyle() {
    return style;
  }

  bool isCloseable() {
    return Utils.toTrue(closeable);
  }

  List<View> getLayout() {
    return layout;
  }

  @override
  String toString() =>
      "Popup [id=${getId()}, title=${getTitle()}, orientation=$orientation]";
}
