/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import '../../../../form/bean/server_response.dart';
import '../../../config/defaults.dart';
import '../../../util/cipher/cryptor.dart';
import '../../../util/io_utils.dart';
import '../../../util/network.dart';
import '../../../util/utils.dart';
import 'handler/rmi/soap_rmi.dart';

abstract class BRMI extends SoapRMI {
  final String license;

  BRMI(String endpoint, String certificate, String certificateKey, int timeout,
      this.license)
      : super(endpoint, certificate, certificateKey, getTimeout(timeout));

  @override
  Future<ServerResponse> invoke(String service, String method, String namespace,
      int timeout, String clazz, Map<String, Object> args) async {
    Cryptor.pack_(args, license);
    return await super.invoke(service, method, namespace, timeout, clazz, args);
  }

  @override
  Future<bool> isNetworkAvailable() async {
    return await Network.isReachable(endpoint);
  }

  Future<Uint8List> getCertificate(String certificate) async {
    if (Utils.isBlank(certificate) ||
        Utils.equalsIgnoreCase(Defaults.NULL, certificate)) {
      return null;
    }

    return IOUtils.readFile(certificate);
  }

  static int getTimeout(int timeout) {
    return timeout == null ? 120000 : timeout; // 2 minutes
  }
}
