/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../enums/action.dart';
import '../../../structure/model/field.dart';
import '../../../util/utils.dart';
import '../component/base/component.dart';
import '../component/behavior/event/i_change_value.dart';
import '../component/behavior/event/i_click.dart';
import '../renderer/ui_renderer.dart';
import '../view_state.dart';
import 'handler/action_handler.dart';

class EventHandler {
  final UIRenderer renderer;

  const EventHandler(this.renderer);

  void addOnChangeValue(ViewState view) {
    Component component = view.getComponent();

    if (component is IChangeValue) {
      Action action;

      if (!Utils.isBlank(view.getRender()) ||
          !Utils.isBlank(view.getDependents())) {
        action = Action.RENDER;
      } else if (checkForClearAction(view)) {
        action = Action.CLEAR;
      } else {
        Field field = view.getField();

        if (field != null) {
          if (!Utils.isBlank(view.getParentOf()) ||
              !Utils.isBlank(field.getReplicate(renderer)) ||
              field.getLuhn() != null ||
              field.hasImmediateConstraints_(renderer)) {
            action = Action.NOTHING;
          }
        }
      }

      final String sAction =
          action == null ? view.getOnValueChange() : action.toString();

      if (!Utils.isBlank(sAction)) {
        (component as IChangeValue).onValueChange_(
            view, ActionHandler.getInstance(sAction, false, renderer));
      }
    }
  }

  void addOnClick(ViewState view) {
    Component component = view.getComponent();

    if (component is IClick) {
      String action = view.getOnClick();

      if (Utils.isBlank(action)) {
        if (!Utils.isBlank(view.getPopupId())) {
          action = Action.SHOWPOPUP.name();
        } else if (!Utils.isBlank(view.getValidate())) {
          action = Action.VALIDATE.name();
        } else if (!Utils.isBlank(view.getRender())) {
          action = Action.RENDER.name();
        } else if (!Utils.isBlank(view.getEnable())) {
          action = Action.ENABLE.name();
        } else if (checkForClearAction(view)) {
          action = Action.CLEAR.name();
        }

        view.setOnClick(action);
      }

      if (!Utils.isBlank(action)) {
        (component as IClick)
            .onClick(view, ActionHandler.getInstance(action, false, renderer));
      }
    }
  }

  bool checkForClearAction(ViewState view) {
    return !Utils.isBlank(view.getClear()) ||
        !Utils.isBlank(view.getSet()) ||
        !Utils.isBlank(view.getReset());
  }
}
