/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../../contract/i_rmi.dart';
import '../../../../../structure/ui/method/method.dart';
import '../../../../../util/utils.dart';
import '../../../form_controller.dart';
import '../../../renderer/features.dart';
import '../../../renderer/ui_renderer.dart';
import '../../../view_state.dart';

abstract class BaseIRMI implements IRmi {
  static const String UNAVAILABLE = "rmi.unavailable";
  static const String DB_UNAVAILABLE = "database.unavailable";

  @override
  bool isReady(FormController controller) {
    return true;
  }

  @override
  void prepare(Map<String, Object> data, bool all, Method method,
      FormController controller) {
    // do nothing as default
  }

  @override
  void onError(
      Exception e, Method method, ViewState vSrc, FormController controller) {
    UIRenderer renderer = controller.getRenderer();
    Features.getAlert_(renderer)
        .error(renderer.get(isDbErr(e) ? DB_UNAVAILABLE : UNAVAILABLE));
  }

  static bool isDbErr(Exception e) {
    return Utils.contains__(
        e, ["PersistenceException", "JDBCException", "SQLException"]);
  }
}
