/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:io' show HttpClientRequest, HttpClientResponse;
import 'dart:typed_data' show Uint8List;

import '../../../../../config/config.dart';
import '../../../../../enums/code.dart';
import '../../../../../util/log/logger.dart';
import 'mpos_http_transport_se.dart';
import 'mpos_https_transport_se.dart';

abstract class Transport {
  final Uri uri;
  final int timeout;

  bool debug = false;

  // ignore: close_sinks
  HttpClientRequest conn;

  Transport(String url, this.timeout) : this.uri = Uri.parse(url);

  Future<String> call(String envelope);

  Future<String> call_(String envelope, Map<String, String> headers);

  Future<HttpClientResponse> call__(
      String envelope, Map<String, String> headers);

  Future<HttpClientRequest> getServiceConnection();

  static Transport init(bool secured, String url, int timeout,
      Uint8List certificate, String certificateKey,
      {String method}) {
    Logger.info("Connecting via ${(secured ? Code.HTTPS : Code.HTTP).code}...");
    Transport transport = secured
        ? MposHttpsTransportSE(url, timeout, certificate, certificateKey,
            method: method)
        : MposHttpTransportSE(url, timeout, method: method);
    transport.debug = Config.DEBUG;
    return transport;
  }
}
