/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../structure/model/field.dart';
import '../../../util/form_utils.dart';
import '../../../util/stop_watch.dart';
import '../../../util/utils.dart';
import '../component/base/component.dart';
import '../data/data_handler.dart';
import '../renderer/component_factory.dart';
import '../renderer/features.dart';
import '../renderer/ui_renderer.dart';
import '../view_state.dart';
import 'action_base.dart';
import 'handler/action_handler.dart';
import 'toggle_action.dart';

class RelativeRenderAction extends ActionBase {
  const RelativeRenderAction(ViewState view, ActionHandler handler)
      : super(view, handler);

  void render(Object value) {
    if (Utils.anyNull([view, handler])) {
      return;
    }

    StopWatch stopWatch = StopWatch.start();
    Set<ViewState> dependents = view.getDependents();

    if (!Utils.isBlank(dependents)) {
      UIRenderer renderer = getRenderer();
      List<ViewState> inOrder = DataHandler.arrangeInInputOrder_(dependents);
      ComponentFactory factory = Features.getFactory_(renderer);

      for (ViewState dependent in inOrder) {
        // On value change, we should refresh the dependent views
        refresh(dependent, dependent.getRenderedIf(), true, renderer);
        refresh(dependent, dependent.getEnabledIf(), false, renderer);

        Component component = dependent.getComponent();

        if (FormUtils.hasScript(dependent.getLabel())) {
          factory.setLabel(dependent, component);
        }

        if (FormUtils.hasScript(dependent.getTitle())) {
          factory.setTitle(dependent, component);
        }

        if (FormUtils.hasScript(dependent.getSubTitle())) {
          factory.setSubTitle(dependent, component);
        }

        // On value change, we should refresh the dependent views's constraint affecting the UI
        Field field = dependent.getField();

        if (field != null && !Utils.isBlank(field.getConstraints())) {
          factory.reflectConstraints(dependent);
        }
      }
    }

    // re-render component(s) listed in 'render' attribute
    // TODO render is currently supported only for TickBox
    if (value is bool) {
      ToggleAction().toggleRender(view, value, handler);
    }

    StopWatch.stop_(value, stopWatch);
  }

  // re-render/re-enable component(s) that are in hierarchy
  static void refresh(ViewState dependent, String scriptId, bool rendered,
      UIRenderer renderer) {
    if (dependent != null) {
      dependent.refresh(scriptId, rendered, renderer);

      ViewState vParent = FormUtils.getView_(dependent.getParentOf(), renderer);
      refresh(vParent, scriptId, rendered, renderer);
    }
  }
}
