/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../../../form/bean/mpos/base_mpos_constant.dart';
import '../../../../../config/defaults.dart';
import '../../../../../util/base64.dart';
import '../../../../../util/bean_utils.dart';
import '../../../../../util/date_utils.dart';
import '../../../../../util/enum_utils.dart';
import '../../../../../util/utils.dart';
import '../../../../../validation/converter/c_binary.dart';
import '../../../../../validation/converter/c_date.dart';
import 'soap_annotations.dart';

class SoapBuilder {
  const SoapBuilder();

  /// Takes all given arguments and convert it to SOAP and add it's property in the SoapObject.
  ///
  /// Should not use case-insensitive key map here.
  String build(String method, String namespace, Map<String, Object> data) {
    StringBuffer soap = StringBuffer('<n1:$method xmlns:n1="$namespace">');

    if (!Utils.isBlank(data)) {
      for (String property in data.keys) {
        addProperty(property, property, data[property], true, soap);
      }
    }

    soap.write('</n1:$method>');
    return soap.toString();
  }

  StringBuffer build_(Object bean) {
    final String tagName = Utils.firstToLower(BeanUtils.getName(bean, true));
    StringBuffer root = StringBuffer('<$tagName>');
    Set<String> fields = BeanUtils.getFields(bean);
    SoapAnnotations annotations = SoapAnnotations();

    for (String field in fields) {
      if (!annotations.isTransient(field, bean)) {
        // we don't want transient class member in the SOAP
        String propertyName = annotations.getPropertyName(field, bean);
        String entryName = annotations.getEntryName(field, bean);
        Object value = BeanUtils.getFieldValue(field, bean);

        addProperty(propertyName, entryName, value, true, root);
      }
    }

    root.write('</$tagName>');
    return root;
  }

  String createList(String property, String entryName, List list) {
    StringBuffer root = StringBuffer();

    for (Object item in list) {
      if (!Utils.isBlank(item)) {
        root.write(BeanUtils.isComplexType(item)
            ? build_(item)
            : '<$entryName>$item</$entryName>');
      }
    }

    return root.toString();
  }

  StringBuffer createMap(String entryName, Map map) {
    StringBuffer root = StringBuffer('<$entryName>');
    final String entryTagName = BaseMposConstant.PROPERTY_ENTRY;

    for (Object key in map.keys) {
      final Object value = map[key];

      if (!Utils.isBlank(value)) {
        StringBuffer entryElement = StringBuffer('<$entryTagName>');
        addProperty(BaseMposConstant.PROPERTY_ENTRY_KEY, entryName, key, false,
            entryElement);
        addProperty(BaseMposConstant.PROPERTY_ENTRY_VALUE, entryName, value,
            false, entryElement);
        entryElement.write('</$entryTagName>');
        root.write(entryElement);
      }
    }

    root.write('</$entryName>');
    return root;
  }

  // TODO Do we need separate Date Format for dates inside Map entries?
  void addProperty(String property, String entryName, Object value,
      bool soapDateFormat, StringBuffer soap) {
    if (!Utils.isBlank(value)) {
      if (CBinary.isBinary(value)) {
        value = Base64.encode_(value);
      } else if (EnumUtils.isEnum(value)) {
        value = build_(value);
      } else if (CDate.checkType(value)) {
        value = DateUtils.format_(
            soapDateFormat
                ? Defaults.DATE_TIME_SOAP_FORMAT
                : Defaults.DATE_TIME_CONVERSION_FORMAT,
            value as DateTime);
      } else if (BeanUtils.isMap(value)) {
        value = createMap(entryName, value as Map);
      } else if (BeanUtils.isList(value)) {
        value = createList(property, entryName, value as List);
      } else if (BeanUtils.isComplexType(value)) {
        value = build_(value);
      }

      soap.write(
          value is StringBuffer ? value : '<$property>$value</$property>');
    }
  }
}
