/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../../../form/bean/server_response.dart';
import '../../../../../util/utils.dart';
import '../../handler/rmi/soap_rmi.dart';
import '../error/remote_service_exception.dart';
import '../error/server_unavailable_exception.dart';
import 'soap_deserializer.dart';

abstract class SoapClient extends SoapRMI {
  final String name;
  final String namespace;
  final String securityToken;
  final int instanceId;
  final bool offlineLogin;

  SoapClient(
      this.name,
      this.namespace,
      String endpoint,
      String certificate,
      String certificateKey,
      int timeout,
      this.securityToken,
      this.instanceId,
      this.offlineLogin)
      : super(endpoint, certificate, certificateKey, timeout);

  Future<ServerResponse> invoke__(
      String method, Map<String, Object> data) async {
    // there is no point of moving forward if end point couldn't be reached.
    if (!await isNetworkAvailable()) {
      throw ServerUnavailableException(getMsg("label_internal_error"));
    }

    String response = await callWS(createSoap(method, data), prefixEndpoint(),
        SoapRMI.getSoapAction(namespace, method), defaultTimeout, data);

    if (response == null) {
      throw RemoteServiceException(getMsg("label_remote_network_unstable"));
    }

    return SoapDeserializer.deserialize(response);
  }

  String createSoap(String method, Map<String, Object> data) {
    StringBuffer soap =
        StringBuffer(soapBuilder.build(method, namespace, data));

    if (!Utils.isBlank(securityToken)) {
      soap.write(
          '<${SoapRMI.PROP_REQUEST_SECURITY_TOKEN}>$securityToken</${SoapRMI.PROP_REQUEST_SECURITY_TOKEN}>');
    }

    soap.write(
        '<${SoapRMI.PROP_REQUEST_INSTANCE_ID}>$instanceId</${SoapRMI.PROP_REQUEST_INSTANCE_ID}>');
    soap.write(
        '<${SoapRMI.PROP_REQUEST_OFFLINE_LOGIN_FLAG}>$offlineLogin</${SoapRMI.PROP_REQUEST_OFFLINE_LOGIN_FLAG}>');
    soap.write(
        '<${SoapRMI.PROP_REQUEST_OPERATION_NAME}>$method</${SoapRMI.PROP_REQUEST_OPERATION_NAME}>');

    return soap.toString();
  }

  String prefixEndpoint() {
    return endpoint + name;
  }
}
