/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../config/defaults.dart';
import '../../../../util/date_utils.dart';
import '../../../../validation/converter/c_date.dart';
import '../../../../validation/converter/c_long.dart';
import '../../../../validation/converter/c_number.dart';

class CDateTimePicker {
  const CDateTimePicker._();

  static DateTime toDate(Object val) {
    return toDate_(val, null);
  }

  static DateTime toDate_(Object val, String displayPattern) {
    return toDateTime(val, displayPattern, Defaults.DATE_CONVERSION_FORMAT);
  }

  static DateTime toTime(Object val) {
    return toTime_(val, null);
  }

  static DateTime toTime_(Object val, String displayPattern) {
    return toDateTime(val, displayPattern, Defaults.TIME_CONVERSION_FORMAT);
  }

  static DateTime toDateTime(
      Object val, String displayPattern, String defaultPattern) {
    DateTime dateTime;

    if (val is DateTime) {
      dateTime = val;
    } else if (val is int) {
      dateTime = DateUtils.millisToDate(val);
    } else if (val is String) {
      final String sTime = val.trim();

      // 1. Try to use the given time as 'long' (ms)
      final int milliseconds = CLong.parseNoErr(sTime);

      if (milliseconds != null && milliseconds > CNumber.MAX_VALUE) {
        dateTime = DateUtils.millisToDate(milliseconds);
      } else {
        // 2. Try to parse using all possible patterns
        dateTime = CDate.parseWithMany(sTime, [
          displayPattern,
          defaultPattern,
          Defaults.DATE_TIME_CONVERSION_FORMAT,
          Defaults.YYYY_MM_DD,
          Defaults.DD_MM_YYYY,
          Defaults.DATE_FORMAT
        ]);
      }
    }

    final int year =
        dateTime == null ? Defaults.MISSING : DateUtils.getYear(dateTime);
    return year == Defaults.MISSING || year < 1000 || year > 9999
        ? null
        : dateTime;
  }
}
