/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../enums/code.dart';
import '../../../../util/form_utils.dart';
import '../../../../util/structure/string_set.dart';
import '../../../../util/utils.dart';
import '../../component/behavior/i_identify.dart';
import '../../component/behavior/i_input.dart';
import '../../component/behavior/i_value.dart';
import '../../component/tick_box.dart';
import '../../form_controller.dart';
import '../../renderer/component_factory.dart';
import '../../renderer/features.dart';
import '../../util/key_value_pair.dart';
import '../on_value_change_listener.dart';

class CMultipleChoice {
  static List<TickBox> toChoices(
      Set<KeyValuePair> array, bool enabled, FormController controller) {
    List<TickBox> choices = [];
    ComponentFactory factory = Features.getFactory(controller);

    for (KeyValuePair pair in array) {
      TickBox tickBox = factory.tickBox();
      tickBox.setIdentity(pair.getIdentity());
      tickBox.setLabel(pair.getText());
      tickBox.setEnabled(enabled);
      choices.add(tickBox);
    }

    return choices;
  }

  static List<String> getVal(List<TickBox> choices) {
    Set<String> selected = {};

    if (!Utils.isBlank(choices)) {
      for (TickBox tickBox in choices) {
        if (tickBox.getVal() as bool) {
          selected.add(tickBox.getIdentity());
        }
      }
    }

    return Utils.toArray(selected);
  }

  static void setVal(Object val, IInput iInput, List<TickBox> choices,
      OnValueChangeListener onValueChange) {
    if (!Utils.isBlank(choices)) {
      final StringSet oldVal = toSet(iInput.getVal(), choices);
      final StringSet newVal = toSet(val, choices);

      // ensure value change
      if (oldVal != newVal) {
        for (TickBox tickBox in choices) {
          tickBox.setVal(newVal.contains(tickBox.getIdentity()));
        }

        FormUtils.fireOnValueChange__(newVal, false, onValueChange);
      }
    }
  }

  static void setVal_(Object val, IInput iInput, Set<KeyValuePair> choices,
      IValue<StringSet> iValue, OnValueChangeListener onValueChange) {
    if (!Utils.isBlank(choices)) {
      final StringSet oldVal = toSet(iInput.getVal(), choices);
      final StringSet newVal = toSet(val, choices);

      // ensure value change
      if (oldVal != newVal) {
        iValue.setValue(newVal);
        FormUtils.fireOnValueChange__(newVal, false, onValueChange);
      }
    }
  }

  static StringSet toSet(Object val, Iterable<IIdentify> choices) {
    if (val is String) {
      final String value = val as String;

      if (Code.ALL.match(value)) {
        final int size = Utils.getSize(choices);
        List<String> keys = [];

        for (int index = 0; index < size; index++) {
          keys[index] = choices.elementAt(index).getIdentity();
        }

        val = keys;
      } else {
        val = Utils.splitByComma_(value, true);
      }
    }

    List<String> selectionList = [];

    if (val is List<String>) {
      selectionList = val;
    }

    return StringSet.list(selectionList);
  }
}
