/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../util/form_utils.dart';
import '../../../../util/utils.dart';
import '../../component/behavior/i_input.dart';
import '../../component/behavior/i_value.dart';
import '../../component/radio_button.dart';
import '../../form_controller.dart';
import '../../renderer/component_factory.dart';
import '../../renderer/features.dart';
import '../../util/key_value_pair.dart';
import '../on_value_change_listener.dart';

class CSingleChoice {
  const CSingleChoice._();

  static List<RadioButton> toChoices(
      Set<KeyValuePair> array, bool enabled, FormController controller) {
    List<RadioButton> choices = [];
    ComponentFactory factory = Features.getFactory(controller);

    for (KeyValuePair pair in array) {
      RadioButton radioButton = factory.radioButton();
      radioButton.setIdentity(pair.getIdentity());
      radioButton.setLabel(pair.getText());
      radioButton.setEnabled(enabled);
      choices.add(radioButton);
    }

    return choices;
  }

  static Object getVal(List<RadioButton> choices) {
    if (!Utils.isBlank(choices)) {
      for (RadioButton radio in choices) {
        if (Utils.toFalse(radio.getVal() as bool)) {
          return radio.getIdentity();
        }
      }
    }

    return null;
  }

  static void setVal(Object val, IInput iInput, IValue<String> iValue,
      OnValueChangeListener onValueChange, bool ensureChange) {
    if (val == null || val is String) {
      final String newVal = Utils.trimOrToBlank(val as String, true);

      if (ensureChange) {
        final String oldVal = Utils.toBlank(iInput.getVal() as String);

        // ensure value change
        if (oldVal != newVal) {
          setVal__(newVal, iValue, onValueChange, false);
        }
      } else {
        setVal__(newVal, iValue, onValueChange, true);
      }
    }
  }

  static void setVal_(String value, List<RadioButton> choices) {
    if (!Utils.isBlank(choices)) {
      for (RadioButton radioButton in choices) {
        radioButton
            .setVal(Utils.equalsIgnoreCase(radioButton.getIdentity(), value));
      }
    }
  }

  static void setVal__(String val, IValue<String> iValue,
      OnValueChangeListener onValueChange, bool ensureChange) {
    final String newVal = Utils.toNull(val);
    iValue.setValue(newVal);
    FormUtils.fireOnValueChange__(newVal, ensureChange, onValueChange);
  }
}
