/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../enums/alignment.dart';
import '../../../../enums/font_style.dart';
import '../../../../enums/orientation.dart';
import '../../../../util/bean/range/int_range.dart';
import '../../action/handler/action_handler.dart';
import '../../common/on_value_change_listener.dart';
import '../../util/key_value_pair.dart';
import '../../view_state.dart';
import '../base/component.dart';
import '../base/input_component.dart';
import '../behavior/event/i_click.dart';
import '../behavior/i_align.dart';
import '../behavior/i_array.dart';
import '../behavior/i_capture.dart';
import '../behavior/i_column.dart';
import '../behavior/i_contain.dart';
import '../behavior/i_err.dart';
import '../behavior/i_font.dart';
import '../behavior/i_hint.dart';
import '../behavior/i_label.dart';
import '../behavior/i_mask.dart';
import '../behavior/i_orient.dart';
import '../behavior/i_pattern.dart';
import '../behavior/i_row.dart';
import '../behavior/i_select_date.dart';
import '../behavior/i_select_file.dart';
import '../label.dart';
import '../message.dart';
import '../tick_box.dart';

class LabelledComponent
    implements
        InputComponent,
        ILabel,
        IFont,
        IErr,
        IArray,
        ICapture,
        ISelectFile,
        IPattern,
        ISelectDate,
        IColumn,
        IRow,
        IAlign,
        IHint,
        IMask,
        IOrient,
        IClick {
  Component tickboxHolder; // Can be Tick box or Label
  Label
      lblText; // Either lblText or msgErr can be null but both can't be null at the same time.
  InputComponent inputComponent; // can never be null
  Message msgErr;

  bool _toastError = false;

  LabelledComponent() {
    // nothing to do here
  }

  @override
  String getIdentity() {
    return inputComponent.getIdentity();
  }

  @override
  void setIdentity(String id) {
    inputComponent.setIdentity(id);
  }

  @override
  void setErrMsg(String msg) {
    IErr err = (inputComponent is IErr ? inputComponent : msgErr) as IErr;

    if (err != null) {
      err.setErrMsg(msg);
    }
  }

  @override
  Object getVal() {
    return inputComponent.getVal();
  }

  @override
  void setVal(Object val) {
    this.inputComponent.setVal(val);
  }

  @override
  bool isEnabled() {
    return inputComponent.isEnabled();
  }

  @override
  void setEnabled(bool enabled) {
    inputComponent.setEnabled(enabled);
  }

  @override
  void setArray(Set<KeyValuePair> array, bool blankItem, String blankMsg) {
    if (inputComponent is IArray) {
      (inputComponent as IArray).setArray(array, blankItem, blankMsg);
    }
  }

  @override
  void clearArray() {
    if (inputComponent is IArray) {
      (inputComponent as IArray).clearArray();
    }
  }

  @override
  bool haveItems() {
    return inputComponent is IArray && (inputComponent as IArray).haveItems();
  }

  @override
  bool isValidItem(Object val) {
    return inputComponent is IArray &&
        (inputComponent as IArray).isValidItem(val);
  }

  Component getTickboxHolder() {
    return tickboxHolder;
  }

  void setTickboxHolder(Component tickboxHolder) {
    this.tickboxHolder = tickboxHolder;
  }

  Label getLblText() {
    return this.lblText;
  }

  void setLblText(Label label) {
    this.lblText = label;
  }

  void setMandatory(bool mandatory) {
    if (lblText != null) {
      lblText.setMandatory(mandatory);
    }
  }

  InputComponent getInputComponent() {
    return this.inputComponent;
  }

  void setInputComponent(InputComponent component) {
    this.inputComponent = component;
  }

  void setMsgErr(Message msgErr) {
    this.msgErr = msgErr;
  }

  @override
  OnValueChangeListener onValueChange() {
    return inputComponent.onValueChange();
  }

  @override
  void onValueChange_(ViewState view, ActionHandler handler) {
    inputComponent.onValueChange_(view, handler);
  }

  @override
  void setDimension(IntRange width, IntRange height, bool autoScale) {
    if (inputComponent is ICapture) {
      (inputComponent as ICapture).setDimension(width, height, autoScale);
    }
  }

  @override
  void setQuality(int quality) {
    if (inputComponent is ICapture) {
      (inputComponent as ICapture).setQuality(quality);
    }
  }

  @override
  void setFeatures(bool cameraSwitch, bool flash) {
    if (inputComponent is ICapture) {
      (inputComponent as ICapture).setFeatures(cameraSwitch, flash);
    }
  }

  @override
  void onConstraintChanged() {
    if (inputComponent is ICapture) {
      (inputComponent as ICapture).onConstraintChanged();
    }
  }

  @override
  void setExts(String exts) {
    if (inputComponent is ISelectFile) {
      (inputComponent as ISelectFile).setExts(exts);
    }
  }

  @override
  void setFileSize(IntRange fileSize, bool autoCompress) {
    if (inputComponent is ISelectFile) {
      (inputComponent as ISelectFile).setFileSize(fileSize, autoCompress);
    }
  }

  @override
  bool isRendered() {
    return inputComponent.isRendered();
  }

  @override
  void setRendered(bool rendered) {
    if (tickboxHolder != null) {
      tickboxHolder.setRendered(rendered);
    }

    if (lblText != null) {
      lblText.setRendered(rendered);
    }

    inputComponent.setRendered(rendered);

    if (msgErr != null) {
      msgErr.setRendered(rendered);
    }
  }

  void addToContainer(IContain container) {
    if (tickboxHolder != null) {
      container.add(tickboxHolder);
    }

    if (lblText != null) {
      container.add(lblText);
    }

    container.add(inputComponent);

    if (msgErr != null) {
      container.add(msgErr);
    }
  }

  @override
  void setWidth(String width) {
    inputComponent.setWidth(width);
  }

  @override
  void setHeight(String height) {
    inputComponent.setHeight(height);
  }

  @override
  void setFontColor(String color) {
    if (lblText != null) {
      lblText.setFontColor(color);
    }

    if (inputComponent is IFont) {
      (inputComponent as IFont).setFontColor(color);
    }

    if (msgErr is IFont) {
      (msgErr as IFont).setFontColor(color);
    }
  }

  @override
  void setFontSize(String size) {
    if (lblText != null) {
      lblText.setFontSize(size);
    }

    if (inputComponent is IFont) {
      (inputComponent as IFont).setFontSize(size);
    }

    if (msgErr is IFont) {
      (msgErr as IFont).setFontSize(size);
    }
  }

  @override
  void setFontStyle(FontStyle style) {
    if (lblText != null) {
      lblText.setFontStyle(style);
    }

    if (inputComponent is IFont) {
      (inputComponent as IFont).setFontStyle(style);
    }

    if (msgErr is IFont) {
      (msgErr as IFont).setFontStyle(style);
    }
  }

  @override
  void setStyle(String style) {
    inputComponent.setStyle(style);
  }

  @override
  void setPattern(String pattern) {
    if (inputComponent is IPattern) {
      (inputComponent as IPattern).setPattern(pattern);
    }
  }

  @override
  void setSelectionDate(DateTime date) {
    if (inputComponent is ISelectDate) {
      (inputComponent as ISelectDate).setSelectionDate(date);
    }
  }

  @override
  void setColumns(int columns) {
    if (inputComponent is IColumn) {
      (inputComponent as IColumn).setColumns(columns);
    }
  }

  @override
  String getLabel() {
    return lblText == null ? null : lblText.getLabel();
  }

  @override
  void setLabel(String label) {
    if (lblText != null) {
      lblText.setLabel(label);
    }
  }

  @override
  void setAlignment(Alignment alignment) {
    if (inputComponent is IAlign) {
      (inputComponent as IAlign).setAlignment(alignment);
    }
  }

  @override
  void setRows(int rows) {
    if (inputComponent is IRow) {
      (inputComponent as IRow).setRows(rows);
    }
  }

  @override
  void setHint(String hint) {
    if (inputComponent is IHint) {
      (inputComponent as IHint).setHint(hint);
    }
  }

  @override
  void focus() {
    inputComponent.focus();
  }

  @override
  void setMask(String mask, String placeholder, bool useMaskedValue) {
    if (inputComponent is IMask) {
      (inputComponent as IMask).setMask(mask, placeholder, useMaskedValue);
    }
  }

  @override
  void setOrientation(Orientation orientation) {
    if (inputComponent is IOrient) {
      (inputComponent as IOrient).setOrientation(orientation);
    }
  }

  @override
  void onClick(ViewState view, ActionHandler handler) {
    if (inputComponent is IClick) {
      (inputComponent as IClick).onClick(view, handler);
    }
  }

  @override
  bool click() {
    return inputComponent is IClick && (inputComponent as IClick).click();
  }

  bool toastError() {
    return _toastError;
  }

  void setToastError(bool toastError) {
    this._toastError = toastError;
  }

  void setRejected(bool rejected) {
    if (tickboxHolder is TickBox) {
      (tickboxHolder as TickBox).setVal(rejected);
    }
  }

  bool isRejected() {
    // NPE not expected because TickBox.getVal() is promised to return primitive boolean.
    return tickboxHolder is TickBox
        ? (tickboxHolder as TickBox).getVal() as bool
        : false;
  }

  static Component getComponent(Component component) =>
      component is LabelledComponent
          ? component.getInputComponent()
          : component;
}
