/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/t_map.dart';
import '../../enums/field_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../structure/model/field.dart';
import '../../structure/ui/view.dart';
import '../../util/form_utils.dart';
import '../../util/structure/string_set.dart';
import '../../util/utils.dart';
import '../../validation/constraint/c_mandatory.dart';
import 'renderer/ui_renderer.dart';

class FieldState {
  final Field field;
  final bool undeclared;
  final StringSet activeViews;

  String selectedArray;

  FieldState(Field field)
      : this.field = field,
        this.undeclared = field == null,
        this.activeViews = StringSet();

  static FieldState get(
      View view, TMap<FieldState> fields, UIRenderer renderer) {
    final String fieldId = view.getField();
    Field field = FormUtils.getField__(fieldId, renderer);
    final bool declared = field != null;
    FieldState fieldState;

    if (view.getType().canInput()) {
      if (declared) {
        field.addView(view.getId());
      }

      fieldState = fields[fieldId];

      if (fieldState == null) {
        fieldState = FieldState(field);
        fields.put(fieldId, fieldState);
      }
    } else if (declared) {
      throw IllegalArgumentException(
          "Only input components can have field declaration [$fieldId]");
    }

    return fieldState;
  }

  Field getField() {
    return field;
  }

  StringSet getActiveViews() {
    return activeViews;
  }

  String getId() {
    return undeclared ? null : field.getId();
  }

  FieldType getType() {
    return undeclared ? null : field.getType();
  }

  String getDefinedArray() {
    return undeclared ? null : field.getArray();
  }

  String getArray() {
    return Utils.toDefault(getDefinedArray(), selectedArray);
  }

  void setArray(String selectedArray) {
    if (Utils.isBlank(getDefinedArray())) {
      this.selectedArray = selectedArray;
    }
  }

  String getParentOf() {
    return undeclared ? null : field.getParentOf();
  }

  String getInclusion() {
    return undeclared ? null : field.getInclusion();
  }

  String getPreviewValue() {
    return undeclared ? null : field.getPreviewValue();
  }

  bool isMandatory(UIRenderer renderer) {
    return field != null && field.get(CMandatory.TYPE, renderer) != null;
  }

  @override
  String toString() => undeclared ? "_undeclared" : field.toString();
}
