/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/string_object_map.dart';
import '../../enums/mode.dart';
import '../../structure/ui/method/method.dart';
import '../../structure/ui/page.dart';
import '../../util/script/el_functions.dart';
import '../../validation/constraint_factory.dart';
import 'action/data/accept_reject_action.dart';
import 'action/data/data_consumer.dart';
import 'action/data/data_printer.dart';
import 'action/data/data_provider.dart';
import 'action/data/rmi.dart';
import 'action/data/value_resolver.dart';
import 'action/sms/sms_sender.dart';
import 'renderer/component_factory.dart';
import 'renderer/params/form_params.dart';
import 'renderer/ui_renderer.dart';
import 'view_state.dart';

abstract class FormController {
  /// In charge of creating objects and interface
  Future<void> onCreate();

  /// Reset the form in the given mode but if it isn't provided, use the mode initially used to render the form.
  Future<void> onReset(Mode mode);

  void onRecreate(FormParams params);

  void navigate(Page page);

  void onClose(bool immediate);

  void destroy(bool allFiles);

  void cleanUp(bool allFiles);

  ComponentFactory<FormController> getFactory();

  ConstraintFactory getConstraintFactory();

  UIRenderer getRenderer();

  Object getFormPK();

  void setTitle_(String title, String subTitle);

  StringObjectMap getExtras();

  void scrollToView(ViewState view);

  /// Return null if you have no custom functions to add.
  ELFunctions getELFunctions();

  bool runScript(String script, ViewState vSrc, Map<String, Object> args);

  void log(Object object);

  void alertSecurityExpired();

  // TODO Move all actions to one class

  DataProvider getDataProvider();

  DataConsumer getDataConsumer();

  AcceptRejectAction getAcceptRejectAction();

  DataPrinter getDataPrinter();

  ValueResolver getValueResolver();

  SmsSender getSmsSender();

  RMI getRMI(Method method);
}
