/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../config/config.dart';
import '../../../util/log/logger.dart';
import '../../../util/utils.dart';

class AggregateBundle {
  final Map<String, String> external; // injected from outside CMS (optional)
  final Map<String, String>
      custom; // part of the UI framework (mt-form-faces etc.)
  final Map<String, String> base; // part of mt-formalyze

  AggregateBundle(String language, Map<String, Map<String, String>> custom,
      Map<String, Map<String, String>> base)
      : this.external = getBundle(language, getExternalBundle()),
        this.custom = getBundle(language, custom),
        this.base = getBundle(language, base);

  String getMsg(String msgId) {
    String msg;

    try {
      msg = getString(msgId, external);
    } catch (e) {
      try {
        // if message isn't found in the external bundle then we try to look for it in the custom bundle.
        msg = getString(msgId, custom);
      } catch (e2) {
        try {
          // if message isn't found in the custom bundle then we try to look for it in the API bundle.
          msg = getString(msgId, base);
        } catch (e3) {
          msg =
              msgId; // if missing in all the bundles, we consider it as external.
        }
      }
    }

    return Utils.toUtf8_(msg);
  }

  String getString(String msgId, Map<String, String> bundle) {
    final String key = msgId.toLowerCase();

    if (!bundle.containsKey(key)) {
      throw ("");
    }

    return bundle[key]; // will throw NPE if bundle is null
  }

  static Map<String, Map<String, String>> getExternalBundle() {
    Map<String, Map<String, String>> external = Config.EXTERNAL_BUNDLE;

    // In development mode, we don't do caching because it may change on the fly.
    if (Config.PRODUCTION_MODE) {
      Config.EXTERNAL_BUNDLE = null; // free up resources
    }

    if (Utils.isBlank(external)) {
      Logger.warn("External resource bundle is missing.");
    }

    return external;
  }

  // Filters out translations of the given language only
  static Map<String, String> getBundle(
      String language, Map<String, Map<String, String>> fullBundle) {
    Map<String, String> bundle = {};

    if (!Utils.isBlank(fullBundle)) {
      String langInUpper = language.toUpperCase();

      for (String key in fullBundle.keys) {
        bundle[key.toLowerCase()] = fullBundle[key][langInUpper];
      }
    }

    return bundle;
  }
}
