/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'package:flutter_js/javascript_runtime.dart' show JavascriptRuntime;

import '../../../util/bean_utils.dart';
import '../../../util/form_utils.dart';
import '../../../util/script/script_evaluator.dart';
import '../../../util/utils.dart';
import '../../../util/value/distributor.dart';
import '../../../util/value/replicator.dart';
import '../../../validation/constraint_factory.dart';
import '../../../validation/validator.dart';
import '../action/data/accept_reject_action.dart';
import '../action/data/data_comparator.dart';
import '../action/data/data_consumer.dart';
import '../action/data/data_preparer.dart';
import '../action/data/data_printer.dart';
import '../action/data/data_provider.dart';
import '../action/data/value_resolver.dart';
import '../action/event_handler.dart';
import '../action/handler/comm_actions.dart';
import '../action/handler/data_actions.dart';
import '../action/handler/form_actions.dart';
import '../action/handler/rmi/rmi_action.dart';
import '../action/handler/table_actions.dart';
import '../component/alert.dart';
import '../data/data_handler.dart';
import '../form_controller.dart';
import 'array_value_handler.dart';
import 'component_factory.dart';
import 'event_processor.dart';
import 'ui_renderer.dart';

class Features {
  UIRenderer renderer;

  DataComparator dataComparator;
  DataPreparer dataPreparer;
  DataProvider dataProvider;
  ValueResolver resolver;

  RMIAction rmiAction;
  CommActions commActions;
  DataActions dataActions;
  FormActions formActions;
  TableActions tableActions;

  EventHandler eventHandler;
  DataHandler dataHandler;
  ArrayValueHandler arrayValueHandler;
  ComponentFactory factory;
  EventProcessor eventProcessor;

  Distributor distributor;
  Replicator replicator;

  Alert alert;
  ConstraintFactory constraintFactory;
  Validator validator;
  JavascriptRuntime engine;

  Features(this.renderer);

  void cleanUp() {
    renderer = null;

    dataComparator = null;
    dataPreparer = null;
    dataProvider = null;
    resolver = null;

    rmiAction = null;
    commActions = null;
    dataActions = null;
    formActions = null;
    tableActions = null;

    eventHandler = null;
    dataHandler = null;
    arrayValueHandler = null;
    factory = null;
    eventProcessor = null;

    distributor = null;
    replicator = null;

    alert = null;
    constraintFactory = null;
    validator = null;

    if (engine != null) {
      engine.dispose();
      engine = null;
    }
  }

  FormController getController() {
    return renderer.getController();
  }

  static Features getFeatures(UIRenderer renderer) {
    return renderer.getFeatures();
  }

  static AcceptRejectAction getAcceptRejectAction(UIRenderer renderer) {
    return renderer == null
        ? null
        : getFeatures(renderer).getController().getAcceptRejectAction();
  }

  static DataComparator getDataComparator(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDataComparator();
  }

  DataComparator _getDataComparator() {
    if (dataComparator == null) {
      dataComparator = DataComparator(renderer);
    }

    return dataComparator;
  }

  static DataConsumer getDataConsumer(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDataConsumer();
  }

  DataConsumer _getDataConsumer() {
    final String clazz = FormUtils.getUI_(renderer).getDataConsumer();
    return Utils.isBlank(clazz)
        ? getController().getDataConsumer()
        : BeanUtils.newInstance(clazz);
  }

  static DataPreparer getDataPreparer(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDataPreparer();
  }

  DataPreparer _getDataPreparer() {
    if (dataPreparer == null) {
      dataPreparer = DataPreparer(renderer);
    }

    return dataPreparer;
  }

  static DataPrinter getDataPrinter(UIRenderer renderer) {
    return renderer == null
        ? null
        : getFeatures(renderer).getController().getDataPrinter();
  }

  static DataProvider getDataProvider(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDataProvider();
  }

  DataProvider _getDataProvider() {
    if (dataProvider == null) {
      final String clazz = FormUtils.getUI_(renderer).getDataProvider();
      dataProvider = Utils.isBlank(clazz)
          ? getController().getDataProvider()
          : BeanUtils.newInstance(clazz);
    }

    return dataProvider;
  }

  static ValueResolver getResolver_(FormController controller) {
    return getResolver(FormUtils.getRenderer(controller));
  }

  static ValueResolver getResolver(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getResolver();
  }

  ValueResolver _getResolver() {
    if (resolver == null) {
      resolver = getController().getValueResolver();
    }

    return resolver;
  }

  static RMIAction getRMIAction(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getRMIAction();
  }

  RMIAction _getRMIAction() {
    if (rmiAction == null) {
      rmiAction = RMIAction(renderer);
    }

    return rmiAction;
  }

  static CommActions getCommActions(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getCommActions();
  }

  CommActions _getCommActions() {
    if (commActions == null) {
      commActions = CommActions(renderer);
    }

    return commActions;
  }

  static DataActions getDataActions(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDataActions();
  }

  DataActions _getDataActions() {
    if (dataActions == null) {
      dataActions = DataActions(renderer);
    }

    return dataActions;
  }

  static FormActions getFormActions(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getFormActions();
  }

  FormActions _getFormActions() {
    if (formActions == null) {
      formActions = FormActions(renderer);
    }

    return formActions;
  }

  static TableActions getTableActions(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getTableActions();
  }

  TableActions _getTableActions() {
    if (tableActions == null) {
      tableActions = TableActions(renderer);
    }

    return tableActions;
  }

  static EventHandler getEventHandler(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getEventHandler();
  }

  EventHandler _getEventHandler() {
    if (eventHandler == null) {
      eventHandler = EventHandler(renderer);
    }

    return eventHandler;
  }

  static DataHandler getDataHandler(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDataHandler();
  }

  DataHandler _getDataHandler() {
    if (dataHandler == null) {
      dataHandler = DataHandler(renderer);
    }

    return dataHandler;
  }

  static ArrayValueHandler getArrayValueHandler(UIRenderer renderer) {
    return renderer == null
        ? null
        : getFeatures(renderer)._getArrayValueHandler();
  }

  ArrayValueHandler _getArrayValueHandler() {
    if (arrayValueHandler == null) {
      arrayValueHandler = ArrayValueHandler(renderer);
    }

    return arrayValueHandler;
  }

  static ComponentFactory getFactory(FormController controller) {
    return getFactory_(FormUtils.getRenderer(controller));
  }

  static ComponentFactory getFactory_(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getFactory();
  }

  ComponentFactory _getFactory() {
    if (factory == null) {
      factory = getController().getFactory();
    }

    return factory;
  }

  static EventProcessor getEventProcessor(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getEventProcessor();
  }

  EventProcessor _getEventProcessor() {
    if (eventProcessor == null) {
      eventProcessor = EventProcessor(renderer);
    }

    return eventProcessor;
  }

  static Distributor getDistributor(FormController controller) {
    return getDistributor_(FormUtils.getRenderer(controller));
  }

  static Distributor getDistributor_(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getDistributor();
  }

  Distributor _getDistributor() {
    if (distributor == null) {
      distributor = Distributor(renderer);
    }

    return distributor;
  }

  static Replicator getReplicator(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getReplicator();
  }

  Replicator _getReplicator() {
    if (replicator == null) {
      replicator = Replicator(renderer);
    }

    return replicator;
  }

  static Alert getAlert(FormController controller) {
    return getAlert_(FormUtils.getRenderer(controller));
  }

  static Alert getAlert_(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getAlert();
  }

  Alert _getAlert() {
    if (alert == null) {
      alert = _getFactory().getAlert();
    }

    return alert;
  }

  static ConstraintFactory getConstraintFactory(UIRenderer renderer) {
    return renderer == null
        ? null
        : getFeatures(renderer)._getConstraintFactory();
  }

  ConstraintFactory _getConstraintFactory() {
    if (constraintFactory == null) {
      constraintFactory = getController().getConstraintFactory();
    }

    return constraintFactory;
  }

  static Validator getValidator(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getValidator();
  }

  Validator _getValidator() {
    if (validator == null) {
      validator = Validator(renderer);
    }

    return validator;
  }

  static JavascriptRuntime getEngine(UIRenderer renderer) {
    return renderer == null ? null : getFeatures(renderer)._getEngine();
  }

  JavascriptRuntime _getEngine() {
    if (engine == null) {
      engine = ScriptEvaluator.getEngine(renderer);
    }

    return engine;
  }
}
