/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../config/defaults.dart';
import '../../../../enums/mode.dart';
import '../../../../error/illegal_argument_exception.dart';
import '../../../../util/form_utils.dart';
import '../../../../util/structure/unique_string_map.dart';
import '../../../../util/utils.dart';
import '../../../../validation/converter/c_string.dart';
import '../ui_renderer.dart';

class FormParams {
  final String formId;
  final String language;
  final Mode mode;
  final Object formPK;

  const FormParams._(this.formId, this.language, this.mode, this.formPK);

  static FormParams getInstance(String inlinePairs, UIRenderer renderer) {
    String formId, language;
    Mode mode;
    Object formPK;
    UniqueStringMap pairs = FormUtils.pairsToMap(inlinePairs);

    if (!Utils.isBlank(pairs)) {
      for (String param in pairs.keys) {
        final Object val = FormUtils.resolve_(pairs[param], renderer);

        if (Utils.equalsIgnoreCase(Defaults.FORM_ID, param)) {
          formId = CString.convert(val);
        } else if (Utils.equalsIgnoreCase(Defaults.LANGUAGE, param)) {
          language = CString.convert(val);
        } else if (Utils.equalsIgnoreCase(Defaults.MODE, param)) {
          mode = Mode.toMode(CString.convert(val), false);
        } else if (Utils.equalsIgnoreCase(Defaults.FORM_PK, param)) {
          formPK = val;
        } else {
          throw IllegalArgumentException("Unknown param: $param");
        }
      }
    }

    return FormParams._(formId, language, mode, formPK);
  }

  String getFormId(String formId) {
    return Utils.toDefault(this.formId, formId);
  }

  String getLanguage(String language) {
    return Utils.toDefault(this.language, language);
  }

  Mode getMode(Mode mode) {
    return Utils.toDefault(this.mode, mode);
  }

  Object getFormPK(Object formPK) {
    return Utils.toDefault(this.formPK, formPK);
  }

  @override
  String toString() =>
      "FormParams [formId=$formId, language=$language, mode=$mode, formPK=$formPK]";
}
