/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../../form/bean/master_data_version_bean.dart';
import '../../../../config/config.dart';
import '../../../../enums/mode.dart';
import '../../../../generator/form.dart';
import '../../../../permission/permissions.dart';
import '../../../../structure/i18n/i18n.dart';
import '../../../../structure/i18n/locale.dart';
import '../../../../structure/model/model.dart';
import '../../../../structure/ui/ui.dart';
import '../../../../structure/ui/view.dart';
import '../../../../util/utils.dart';

class RendererParams {
  final String formId;
  final Form form;
  final bool showRaw;

  Locale currentLocale;
  Mode currentMode;

  final Permissions permissions;

  RendererParams(Form form, bool showRaw, String language, Mode mode,
      Permissions permissions)
      : this.full(null, form, showRaw, language, mode, permissions);

  RendererParams.full(this.formId, this.form, this.showRaw, String language,
      Mode mode, Permissions permissions)
      : this.permissions =
            permissions == null ? Permissions.all() : permissions {
    setConfig();

    changeLocale(language);

    this.currentMode = Utils.toDefault(mode, Mode.CREATE);
  }

  void setConfig() {
    if (form != null) {
      UI ui = getBaseUI();

      Config.ENCODE_TEXT = ui.getEncode();
      Config.DEBUG = ui.isDebug();
      Config.IMMEDIATE = getModel().isImmediate();
    }
  }

  void reset(String language, Mode mode) {
    changeMode(mode);
    changeLocale(language);
  }

  /// Updates the current mode to the new mode if its different.
  ///
  /// @return returns true if current mode is updated, false otherwise.
  bool changeMode(Mode mode) {
    bool updated = false;

    if (mode != null && mode != currentMode) {
      currentMode = mode;
      updated = true;
    }

    return updated;
  }

  /// Updates the current locale. If blank is provided, default locale is used.
  ///
  /// @return true if current locale is updated, false otherwise.
  bool changeLocale(String language) {
    Locale locale = getI18n().toLocale(language);
    bool updated = false;

    // FIXME Because of workaround LOCs in FormActivity we don't check if the 'locale'
    // is different to 'currentLocale' but we should add this check '&& locale != currentLocale'
    if (locale != null) {
      currentLocale = locale;
      updated = true;
    }

    return updated;
  }

  String getFormId() {
    return this.formId;
  }

  Form getForm() {
    return this.form;
  }

  Model getModel() {
    return this.form.getModel();
  }

  UI getBaseUI() {
    return this.form.getUi();
  }

  I18n getI18n() {
    return this.form.getI18n();
  }

  String getTitle() {
    return getBaseUI().getTitle();
  }

  String getSubTitle() {
    return getBaseUI().getSubTitle();
  }

  bool isShowRaw() {
    return showRaw;
  }

  Locale getCurrentLocale() {
    return currentLocale;
  }

  Mode getCurrentMode() {
    return currentMode;
  }

  bool permitted(View view) {
    return permissions.permitted__(view);
  }

  Permissions getPermissions() {
    return permissions;
  }

  MasterDataVersionBean getFormVersion() {
    return MasterDataVersionBean.formId(form.getId(), getModel().getVersion(),
        getI18n().getVersion(), getBaseUI().getVersion());
  }
}
