/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../form/structure/ordered_map.dart';
import '../../../enums/scope.dart';
import '../../../structure/ui/view.dart';
import '../../../util/form_utils.dart';
import '../../../util/utils.dart';
import '../form_controller.dart';
import '../view_state.dart';

class FocusUtils {
  const FocusUtils._();

  static ViewState getTopVisibleView(
      List<View> layout, FormController controller) {
    OrderedMap<ViewState> allViews = FormUtils.getAllViews(controller);

    for (View child in layout) {
      ViewState vChild = allViews[child.getId()];

      if (vChild != null && vChild.isReallyRendered()) {
        return vChild;
      }
    }

    return null;
  }

  static String getFocus(String rootId, OrderedMap<ViewState> allViews) {
    if (Utils.isBlank(allViews)) {
      return null;
    }

    ViewState vRoot = allViews[rootId];
    String viewId = Utils.trim(vRoot.getFocus());

    if (Scope.isNone(viewId)) {
      return null;
    }

    ViewState view = allViews[viewId];
    return isFocusableAndVisible(view)
        ? view.getId()
        : getTopFocusableView(vRoot.getChildren());
  }

  static String getTopFocusableView(List<ViewState> children) {
    if (!Utils.isBlank(children)) {
      for (ViewState view in children) {
        if (isFocusableAndVisible(view)) {
          return view.getId();
        } else if (view.isContainer()) {
          String focusId = getTopFocusableView(view.getChildren());

          if (!Utils.isBlank(focusId)) {
            return focusId;
          }
        }
      }
    }

    return null;
  }

  static bool isFocusableAndVisible(ViewState view) {
    return view != null && view.isFocusable() && view.isReallyRendered();
  }
}
