/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../form/structure/string_map.dart';
import '../../../util/structure/string_set.dart';
import '../../../util/utils.dart';
import '../component/behavior/i_identify.dart';

class KeyValuePair implements IIdentify {
  final String id;
  final String text;

  const KeyValuePair(this.id, this.text);

  static KeyValuePair nullPair(String text) {
    return KeyValuePair(null, Utils.toBlank(text));
  }

  static String getId(KeyValuePair pair) {
    return pair == null ? null : pair.getIdentity();
  }

  static List<String> getIds(List<KeyValuePair> pairs) {
    if (Utils.isBlank(pairs)) {
      return null;
    }

    List<String> ids = [];

    for (KeyValuePair pair in pairs) {
      ids.add(pair.id);
    }

    return ids;
  }

  static String getText_(KeyValuePair pair) {
    return pair == null ? null : pair.getText();
  }

  static bool equals(KeyValuePair pair1, KeyValuePair pair2) {
    if (pair1 == pair2) {
      // checks if both are 'null' as well
      return true;
    }

    if (Utils.anyNull([pair1, pair2])) {
      // either one is null
      return false;
    }

    return pair1 == pair2;
  }

  static List<KeyValuePair> toArray(Set<KeyValuePair> array) {
    List<KeyValuePair> blankArray = [];
    return Utils.isBlank(array) ? blankArray : array.toList();
  }

  static void addToMap(Set<KeyValuePair> array, StringMap items) {
    for (KeyValuePair pair in array) {
      final String identity = pair.getIdentity();
      items.put(identity, identity);
    }
  }

  static KeyValuePair findById(String id, Set<KeyValuePair> array) {
    if (!Utils.isBlank(id) && !Utils.isBlank(array)) {
      for (KeyValuePair pair in array) {
        if (Utils.equalsIgnoreCase(id, pair.getIdentity())) {
          return pair;
        }
      }
    }

    return null;
  }

  static List<KeyValuePair> findByIds(StringSet ids, Set<KeyValuePair> array) {
    if (!Utils.isBlank(ids) && !Utils.isBlank(array)) {
      List<KeyValuePair> pairs = [];

      for (KeyValuePair pair in array) {
        if (ids.contains(pair.id)) {
          pairs.add(pair);
        }
      }

      return pairs;
    }

    return null;
  }

  @override
  String getIdentity() {
    return id;
  }

  @override
  void setIdentity(String id) {
    // nothing to do here
  }

  String getText() {
    return text;
  }

  @override
  String toString() => text;

  @override
  int get hashCode => 31 + (id == null ? 0 : id.hashCode);

  @override
  bool operator ==(obj) {
    if (identical(this, obj)) {
      return true;
    }

    if (obj == null) {
      return false;
    }

    if (runtimeType != obj.runtimeType) {
      return false;
    }

    KeyValuePair other = obj;

    if (id == null) {
      if (other.id != null) {
        return false;
      }
    } else if (!Utils.equalsIgnoreCase(id, other.id)) {
      return false;
    }

    return true;
  }
}
