/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../../util/log/logger.dart';
import '../../../../util/utils.dart';

class Languages {
  const Languages._();

  static const Language EN = Language("EN", "a-zA-Z", "0-9");
  static const Language FR = Language("FR", "a-zA-Z\u00C0-\u017F", "0-9");
  static const Language AR = Language("AR",
      "\u0620-\u063A\u0641-\u064A\u0671-\u06D3\u06FA-\u06FF", "\u0660-\u0669");
  static const Language FA = Language(
      "FA", "\u0600-\u065F\u066A-\u06EF\u06FA-\u06FF", "\u06F0-\u06F9");
  static const Language KU = Language(
      "KU",
      "\u0626-\u0628\u062A-\u062F\u0631-\u0634\u067E\u0641-\u0648\u0686\u0639\u063A\u0695\u0698\u06A4\u06A9\u06AF\u06B5\u06C6\u06C7\u06CA\u06BE\u06D5\u06CC\u06CE",
      "\u0660-\u0669");
  static const Language BU = Language(
      "EN_GB", "\u1000-\u103F\u104A-\u104F\u105A-\u109F", "\u1040-\u1049");

  static const Language DEFAULT = EN;
  static const List<Language> LANGUAGES = [EN, FR, AR, FA, KU, BU];

  static Language get(String id) {
    for (Language language in LANGUAGES) {
      if (Utils.equalsIgnoreCase(language.id, id)) {
        return language;
      }
    }

    Logger.warn("Unsupported language $id");
    return null;
  }
}

class Language {
  final String id;
  final String letters;
  final String digits;

  const Language(this.id, this.letters, this.digits);

  String getAllLetters() {
    return Utils.expand(letters);
  }

  String getAllDigits() {
    return Utils.expand(digits);
  }

  @override
  String toString() => "Language [id=$id, letters=$letters, digits=$digits]";
}
