/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../config/defaults.dart';
import '../../../validation/converter/c_date.dart';
import '../../date_utils.dart';
import '../../utils.dart';
import 'miscellaneous_binary_data.dart' as MiscBinaryData;
import 'miscellaneous_text_data.dart' as MiscTextData;

// We ignore all the Arabic fields
class CardData {
  static const String DELIMITER = ",";

  String AddressArabic;
  String AddressEnglish;
  String ArabicFirstName;
  String ArabicFullName;
  String ArabicLastName;
  String ArabicMiddleName2;
  String ArabicMiddleName3;
  String ArabicMiddleName4;
  String ArabicMiddleName5;
  String BirthDate;
  String CardCountry;
  String CardexpiryDate;
  String CardIssueDate;
  String CardSerialNumber;
  String CardVersion;
  String EmploymentFlag;
  String EmploymentId;
  String EmploymentNameArabic;
  String EmploymentNameEnglish;
  String EnglishFirstName;
  String EnglishFullName;
  String EnglishLastName;
  String EnglishMiddleName2;
  String EnglishMiddleName3;
  String EnglishMiddleName4;
  String EnglishMiddleName5;
  String FingerprintCode;
  String Gender;
  String IacoNationalityCode;
  String IsoNationalityCode;
  String IdNumber;
  String IsMatchOnCardAvailiable;
  MiscBinaryData.MiscellaneousBinaryData MiscellaneousBinaryData;
  MiscTextData.MiscellaneousTextData MiscellaneousTextData;
  String NationalityCode; // it's a non-standard number (e.g. 499 is for BHR)
  String OccupationArabic;
  String OccupationEnglish;
  String PassportExpiryDate;
  String PassportIssueDate;
  String PassportNumber;
  String PassportType;
  String Photo;
  String PhotoB64Encoded;
  String Signature;
  String SignB64Encoded;
  String SponserId;
  String SponserNameArabic;
  String SponserNameEnglish;
  String ErrorDescription;

  CardData() {
    // nothing to do here
  }

  CardData.json(Map json) {
    this.AddressArabic = json['AddressArabic'];
    this.AddressEnglish = json['AddressEnglish'];
    this.ArabicFirstName = json['ArabicFirstName'];
    this.ArabicFullName = json['ArabicFullName'];
    this.ArabicLastName = json['ArabicLastName'];
    this.ArabicMiddleName2 = json['ArabicMiddleName2'];
    this.ArabicMiddleName3 = json['ArabicMiddleName3'];
    this.ArabicMiddleName4 = json['ArabicMiddleName4'];
    this.ArabicMiddleName5 = json['ArabicMiddleName5'];
    this.BirthDate = json['BirthDate'];
    this.CardCountry = json['CardCountry'];
    this.CardexpiryDate = json['CardexpiryDate'];
    this.CardIssueDate = json['CardIssueDate'];
    this.CardSerialNumber = json['CardSerialNumber'];
    this.CardVersion = json['CardVersion'];
    this.EmploymentFlag = json['EmploymentFlag'];
    this.EmploymentId = json['EmploymentId'];
    this.EmploymentNameArabic = json['EmploymentNameArabic'];
    this.EmploymentNameEnglish = json['EmploymentNameEnglish'];
    this.EnglishFirstName = json['EnglishFirstName'];
    this.EnglishFullName = json['EnglishFullName'];
    this.EnglishLastName = json['EnglishLastName'];
    this.EnglishMiddleName2 = json['EnglishMiddleName2'];
    this.EnglishMiddleName3 = json['EnglishMiddleName3'];
    this.EnglishMiddleName4 = json['EnglishMiddleName4'];
    this.EnglishMiddleName5 = json['EnglishMiddleName5'];
    this.FingerprintCode = json['FingerprintCode'];
    this.Gender = json['Gender'];
    this.IacoNationalityCode = json['IacoNationalityCode'];
    this.IsoNationalityCode = json['IsoNationalityCode'];
    this.IdNumber = json['IdNumber'];
    this.IsMatchOnCardAvailiable = json['IsMatchOnCardAvailiable'];

    if (json.containsKey('MiscellaneousBinaryData')) {
      this.MiscellaneousBinaryData =
          MiscBinaryData.MiscellaneousBinaryData.json(
              json['MiscellaneousBinaryData']);
    }

    if (json.containsKey('MiscellaneousTextData')) {
      this.MiscellaneousTextData = MiscTextData.MiscellaneousTextData.json(
          json['MiscellaneousTextData']);
    }

    this.NationalityCode = json['NationalityCode'];
    this.OccupationArabic = json['OccupationArabic'];
    this.OccupationEnglish = json['OccupationEnglish'];
    this.PassportExpiryDate = json['PassportExpiryDate'];
    this.PassportIssueDate = json['PassportIssueDate'];
    this.PassportNumber = json['PassportNumber'];
    this.PassportType = json['PassportType'];
    this.Photo = json['Photo'];
    this.PhotoB64Encoded = json['PhotoB64Encoded'];
    this.Signature = json['Signature'];
    this.SignB64Encoded = json['SignB64Encoded'];
    this.SponserId = json['SponserId'];
    this.SponserNameArabic = json['SponserNameArabic'];
    this.SponserNameEnglish = json['SponserNameEnglish'];
    this.ErrorDescription = json['ErrorDescription'];
  }

  String format(String date) {
    return DateUtils.formatDefault(CDate.parseNoErr(date, Defaults.DD_MM_YYYY));
  }

  String getAddressEnglish() {
    if (Utils.isBlank(AddressEnglish)) {
      MiscTextData.MiscellaneousTextData data = getMiscTextData();

      StringBuffer address = StringBuffer();
      address.write("Flat/Villa ");
      address.write(Utils.toBlank(data.FlatNo));
      address.write(" Bldg #:");
      address.write(Utils.toBlank(data.BuildingNo));
      address.write(" Road :");
      address.write(Utils.toBlank(data.RoadNo));
      address.write(" Rd Name :");
      address.write(Utils.toBlank(data.RoadName));
      address.write("/");
      address.write(Utils.toBlank(data.RoadNameArabic));
      address.write(" Block #:");
      address.write(Utils.toBlank(data.BlockNo));
      address.write(" Block :");
      address.write(Utils.toBlank(data.BlockName));
      address.write("/");
      address.write(Utils.toBlank(data.BlockNameArabic));
      address.write(" Gov #:");
      address.write(Utils.toBlank(data.GovernorateNo));

      AddressEnglish = address.toString();
    }

    return AddressEnglish;
  }

  String getBirthDate() {
    final String birthdate = Utils.toBlank(format(BirthDate));
    return Utils.isBlank(birthdate)
        ? getMiscTextData().getDateOfBirth()
        : birthdate;
  }

  String getEmploymentFlag() {
    return Utils.toDefault(EmploymentFlag, getMiscTextData().EmployerFlag1);
  }

  String getEnglishFirstName() {
    final String name =
        Utils.toDefault(EnglishFirstName, getMiscTextData().FirstNameEnglish);

    if (Utils.isBlank(name)) {
      return Utils.hasSpace(EnglishFullName)
          ? EnglishFullName.substring(0, EnglishFullName.indexOf(' '))
          : EnglishFullName;
    }

    return name;
  }

  String getEnglishLastName() {
    final String name =
        Utils.toDefault(EnglishLastName, getMiscTextData().LastNameEnglish);
    return Utils.isBlank(name) && Utils.hasSpace(EnglishFullName)
        ? EnglishFullName.substring(EnglishFullName.indexOf(' ') + 1)
        : name;
  }

  String getEnglishMiddleName2() {
    return Utils.toDefault(
        EnglishMiddleName2, getMiscTextData().MiddleName1English);
  }

  String getEnglishMiddleName3() {
    return Utils.toDefault(
        EnglishMiddleName3, getMiscTextData().MiddleName2English);
  }

  String getEnglishMiddleName4() {
    return Utils.toDefault(
        EnglishMiddleName4, getMiscTextData().MiddleName3English);
  }

  String getEnglishMiddleName5() {
    return Utils.toDefault(
        EnglishMiddleName5, getMiscTextData().MiddleName4English);
  }

  String getGender() {
    return Utils.toDefault(Gender, getMiscTextData().Gender);
  }

  String getIacoNationalityCode() {
    return Utils.toDefault(IacoNationalityCode, getMiscTextData().IACOCode);
  }

  MiscTextData.MiscellaneousTextData getMiscTextData() {
    if (MiscellaneousTextData == null) {
      MiscellaneousTextData = MiscTextData.MiscellaneousTextData();
    }

    return MiscellaneousTextData;
  }

  String getNationalityCode() {
    return Utils.toDefault(NationalityCode, getMiscTextData().Nationality);
  }

  String getOccupationEnglish() {
    return Utils.toDefault(
        OccupationEnglish, getMiscTextData().OccupationDescription1);
  }

  String getPassportExpiryDate() {
    final String expiryDate = Utils.toBlank(format(PassportExpiryDate));
    return Utils.isBlank(expiryDate)
        ? getMiscTextData().getExpiryDate()
        : expiryDate;
  }

  String getPassportIssueDate() {
    final String issueDate = Utils.toBlank(format(PassportIssueDate));
    return Utils.isBlank(issueDate)
        ? getMiscTextData().getIssueDate()
        : issueDate;
  }

  String getPassportNumber() {
    return Utils.toDefault(PassportNumber, getMiscTextData().PassportNo);
  }

  String getPassportType() {
    return Utils.toDefault(PassportType, getMiscTextData().PassportType);
  }

  @override
  String toString() =>
      Utils.escapeAndToBlank(getAddressEnglish()) +
      DELIMITER +
      Utils.toBlank(getBirthDate()) +
      DELIMITER +
      Utils.toBlank(CardCountry) +
      DELIMITER +
      Utils.toBlank(format(CardexpiryDate)) +
      DELIMITER +
      Utils.toBlank(format(CardIssueDate)) +
      DELIMITER +
      Utils.toBlank(CardSerialNumber) +
      DELIMITER +
      Utils.toBlank(getEmploymentFlag()) +
      DELIMITER +
      Utils.toBlank(EmploymentId) +
      DELIMITER +
      Utils.escapeAndToBlank(EmploymentNameEnglish) +
      DELIMITER +
      Utils.escapeAndToBlank(getEnglishFirstName()) +
      DELIMITER +
      Utils.escapeAndToBlank(EnglishFullName) +
      DELIMITER +
      Utils.escapeAndToBlank(getEnglishLastName()) +
      DELIMITER +
      Utils.escapeAndToBlank(getEnglishMiddleName2()) +
      DELIMITER +
      Utils.escapeAndToBlank(getEnglishMiddleName3()) +
      DELIMITER +
      Utils.escapeAndToBlank(getEnglishMiddleName4()) +
      DELIMITER +
      Utils.escapeAndToBlank(getEnglishMiddleName5()) +
      DELIMITER +
      Utils.toBlank(getGender()) +
      DELIMITER +
      Utils.toBlank(getIacoNationalityCode()) +
      DELIMITER +
      Utils.toBlank(IdNumber) +
      DELIMITER +
      getMiscTextData().toString() +
      DELIMITER +
      Utils.toBlank(getNationalityCode()) +
      DELIMITER +
      Utils.escapeAndToBlank(getOccupationEnglish()) +
      DELIMITER +
      Utils.toBlank(getPassportExpiryDate()) +
      DELIMITER +
      Utils.toBlank(getPassportIssueDate()) +
      DELIMITER +
      Utils.toBlank(getPassportNumber()) +
      DELIMITER +
      Utils.toBlank(getPassportType()) +
      DELIMITER +
      Utils.toBlank(Photo) +
      DELIMITER +
      Utils.toBlank(Signature);
}
