/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../validation/converter/c_date.dart';
import '../../date_utils.dart';
import '../../utils.dart';

// We ignore all the Arabic fields
class MiscellaneousTextData {
  static const String DELIMITER = ",";

  String FirstNameArabic;
  String LastNameArabic;
  String MiddleName1Arabic;
  String MiddleName2Arabic;
  String MiddleName3Arabic;
  String MiddleName4Arabic;
  String BloodGroup;
  String CPRNO;
  String DateOfBirth; // duplicated in CardData (BirthDate) with the same value
  String
      FirstNameEnglish; // duplicated in CardData (EnglishFirstName) with the same value
  String
      LastNameEnglish; // duplicated in CardData (EnglishLastName) with the same value
  String
      MiddleName1English; // duplicated in CardData (EnglishMiddleName2) with the same value
  String
      MiddleName2English; // duplicated in CardData (EnglishMiddleName3) with the same value
  String
      MiddleName3English; // duplicated in CardData (EnglishMiddleName4) with the same value
  String
      MiddleName4English; // duplicated in CardData (EnglishMiddleName5) with the same value
  String Gender; // duplicated in CardData (Gender) with the same value
  String Email;
  String ContactNo;
  String ResidenceNo;
  String FlatNo;
  String BuildingNo;
  String BuildingAlpha;
  String BuildingAlphaArabic;
  String RoadNo;
  String RoadName;
  String RoadNameArabic;
  String BlockNo;
  String BlockName;
  String BlockNameArabic;
  String GovernorateNo;
  String GovernorateNameEnglish;
  String GovernorateNameArabic;
  String EmployerName1Arabic;
  String LatestEducationDegreeArabic;
  String OccupationDescription1Arabic;
  String SponsorNameArabic;
  String ClearingAgentIndicator;
  String EmployerFlag1;
  String EmployerName1;
  String EmployerNo1;
  String
      EmploymentFlag1; // duplicated in CardData (EmploymentFlag) with the same value
  String LaborForceParticipation;
  String LatestEducationDegree;
  String
      OccupationDescription1; // duplicated in CardData (OccupationEnglish) with the same value
  String SponsorCPRNoorUnitNo;
  String SponsorFlag;
  String SponsorName;
  String LfpNameEnglish;
  String LfpNameArabic;
  String EnglishCountryName;
  String ArabicCountryName;
  String
      IACOCode; // duplicated in CardData (IacoNationalityCode) with the same value
  String Alpha2Code;
  String
      Alpha3Code; // it's actually the nationality code following ISO standards (e.g. BHR)
  String
      Nationality; // duplicated in CardData (NationalityCode) with the same value
  String CountryOfBirth;
  String
      PassportNo; // duplicated in CardData (PassportNumber) with the same value
  String
      PassportType; // duplicated in CardData (PassportType) with the same value
  String PassportSequnceNo;
  String
      IssueDate; // passport issue date -> duplicated in CardData (PassportIssueDate) with the same value
  String
      ExpiryDate; // passport expiry date -> duplicated in CardData (PassportExpiryDate) with the same value
  String VisaNo;
  String VisaExpiryDate;
  String VisaType;
  String ResidentPermitNo;
  String ResidentPermitExpiryDate;
  String TypeOfResident;

  MiscellaneousTextData() {
    // nothing to do here
  }

  MiscellaneousTextData.json(Map json) {
    this.FirstNameArabic = json['FirstNameArabic'];
    this.LastNameArabic = json['LastNameArabic'];
    this.MiddleName1Arabic = json['MiddleName1Arabic'];
    this.MiddleName2Arabic = json['MiddleName2Arabic'];
    this.MiddleName3Arabic = json['MiddleName3Arabic'];
    this.MiddleName4Arabic = json['MiddleName4Arabic'];
    this.BloodGroup = json['BloodGroup'];
    this.CPRNO = json['CPRNO'];
    this.DateOfBirth = json['DateOfBirth'];
    this.FirstNameEnglish = json['FirstNameEnglish'];
    this.LastNameEnglish = json['LastNameEnglish'];
    this.MiddleName1English = json['MiddleName1English'];
    this.MiddleName2English = json['MiddleName2English'];
    this.MiddleName3English = json['MiddleName3English'];
    this.MiddleName4English = json['MiddleName4English'];
    this.Gender = json['Gender'];
    this.Email = json['Email'];
    this.ContactNo = json['ContactNo'];
    this.ResidenceNo = json['ResidenceNo'];
    this.FlatNo = json['FlatNo'];
    this.BuildingNo = json['BuildingNo'];
    this.BuildingAlpha = json['BuildingAlpha'];
    this.BuildingAlphaArabic = json['BuildingAlphaArabic'];
    this.RoadNo = json['RoadNo'];
    this.RoadName = json['RoadName'];
    this.RoadNameArabic = json['RoadNameArabic'];
    this.BlockNo = json['BlockNo'];
    this.BlockName = json['BlockName'];
    this.BlockNameArabic = json['BlockNameArabic'];
    this.GovernorateNo = json['GovernorateNo'];
    this.GovernorateNameEnglish = json['GovernorateNameEnglish'];
    this.GovernorateNameArabic = json['GovernorateNameArabic'];
    this.EmployerName1Arabic = json['EmployerName1Arabic'];
    this.LatestEducationDegreeArabic = json['LatestEducationDegreeArabic'];
    this.OccupationDescription1Arabic = json['OccupationDescription1Arabic'];
    this.SponsorNameArabic = json['SponsorNameArabic'];
    this.ClearingAgentIndicator = json['ClearingAgentIndicator'];
    this.EmployerFlag1 = json['EmployerFlag1'];
    this.EmployerName1 = json['EmployerName1'];
    this.EmployerNo1 = json['EmployerNo1'];
    this.EmploymentFlag1 = json['EmploymentFlag1'];
    this.LaborForceParticipation = json['LaborForceParticipation'];
    this.LatestEducationDegree = json['LatestEducationDegree'];
    this.OccupationDescription1 = json['OccupationDescription1'];
    this.SponsorCPRNoorUnitNo = json['SponsorCPRNoorUnitNo'];
    this.SponsorFlag = json['SponsorFlag'];
    this.SponsorName = json['SponsorName'];
    this.LfpNameEnglish = json['LfpNameEnglish'];
    this.LfpNameArabic = json['LfpNameArabic'];
    this.EnglishCountryName = json['EnglishCountryName'];
    this.ArabicCountryName = json['ArabicCountryName'];
    this.IACOCode = json['IACOCode'];
    this.Alpha2Code = json['Alpha2Code'];
    this.Alpha3Code = json['Alpha3Code'];
    this.Nationality = json['Nationality'];
    this.CountryOfBirth = json['CountryOfBirth'];
    this.PassportNo = json['PassportNo'];
    this.PassportType = json['PassportType'];
    this.PassportSequnceNo = json['PassportSequnceNo'];
    this.IssueDate = json['IssueDate'];
    this.ExpiryDate = json['ExpiryDate'];
    this.VisaNo = json['VisaNo'];
    this.VisaExpiryDate = json['VisaExpiryDate'];
    this.VisaType = json['VisaType'];
    this.ResidentPermitNo = json['ResidentPermitNo'];
    this.ResidentPermitExpiryDate = json['ResidentPermitExpiryDate'];
    this.TypeOfResident = json['TypeOfResident'];
  }

  String format(String date) {
    return DateUtils.formatDefault(CDate.parseNoErr(date, "yyyyMMdd"));
  }

  String getDateOfBirth() {
    return Utils.toBlank(format(DateOfBirth));
  }

  String getIssueDate() {
    return Utils.toBlank(format(IssueDate));
  }

  String getExpiryDate() {
    return Utils.toBlank(format(ExpiryDate));
  }

  @override
  String toString() =>
      Utils.toBlank(CPRNO) +
      DELIMITER +
      Utils.toBlank(Email) +
      DELIMITER +
      Utils.toBlank(ContactNo) +
      DELIMITER +
      Utils.escapeAndToBlank(ResidenceNo) +
      DELIMITER +
      Utils.escapeAndToBlank(FlatNo) +
      DELIMITER +
      Utils.escapeAndToBlank(BuildingNo) +
      DELIMITER +
      Utils.escapeAndToBlank(RoadNo) +
      DELIMITER +
      Utils.escapeAndToBlank(RoadName) +
      DELIMITER +
      Utils.escapeAndToBlank(BlockNo) +
      DELIMITER +
      Utils.escapeAndToBlank(BlockName) +
      DELIMITER +
      Utils.escapeAndToBlank(GovernorateNameEnglish) +
      DELIMITER +
      Utils.toBlank(ClearingAgentIndicator) +
      DELIMITER +
      Utils.toBlank(EmployerFlag1) +
      DELIMITER +
      Utils.escapeAndToBlank(EmployerName1) +
      DELIMITER +
      Utils.toBlank(EmployerNo1) +
      DELIMITER +
      Utils.toBlank(LaborForceParticipation) +
      DELIMITER +
      Utils.toBlank(SponsorCPRNoorUnitNo) +
      DELIMITER +
      Utils.escapeAndToBlank(LfpNameEnglish) +
      DELIMITER +
      Utils.toBlank(EnglishCountryName) +
      DELIMITER +
      Utils.toBlank(Alpha2Code) +
      DELIMITER +
      Utils.toBlank(Alpha3Code) +
      DELIMITER +
      Utils.toBlank(PassportSequnceNo);
}
