/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../validation/converter/c_date.dart';
import '../date_utils.dart';
import '../utils.dart';

class MrzBean {
  static const String DELIMITER = ",";

  // all possible date conversion formats should be listed here
  static const List<String> DATE_FORMATS = [
    "yyMMdd",
    "dd-MM-yyyy",
    Defaults.DD_MM_YYYY
  ];

  String documentType;
  String issuingState;
  String primaryIdentifier;
  String secondaryIdentifier;
  String documentNumber;
  String checkDigitForDocumentNumber;
  String extendedDocumentNumber;
  String nationality;
  String dateOfBirth;
  String checkDigitForDateOfBirth;
  String sex;
  String expiryDate;
  String checkDigitForExpiryDate;
  String optionalData;
  String checkDigitForOptionalData;
  String overallCheckDigit;
  String format;
  String fullname;

  // Extra fields NOT necessarily in MRZ
  String extra1;
  String extra2;
  String extra3;
  String extra4;
  String extra5;
  String extra6;
  String extra7;
  String extra8;
  String extra9;

  MrzBean() {
    // nothing to do here
  }

  String format_(String date) {
    return DateUtils.formatDefault(CDate.parseWithMany(date, DATE_FORMATS));
  }

  String getFullName() {
    return Utils.isBlank(fullname) || Utils.equalsIgnoreCase("null", fullname)
        ? Utils.toBlank(Utils.toBlank(secondaryIdentifier) +
            " " +
            Utils.toBlank(primaryIdentifier))
        : fullname;
  }

  @override
  String toString() =>
      Utils.toBlank(documentType) +
      DELIMITER +
      Utils.toBlank(issuingState) +
      DELIMITER +
      Utils.toBlank(primaryIdentifier) +
      DELIMITER +
      Utils.toBlank(secondaryIdentifier) +
      DELIMITER +
      Utils.toBlank(documentNumber) +
      DELIMITER +
      Utils.toBlank(checkDigitForDocumentNumber) +
      DELIMITER +
      Utils.toBlank(extendedDocumentNumber) +
      DELIMITER +
      Utils.toBlank(nationality) +
      DELIMITER +
      Utils.toBlank(format_(dateOfBirth)) +
      DELIMITER +
      Utils.toBlank(checkDigitForDateOfBirth) +
      DELIMITER +
      Utils.toBlank(sex) +
      DELIMITER +
      Utils.toBlank(format_(expiryDate)) +
      DELIMITER +
      Utils.toBlank(checkDigitForExpiryDate) +
      DELIMITER +
      Utils.toBlank(optionalData) +
      DELIMITER +
      Utils.toBlank(checkDigitForOptionalData) +
      DELIMITER +
      Utils.toBlank(overallCheckDigit) +
      DELIMITER +
      Utils.toBlank(format) +
      DELIMITER +
      getFullName() +
      DELIMITER +
      Utils.toBlank(extra1) +
      DELIMITER +
      Utils.toBlank(extra2) +
      DELIMITER +
      Utils.toBlank(extra3) +
      DELIMITER +
      Utils.toBlank(extra4) +
      DELIMITER +
      Utils.toBlank(extra5) +
      DELIMITER +
      Utils.toBlank(extra6) +
      DELIMITER +
      Utils.toBlank(extra7) +
      DELIMITER +
      Utils.toBlank(extra8) +
      DELIMITER +
      Utils.toBlank(extra9);
}
