/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../form/bean/i_reflect.dart';
import '../config/defaults.dart';
import '../error/no_such_field_exception.dart';
import '../ui/discrete/form_controller.dart';
import '../validation/converter/c_boolean.dart';
import '../validation/converter/c_date.dart';
import '../validation/converter/c_decimal.dart';
import '../validation/converter/c_number.dart';
import '../validation/converter/c_string.dart';
import 'log/logger.dart';
import 'utils.dart';

/// Provides utility methods that deals with java reflections.
// FIXME? 'import 'dart:mirrors';' isn't allowed in Flutter!!!
class BeanUtils {
  static const Type CLASS_LIST = List;
  static const Type CLASS_MAP = Map;

  const BeanUtils._();

  static T newInstance<T>(String clazz) {
    return Utils.isBlank(clazz) ? null : IReflect.newInstance(clazz);
  }

  /// Invokes declared method in the provided class with the given arguments.
  static Object invoke(String method, String clazz, [Object args]) {
    return invoke_(method, clazz, null, args);
  }

  /// Invokes the method with the provided arguments in the given class that should implement the given interface as a
  /// contract.
  ///
  /// @param method
  ///        Should only be the name of the method.
  /// @param clazz
  ///        Full name of the class with package.
  static T invoke_<T>(String method, String clazz, Type iClazz, [Object args]) {
    return null;
    // final ClassMirror beanClazz = findClass(clazz);
    //
    // if (iClazz != null && !isAssignableFrom(iClazz, beanClazz.runtimeType)) {
    //   throw IllegalArgumentException("$clazz should implement $iClazz");
    // }
    //
    // return invokeMethod(method, newInstance_(beanClazz), args) as T;
  }

  /// Invokes declared method in the provided bean with the given arguments.
  static Object invokeMethod(String name, Object bean, [Object args]) {
    // TODO Implement?
    // MethodMirror method = getMethod(name, bean, getArgsClasses(args));
    // method.setAccessible(true);
    // return method.invoke(bean, args);
    return null;
  }

  /// Returns the class types of all the arguments.
  static List<Type> getArgsClasses([Object args]) {
    if (Utils.isBlank(args)) {
      return null;
    }

    List<Type> classes = [];
    int index = Defaults.MISSING;

    for (Object arg in args) {
      Type clazz = arg.runtimeType;

      // if we don't do this then Java would look for exact same controller class that was passed from renderer.
      if (arg is FormController) {
        clazz = FormController;
      } else if (isMap(arg)) {
        clazz = CLASS_MAP; // override Map's subclass to Map
      } else if (arg is Exception) {
        clazz = Exception; // override Exception's subclass to Exception
      }

      classes[++index] = clazz;
    }

    return classes;
  }

  /// Returns the method of the given class.
  static Object getMethod(String name, Object bean, List<Type> args) {
    return null;
    // TODO Use args? Dart don't have overloading
    // return reflect(bean).type.declarations[name] as MethodMirror;
  }

  /// Finds the getter method of the given field in the mentioned class.
  static dynamic getFieldGetter(var field, Type clazz) {
    try {
      // TODO Implement?
      // const String prefix = CBoolean.isBoolean(field.runtimeType) ? "is" : "get";
      // const String name = field.getName();
      //
      // prefix with 'get' and capitalize the first the character.
      // return clazz.getMethod(
      //     prefix + name[0].toUpperCase() + name.substring(1));
      return null;
    } catch (e) {
      return null;
    }
  }

  /// Get all the fields of the class at all levels excluding static ones.
  static Set<String> getFields(Object bean) {
    return bean is IReflect ? bean.getFields() : {};
  }

  /// Filters all the the static fields and add remaining to the given collection.
  static void addNonStaticFields(List<dynamic> declared, List<dynamic> fields) {
    if (!Utils.isBlank(declared)) {
      // TODO Implement?
      // for (Mirror field in declared) {
      //   if (!Modifier.isStatic(field.getModifiers())) {
      //     fields.add(field);
      //   }
      // }
    }
  }

  /// Returns the field for the attribute belonging to the provided bean.
  static String getField(String name, Object bean) {
    bool hasField = bean is IReflect ? bean.getFields().contains(name) : null;

    if (!hasField) {
      throw NoSuchFieldException("Field Missing: $name");
    }

    return name;
  }

  /// Returns the field value for the attribute belonging to the provided bean.
  static Object getFieldValue(String name, Object bean) {
    return bean is IReflect ? bean.getFieldValue(name) : null;
  }

  static Type getGenericType(var field) {
    Type type = field.runtimeType;

    // TODO Implement?
    // if (type.isArray()) {
    //   return type.getComponentType();
    // }
    //
    // return getParameterizedType(field)[0];
    return type;
  }

  static List<Type> getParameterizedType(var field) {
    // Type fieldType = field.runtimeType;
    //
    // // TODO This is just a workaround to deal with super-super generics
    // if (StringObjectMap == fieldType || OrderedMap == fieldType) {
    //   return [CString.CLAZZ, Object];
    // }
    //
    // ParameterizedType type = null;
    //
    // try {
    //   type = field.getGenericType() as ParameterizedType;
    // } catch (e1) {
    //   try {
    //     type = fieldType.getGenericSuperclass() as ParameterizedType;
    //   } catch (e2) {
    //     // nothing to do here
    //   }
    // }
    //
    // return type == null ? [Object
    // , Object] :
    // type.getActualTypeArguments();
    return [Object, Object];
  }

  /// Assigns the attribute's value in the provided bean.
  static void setFieldValue(Object value, String field, Object bean) {
    if (bean is IReflect) {
      bean.setFieldValue(field, value);
    }
  }

  static void setFieldValueNoErr(Object value, String field, Object bean) {
    try {
      setFieldValue(value, field, bean);
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
    }
  }

  /// If given class is not of primitive or simple data type like int, String, Date etc.
  static bool isComplexType(Object object) {
    return object != null &&
        !CBoolean.checkType(object) &&
        !CNumber.checkType(object) &&
        !CDecimal.checkType(object) &&
        !CString.checkType(object) &&
        !CDate.checkType(object);
  }

  static bool isMap(Object object) {
    return object is Map;
  }

  static bool isList(Object object) {
    return object is List;
  }

  static String getName(Object object, bool useXmlTag) {
    if (useXmlTag && object is IReflect) {
      final String xmlTag = object.getXmlTag();

      if (!Utils.isBlank(xmlTag)) {
        return xmlTag;
      }
    }

    return getName_(object.runtimeType);
  }

  static String getName_(Type clazz) {
    return clazz.toString();
    // return reflectType(clazz).simpleName.toString();
  }

  static bool checkForClass(String clazz) {
    try {
      return forName(clazz) != null;
    } catch (e) {
      return false;
    }
  }

  // NOTE: Below functions are NOT in CMS Java

  static bool isAssignableFrom(Type typeA, Type typeB) {
    return typeA == typeB;
    // return typeA == typeB ||
    //     reflectClass(typeB).isAssignableTo(reflectType(typeA));
  }

  static Type forName(String className) {
    return null;
    // ClassMirror classMirror = findClass(className);
    // return classMirror == null ? null : classMirror.runtimeType;
  }

  static dynamic findClass(String className) {
    return null;
    // for (LibraryMirror lib in currentMirrorSystem().libraries.values) {
    //   DeclarationMirror mirror =
    //       lib.declarations[MirrorSystem.getSymbol(className)];
    //
    //   if (mirror != null) {
    //     return mirror;
    //   }
    // }
    //
    // throw IllegalArgumentException("Class $className does not exist");
  }
}
