/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../enum.dart';
import '../enum_utils.dart';
import '../utils.dart';

// TODO Load IV and the key from some secure place
class Algorithm extends Enum<Algorithm> {
  // Key should strictly be 16 bytes
  static const AES128 =
      Algorithm("AES", "AES", null, "1234567890123456", 16, "AES128");
  static const AES256 = Algorithm("AES", "AES/CBC/PKCS5Padding",
      "1234567890123456", "1234567890123456", 16, "AES256");

  // Key should strictly be 8 bytes
  static const DES128 = Algorithm("DES", "DES", null, "Passw0rd", 8, "DES128");
  static const DES256 =
      Algorithm("DES", "DES/ECB/PKCS5Padding", null, "Passw0rd", 8, "DES256");

  static const BLOWFISH =
      Algorithm("Blowfish", "Blowfish", null, "Blowfish", 8, "BLOWFISH");

  final String shortName;
  final String fullName;
  final String iv;
  final String key;
  final int minLength;

  const Algorithm(this.shortName, this.fullName, this.iv, this.key,
      this.minLength, String name)
      : super(name);

  String truncate(String key) {
    return Utils.isBlank(key) || key.length < minLength
        ? this.key
        : key.substring(0, minLength);
  }

  static Algorithm toAlgorithm(String algorithm, bool throwOnMismatch) {
    return EnumUtils.toEnum_(algorithm, _ENUMS, throwOnMismatch);
  }

  static const List<Algorithm> _ENUMS = [
    AES128,
    AES256,
    DES128,
    DES256,
    BLOWFISH
  ];
}
