/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/bean/register/crm_form_data.dart';
import '../../../form/structure/string_object_map.dart';
import '../../enums/code.dart';
import '../../error/illegal_argument_exception.dart';
import '../../generator/reader/json_reader.dart';
import '../log/logger.dart';
import '../stop_watch.dart';
import '../utils.dart';
import 'algorithm.dart';
import 'cypher.dart';
import 'md5.dart';

class Cryptor {
  static const int DEFAULT = 0x11;

  // TODO In Java, this is AES256, support it?
  static final Algorithm CIPHER = Algorithm.DES128;
  static final String SYS_DATA = Code.SYS.prefix("DATA");
  static const String SUM = "SUM";

  final Cypher cypher;

  Cryptor() : this.key(null);

  Cryptor.key(String key) : this.full(key, null);

  Cryptor.full(String key, String iv) : cypher = Cypher.full(CIPHER, iv, key);

  String encrypt(String text) {
    try {
      return MD5.toHex__(cypher.encrypt(text));
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
      return null;
    }
  }

  String decrypt(String text) {
    return Utils.toString_(cypher.decrypt_(MD5.toBytes(text)));
  }

  String decryptNoErr(String text) {
    try {
      return decrypt(text);
    } catch (e) {
      return text;
    }
  }

  String generateHash(String sTxId, String sMsisdn, String sKit) {
    // int txId = parseBigInt(sTxId);
    // int msisdn = parseLong(sMsisdn);
    // int kit = parseLong(sKit);
    // int result = txId * msisdn * kit;

    StringBuffer builder = StringBuffer();
    // FIXME? This function in CMS Java is not used anywhere
    // builder.write(MD5.digestAndFormat(result.toString())).append(MD5.getRandomHex(7)); // add some spam at the end
    return builder.toString();
  }

  String generateHash_(CrmFormData data) {
    return generateHash(data.getTrxId(), data.getMsisdn(), data.getIccid());
  }

  int parseBigInt(String text) {
    return Utils.isBlank(text) ? DEFAULT : int.parse(removeDigits(text));
  }

  int parseLong(String text) {
    if (Utils.isBlank(text)) {
      return DEFAULT;
    }

    final String value = removeDigits(text);
    return int.parse(value.length > 18
        ? value.substring(0, 18)
        : value); // max long is 19 chars
  }

  String removeDigits(String text) {
    return text.replaceAll("\\D+", "");
  }

  CrmFormData pack(CrmFormData data) {
    try {
      StopWatch stopWatch = StopWatch.start();
      CrmFormData packed = CrmFormData.copy(data);
      StringObjectMap unpackedMap = data.getMap();
      StringObjectMap toEncrypt = packed.getMap();
      StringObjectMap result = StringObjectMap();

      // Remove everything from the copy, which is not a file path
      for (String key in unpackedMap.keys) {
        final Object value = unpackedMap[key];

        if (SUM != key) {
          // TODO - Not used!
          result.put(key, value);
        }

        toEncrypt.remove(key); // Remove already created hash
      }

      // Convert the map to JSON, and encrypt
      final String json = JsonReader().toJson(toEncrypt);
      Logger.info(json);
      Logger.info("data length: ${json.length}");

      final String encryptedMap = encrypt(json);

      if (Utils.isBlank(encryptedMap)) {
        // TODO what to do if this fails?
        return data; // Return original map?
      }

      Logger.info("encrypted data length: ${encryptedMap.length}");

      result.put(SYS_DATA, encryptedMap); // put the encrypted map into the map
      // Get our generateHash
      // result.put(SUM, generateHash(unpacked)); // TODO - no signature for now

      packed.setMap(result); // set the map in the CrmFormData
      StopWatch.stop(stopWatch);
      return packed;
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
      return data;
    }
  }

  static void pack_(Map<String, Object> args, String license) {
    if (!Utils.isBlank(args)) {
      final bool noLicense = Utils.isBlank(license);

      for (String key in args.keys) {
        final Object value = args[key];

        if (value is CrmFormData) {
          CrmFormData formData = value;
          final String trxId = formData.getTrxId();
          args[key] =
              Cryptor.key(noLicense ? trxId : license + trxId).pack(formData);
          break;
        }
      }
    }
  }

  CrmFormData unpack(CrmFormData data) {
    if (data == null) {
      throw IllegalArgumentException("CrmFormData can't be null");
    }

    StringObjectMap packedMap = data.getMap();

    if (!Utils.got(SYS_DATA, packedMap)) {
      throw IllegalArgumentException("map missing $SYS_DATA");
    }

    String json = decrypt(packedMap.getAsString(SYS_DATA));
    StringObjectMap decryptedMap = JsonReader().fromJson(json, StringObjectMap);
    packedMap.remove(SYS_DATA); // remove the encrypted portion
    decryptedMap.addAll(packedMap); // add all images
    data.setMap(decryptedMap);
    return data;
  }

  CrmFormData unpackNoErr(CrmFormData data) {
    try {
      return unpack(data);
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
      return null;
    }
  }
}
