/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

/// Credit: https://pub.dev/packages/dart_des
/// We had to alter it a lot because original package is too slow.
class DES {
  static const int _blockSize = 8;

  // Permutation and translation tables for DES
  static final Uint8List _pc1 = Uint8List.fromList([
    56,
    48,
    40,
    32,
    24,
    16,
    8,
    0,
    57,
    49,
    41,
    33,
    25,
    17,
    9,
    1,
    58,
    50,
    42,
    34,
    26,
    18,
    10,
    2,
    59,
    51,
    43,
    35,
    62,
    54,
    46,
    38,
    30,
    22,
    14,
    6,
    61,
    53,
    45,
    37,
    29,
    21,
    13,
    5,
    60,
    52,
    44,
    36,
    28,
    20,
    12,
    4,
    27,
    19,
    11,
    3
  ]);

  // number left rotations of pc1
  static final Uint8List _leftRotations =
      Uint8List.fromList([1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1]);

  // permuted choice key (table 2)
  static final Uint8List _pc2 = Uint8List.fromList([
    13,
    16,
    10,
    23,
    0,
    4,
    2,
    27,
    14,
    5,
    20,
    9,
    22,
    18,
    11,
    3,
    25,
    7,
    15,
    6,
    26,
    19,
    12,
    1,
    40,
    51,
    30,
    36,
    46,
    54,
    29,
    39,
    50,
    44,
    32,
    47,
    43,
    48,
    38,
    55,
    33,
    52,
    45,
    41,
    49,
    35,
    28,
    31
  ]);

  // initial permutation IP
  static final Uint8List _ip = Uint8List.fromList([
    57,
    49,
    41,
    33,
    25,
    17,
    9,
    1,
    59,
    51,
    43,
    35,
    27,
    19,
    11,
    3,
    61,
    53,
    45,
    37,
    29,
    21,
    13,
    5,
    63,
    55,
    47,
    39,
    31,
    23,
    15,
    7,
    56,
    48,
    40,
    32,
    24,
    16,
    8,
    0,
    58,
    50,
    42,
    34,
    26,
    18,
    10,
    2,
    60,
    52,
    44,
    36,
    28,
    20,
    12,
    4,
    62,
    54,
    46,
    38,
    30,
    22,
    14,
    6
  ]);

  // Expansion table for turning 32 bit blocks into 48 bits
  static final Uint8List _expansionTable = Uint8List.fromList([
    31,
    0,
    1,
    2,
    3,
    4,
    3,
    4,
    5,
    6,
    7,
    8,
    7,
    8,
    9,
    10,
    11,
    12,
    11,
    12,
    13,
    14,
    15,
    16,
    15,
    16,
    17,
    18,
    19,
    20,
    19,
    20,
    21,
    22,
    23,
    24,
    23,
    24,
    25,
    26,
    27,
    28,
    27,
    28,
    29,
    30,
    31,
    0
  ]);

  // The (in)famous S-boxes
  static final List<Uint8List> _sBox = [
    // S1
    Uint8List.fromList([
      14,
      4,
      13,
      1,
      2,
      15,
      11,
      8,
      3,
      10,
      6,
      12,
      5,
      9,
      0,
      7,
      0,
      15,
      7,
      4,
      14,
      2,
      13,
      1,
      10,
      6,
      12,
      11,
      9,
      5,
      3,
      8,
      4,
      1,
      14,
      8,
      13,
      6,
      2,
      11,
      15,
      12,
      9,
      7,
      3,
      10,
      5,
      0,
      15,
      12,
      8,
      2,
      4,
      9,
      1,
      7,
      5,
      11,
      3,
      14,
      10,
      0,
      6,
      13
    ]),

    // S2
    Uint8List.fromList([
      15,
      1,
      8,
      14,
      6,
      11,
      3,
      4,
      9,
      7,
      2,
      13,
      12,
      0,
      5,
      10,
      3,
      13,
      4,
      7,
      15,
      2,
      8,
      14,
      12,
      0,
      1,
      10,
      6,
      9,
      11,
      5,
      0,
      14,
      7,
      11,
      10,
      4,
      13,
      1,
      5,
      8,
      12,
      6,
      9,
      3,
      2,
      15,
      13,
      8,
      10,
      1,
      3,
      15,
      4,
      2,
      11,
      6,
      7,
      12,
      0,
      5,
      14,
      9
    ]),

    // S3
    Uint8List.fromList([
      10,
      0,
      9,
      14,
      6,
      3,
      15,
      5,
      1,
      13,
      12,
      7,
      11,
      4,
      2,
      8,
      13,
      7,
      0,
      9,
      3,
      4,
      6,
      10,
      2,
      8,
      5,
      14,
      12,
      11,
      15,
      1,
      13,
      6,
      4,
      9,
      8,
      15,
      3,
      0,
      11,
      1,
      2,
      12,
      5,
      10,
      14,
      7,
      1,
      10,
      13,
      0,
      6,
      9,
      8,
      7,
      4,
      15,
      14,
      3,
      11,
      5,
      2,
      12
    ]),

    // S4
    Uint8List.fromList([
      7,
      13,
      14,
      3,
      0,
      6,
      9,
      10,
      1,
      2,
      8,
      5,
      11,
      12,
      4,
      15,
      13,
      8,
      11,
      5,
      6,
      15,
      0,
      3,
      4,
      7,
      2,
      12,
      1,
      10,
      14,
      9,
      10,
      6,
      9,
      0,
      12,
      11,
      7,
      13,
      15,
      1,
      3,
      14,
      5,
      2,
      8,
      4,
      3,
      15,
      0,
      6,
      10,
      1,
      13,
      8,
      9,
      4,
      5,
      11,
      12,
      7,
      2,
      14
    ]),

    // S5
    Uint8List.fromList([
      2,
      12,
      4,
      1,
      7,
      10,
      11,
      6,
      8,
      5,
      3,
      15,
      13,
      0,
      14,
      9,
      14,
      11,
      2,
      12,
      4,
      7,
      13,
      1,
      5,
      0,
      15,
      10,
      3,
      9,
      8,
      6,
      4,
      2,
      1,
      11,
      10,
      13,
      7,
      8,
      15,
      9,
      12,
      5,
      6,
      3,
      0,
      14,
      11,
      8,
      12,
      7,
      1,
      14,
      2,
      13,
      6,
      15,
      0,
      9,
      10,
      4,
      5,
      3
    ]),

    // S6
    Uint8List.fromList([
      12,
      1,
      10,
      15,
      9,
      2,
      6,
      8,
      0,
      13,
      3,
      4,
      14,
      7,
      5,
      11,
      10,
      15,
      4,
      2,
      7,
      12,
      9,
      5,
      6,
      1,
      13,
      14,
      0,
      11,
      3,
      8,
      9,
      14,
      15,
      5,
      2,
      8,
      12,
      3,
      7,
      0,
      4,
      10,
      1,
      13,
      11,
      6,
      4,
      3,
      2,
      12,
      9,
      5,
      15,
      10,
      11,
      14,
      1,
      7,
      6,
      0,
      8,
      13
    ]),

    // S7
    Uint8List.fromList([
      4,
      11,
      2,
      14,
      15,
      0,
      8,
      13,
      3,
      12,
      9,
      7,
      5,
      10,
      6,
      1,
      13,
      0,
      11,
      7,
      4,
      9,
      1,
      10,
      14,
      3,
      5,
      12,
      2,
      15,
      8,
      6,
      1,
      4,
      11,
      13,
      12,
      3,
      7,
      14,
      10,
      15,
      6,
      8,
      0,
      5,
      9,
      2,
      6,
      11,
      13,
      8,
      1,
      4,
      10,
      7,
      9,
      5,
      0,
      15,
      14,
      2,
      3,
      12
    ]),

    // S8
    Uint8List.fromList([
      13,
      2,
      8,
      4,
      6,
      15,
      11,
      1,
      10,
      9,
      3,
      14,
      5,
      0,
      12,
      7,
      1,
      15,
      13,
      8,
      10,
      3,
      7,
      4,
      12,
      5,
      6,
      11,
      0,
      14,
      9,
      2,
      7,
      11,
      4,
      1,
      9,
      12,
      14,
      2,
      0,
      6,
      10,
      13,
      15,
      3,
      5,
      8,
      2,
      1,
      14,
      7,
      4,
      10,
      8,
      13,
      15,
      12,
      9,
      0,
      3,
      5,
      6,
      11
    ])
  ];

  // 32-bit permutation function P used on the output of the S-boxes
  static final Uint8List _p = Uint8List.fromList([
    15,
    6,
    19,
    20,
    28,
    11,
    27,
    16,
    0,
    14,
    22,
    25,
    4,
    17,
    30,
    9,
    1,
    7,
    23,
    13,
    31,
    26,
    2,
    8,
    18,
    12,
    29,
    5,
    21,
    10,
    3,
    24
  ]);

  // final permutation IP^-1
  static final Uint8List _fp = Uint8List.fromList([
    39,
    7,
    47,
    15,
    55,
    23,
    63,
    31,
    38,
    6,
    46,
    14,
    54,
    22,
    62,
    30,
    37,
    5,
    45,
    13,
    53,
    21,
    61,
    29,
    36,
    4,
    44,
    12,
    52,
    20,
    60,
    28,
    35,
    3,
    43,
    11,
    51,
    19,
    59,
    27,
    34,
    2,
    42,
    10,
    50,
    18,
    58,
    26,
    33,
    1,
    41,
    9,
    49,
    17,
    57,
    25,
    32,
    0,
    40,
    8,
    48,
    16,
    56,
    24
  ]);

  final Uint8List _key;

  // 16 48-bit keys (K1 - K16)
  final List<Uint8List> _kN = List.filled(16, Uint8List(48), growable: false);

  DES(this._key) {
    if (_key.length != _blockSize) {
      throw Exception(
          'Invalid DES key size. Key must be exactly $_blockSize bytes long.');
    }

    _createSubKeys();
  }

  Uint8List encrypt(Uint8List data) => _crypt(_pkcs7Pad(data), true);

  Uint8List decrypt(Uint8List data) => _pkcs7Unpad(_crypt(data, false));

  Uint8List _convertToBits(Uint8List data) {
    List<int> result = [];
    data.forEach((e) => result += e
        .toRadixString(2)
        .padLeft(_blockSize, '0')
        .codeUnits
        .map((el) => el - 48)
        .toList());
    return Uint8List.fromList(result);
  }

  Uint8List _convertBitsToIntList(Uint8List data) {
    List<int> result = [];
    for (int position = 0, c = 0; position < data.length; position++) {
      c += data[position] << (7 - (position % _blockSize));
      if (position % _blockSize == 7) {
        result.add(c);
        c = 0;
      }
    }
    return Uint8List.fromList(result);
  }

  // Permutate this block with the specified table
  Uint8List _permutate(Uint8List table, Uint8List block) =>
      Uint8List.fromList(table.map((e) => block[e]).toList());

  // Transform the secret key, so that it is ready for data processing
  // Create the 16 subkeys, K[1] - K[16]
  void _createSubKeys() {
    // Create the 16 subkeys K[1] to K[16] from the given key
    Uint8List key = _permutate(_pc1, _convertToBits(_key));

    // Split into Left and Right sections
    List<int> left = List.from(key.sublist(0, 28));
    List<int> right = List.from(key.sublist(28, key.length));
    for (int i = 0; i < _kN.length; i++) {
      // Perform circular left shifts
      for (int j = 0; j < _leftRotations[i]; j++) {
        left.add(left[0]);
        left.removeAt(0);
        right.add(right[0]);
        right.removeAt(0);
      }

      // Create one of the 16 subkeys through pc2 permutation
      _kN[i] = _permutate(_pc2, Uint8List.fromList(left + right));
    }
  }

  Uint8List _xor(Uint8List a, Uint8List b) {
    // TODO check lists length
    Uint8List result = Uint8List(a.length);
    for (int i = 0; i < a.length; i++) {
      result[i] = a[i] ^ b[i];
    }
    return result;
  }

  Uint8List _desCrypt(Uint8List block, bool encrypt) {
    block = _permutate(_ip, block);
    Uint8List left = block.sublist(0, 32);
    Uint8List right = block.sublist(32, block.length);

    // Encryption starts from Kn[1] through to Kn[16]
    // Decryption starts from Kn[16] down to Kn[1]
    int iteration = encrypt ? 0 : 15;
    int iterationAdjustment = encrypt ? 1 : -1;

    for (int i = 0; i < _kN.length; i++, iteration += iterationAdjustment) {
      // Make a copy of R[i-1], this will later become L[i]
      Uint8List tempRight = right;

      // Permutate R[i - 1] to start creating R[i]
      right = _permutate(_expansionTable, right);

      // Exclusive or R[i - 1] with K[i], create B[1] to B[8] whilst here
      right = _xor(right, _kN[iteration]);
      List<Uint8List> B = [
        right.sublist(0, 6),
        right.sublist(6, 12),
        right.sublist(12, 18),
        right.sublist(18, 24),
        right.sublist(24, 30),
        right.sublist(30, 36),
        right.sublist(36, 42),
        right.sublist(42)
      ];
      Uint8List bN = Uint8List(32);
      for (int j = 0, position = 0; j < _blockSize; j++, position += 4) {
        // Work out the offsets
        int m = (B[j][0] << 1) + B[j][5];
        int n = (B[j][1] << 3) + (B[j][2] << 2) + (B[j][3] << 1) + B[j][4];

        // Find the permutation value
        int v = _sBox[j][(m << 4) + n];

        // Turn value into bits, add it to result: Bn
        bN[position] = (v & 8) >> 3;
        bN[position + 1] = (v & 4) >> 2;
        bN[position + 2] = (v & 2) >> 1;
        bN[position + 3] = v & 1;
      }

      // Permutate the concatenation of B[1] to B[8] (Bn)
      right = _permutate(_p, bN);

      // Xor with L[i - 1]
      right = _xor(right, left);

      left = tempRight;
    }

    // Final permutation of R[16]L[16]
    return _permutate(_fp, Uint8List.fromList(right + left));
  }

  Uint8List _crypt(Uint8List data, bool encrypt) {
    // Error check the data
    if (data.length == 0) {
      return Uint8List(0);
    }
    if (data.length % _blockSize != 0 && !encrypt) {
      // Decryption must work on 8 byte blocks
      throw Exception(
          'Invalid data length, data must be a multiple of $_blockSize bytes.');
    }

    // Split the data into blocks, crypting each one separately
    int i = 0;
    Uint8List result = Uint8List(data.length);
    while (i < data.length) {
      Uint8List block = _convertToBits(data.sublist(i, i + _blockSize));
      block = _desCrypt(block, encrypt); // processedBlock
      result.setRange(i, i + _blockSize, _convertBitsToIntList(block));
      i += _blockSize;
    }

    return result;
  }

  static Uint8List _pkcs7Pad(Uint8List data) {
    final int left = _blockSize - (data.length % _blockSize);
    final List<int> padding = List.filled(left, left, growable: false);
    return Uint8List.fromList(data + padding.sublist(0, left));
  }

  static Uint8List _pkcs7Unpad(Uint8List data) {
    Iterable<int> reversed = data.reversed;
    final int paddingSize = reversed.first;
    for (int i = 0; i < paddingSize; i++) {
      if (reversed.elementAt(i) != paddingSize) return data;
    }
    return data.sublist(0, data.length - paddingSize);
  }
}
