/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../log/logger.dart';
import '../utils.dart';
import 'crypto/crypto_digest.dart';
import 'crypto/crypto_md5.dart';

class MD5 {
  static const String hex = "0123456789ABCDEF";
  static const int hexLength = hex.length;

  static const String salt = "Hc3t3l1b0Ml3tgn1S#@!";

  const MD5._();

  static String hash(String text) => hash_(Utils.toBytes(text));

  static String hashWithSalt(String text) => hash(salt + text);

  static String hash_(List<int> content) {
    try {
      return md5.convert(content).toString();
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
      return null;
    }
  }

  static List<int> digest(String text) => digest_(Utils.toBytes(text));

  static List<int> digest_(List<int> content) {
    try {
      return md5.convert(content).bytes;
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
      return null;
    }
  }

  static String toHex(int code) =>
      "${hex[code ~/ hexLength]}${hex[code % hexLength]}";

  static String toHex_(String text) =>
      Utils.isBlank(text) ? null : toHex__(text.codeUnits);

  static String fromHex(String text) =>
      Utils.isBlank(text) ? "" : String.fromCharCodes(toBytes(text));

  static List<int> toBytes(String text) {
    if (Utils.isBlank(text)) {
      return null;
    }

    final int length = text.length ~/ 2;
    const List<int> result = [];

    for (int i = 0; i < length; i++) {
      result[i] = int.parse(text.substring(2 * i, 2 * i + 2), radix: hexLength);
    }

    return result;
  }

  static String toHex__(List<int> content) =>
      content == null ? "" : Digest(content).toString();
}
