/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../enums/code.dart';
import 'enum.dart';
import 'utils.dart';

class EnumUtils {
  const EnumUtils._();

  static E toEnum<E extends Enum<E>>(String code, List<E> enums) {
    return toEnum_(code, enums, false);
  }

  static E toEnum_<E extends Enum<E>>(
      String code, List<E> enums, bool throwOnMismatch) {
    if (!Utils.isBlank(code)) {
      final String enumCode = Code.SYS.remove(code.trim());

      for (E e in enums) {
        if (Utils.equalsIgnoreCase(e.name(), enumCode)) {
          return e;
        }
      }

      if (throwOnMismatch) {
        throw 'Unknown value: $code';
      }
    }

    return null;
  }

  static bool match(String match, List<Enum> candidates) {
    if (!Utils.isBlank(match) && !Utils.isBlank(candidates)) {
      final String tMatch = match.trim();

      for (Enum candidate in candidates) {
        if (Utils.equalsIgnoreCase(candidate.name(), tMatch)) {
          return true;
        }
      }
    }

    return false;
  }

  static bool match_(Enum match, List<Enum> candidates) {
    if (match != null && !Utils.isBlank(candidates)) {
      for (Enum candidate in candidates) {
        if (candidate == match) {
          return true;
        }
      }
    }

    return false;
  }

  static bool isEnum(Object object) {
    if (object == null) {
      return false;
    }

    if (object is Enum) {
      return true;
    }

    final List<String> split = object.toString().split('.');
    return split.length > 1 && split[0] == object.runtimeType.toString();
  }

  static String getName(dynamic value) =>
      value == null ? null : value.toString().split(".").last;
}
