/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

class LoggerStackTrace {
  final String functionName;
  final String callerFunctionName;
  final String fileName;
  final int lineNumber;
  final int columnNumber;

  const LoggerStackTrace._(this.functionName, this.callerFunctionName,
      this.fileName, this.lineNumber, this.columnNumber);

  factory LoggerStackTrace.from(StackTrace trace) {
    final List<String> frames = [];

    // Ignore all frames related to StopWatch class
    for (String frame in trace.toString().split('\n')) {
      if (frame.startsWith("#") && !frame.contains("StopWatch.")) {
        frames.add(frame);
      }
    }

    final List<String> fileInfo = _getFileInfoFromFrame(frames[0]);

    return LoggerStackTrace._(
      _getFunctionNameFromFrame(frames[0]),
      frames.length < 2 ? '' : _getFunctionNameFromFrame(frames[1]),
      fileInfo[0],
      int.parse(fileInfo[1]),
      int.parse(fileInfo[2].replaceFirst(')', '')),
    );
  }

  static List<String> _getFileInfoFromFrame(String trace) {
    return trace.substring(trace.indexOf(RegExp('[A-Za-z_]+.dart'))).split(':');
  }

  static String _getFunctionNameFromFrame(String trace) {
    String subStr = trace.substring(trace.indexOf(' '));
    subStr = subStr.substring(subStr.indexOf(RegExp('[A-Za-z0-9]')));
    final int spaceIndex = subStr.indexOf(' ');
    return spaceIndex == -1 ? subStr : subStr.substring(0, spaceIndex);
  }

  @override
  String toString() =>
      '$fileName($callerFunctionName:$functionName:$lineNumber:$columnNumber)';
}
