/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:collection' show HashSet;

import '../../form/structure/ordered_map.dart';
import '../../form/structure/t_map.dart';
import '../error/illegal_argument_exception.dart';
import '../structure/id_tag.dart';
import '../ui/discrete/component/behavior/i_map.dart';
import 'bean_utils.dart';
import 'utils.dart';

class MapUtils {
  const MapUtils._();

  static M toMap<T extends IdTag, M extends Map<String, T>>(
      List<T> list, bool ordered) {
    return toMap_(list, ordered, true);
  }

  static M toMap_<T extends IdTag, M extends Map<String, T>>(
      List<T> list, bool ordered, bool clear) {
    final int size = Utils.getSize(list);
    Map<String, T> map = ordered ? OrderedMap() : TMap();

    if (size > 0) {
      final bool allowSpace = list[0].allowSpace(); // just call it once

      for (T t in list) {
        t.validate(allowSpace);

        final String id = t.resolveId();

        if (map.containsKey(id)) {
          throw IllegalArgumentException(
              "${BeanUtils.getName(t, false)} '$id' is duplicated!");
        }

        t.toMap();
        map[id] = t;
      }

      if (clear) {
        Utils.clear([list]);
      }
    }

    return map;
  }

  static List<T> toList<K, T extends IMap>(Map<K, T> map, bool clear) {
    final int size = Utils.getSize(map);
    List<T> list = [];

    if (size > 0) {
      for (T t in map.values) {
        t.toList();
        list.add(t);
      }

      if (clear) {
        Utils.clear([map]);
      }
    }

    return list;
  }

  static bool allNullValued(Map map) {
    for (Object value in map.values) {
      if (value != null) {
        return false;
      }
    }

    return true;
  }

  static void checkDuplicates<K, V1, V2>(Map<K, V1> mapA, Map<K, V2> mapB) {
    if (!Utils.isBlank(mapA) && !Utils.isBlank(mapB)) {
      for (K key in mapA.keys) {
        if (mapB.containsKey(key)) {
          throw IllegalArgumentException(key.toString());
        }
      }
    }
  }

  static void isValidKey(String key, TMap map) {
    if (Utils.isBlank(key)) {
      throw IllegalArgumentException("Key can not be blank!");
    }

    if (map.containsKey(key)) {
      throw IllegalArgumentException(
          "'${map.trimAndToCase(key)}' is duplicated in the map!");
    }
  }

  static Set<String> keys(Map map) {
    return Utils.isBlank(map) ? null : HashSet.from(map.keys);
  }

  static dynamic findValueByKey(String key, Map<String, dynamic> map) {
    if (map != null) {
      for (String currentKey in map.keys) {
        dynamic value = map[currentKey];

        if (key == currentKey) {
          return value;
        }

        if (value is Map) {
          value = findValueByKey(key, value);

          if (value != null) {
            return value;
          }
        }
      }
    }

    return null;
  }
}
