/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:io' show HttpClient, HttpClientRequest, HttpClientResponse, IOSink;
import 'dart:typed_data' show Uint8List;

import '../../form/bean/enums/base_server_response_code.dart';
import '../enums/code.dart';
import '../ui/discrete/action/handler/rmi/mpos_http_transport_se.dart';
import '../ui/discrete/action/handler/rmi/transport.dart';
import 'io_utils.dart';
import 'log/logger.dart';
import 'utils.dart';

class Network {
  static const int connectionTimeout = 5; // seconds

  const Network._();

  static Future<bool> isReachable(String link) async {
    try {
      HttpClient httpClient = HttpClient();
      httpClient.connectionTimeout = Duration(seconds: connectionTimeout);
      // we can connect but since there is no valid certificate, we simply accept.
      httpClient.badCertificateCallback = (cert, host, port) => true;

      HttpClientRequest request = await httpClient.getUrl(Uri.parse(link));
      request.persistentConnection = false;

      HttpClientResponse response = await request.close();
      return response != null &&
          response.statusCode == BaseServerResponseCode.SUCCESS.getCode();
    } catch (e) {
      return false;
    }
  }

  // TODO Probably not needed?
  static String getMacAddress() {
    return null;
  }

  // TODO Probably not needed?
  static String getHostAddress(bool useIPv4) {
    return "127.0.0.1"; // TODO What's the default IPv6 address?
  }

  static bool isValidLink(String link) {
    try {
      return Utils.isURL(link);
    } catch (e) {
      return false;
    }
  }

  static Future<void> download(
      String url,
      int timeout,
      Uint8List certificate,
      String certificateKey,
      String savePath,
      Object progressData,
      Function(Object, double) onProgress) async {
    Transport transport = Transport.init(
        Code.HTTPS.starts(url), url, timeout, certificate, certificateKey,
        method: MposHttpTransportSE.REQ_METHOD_GET);
    final HttpClientResponse response = await transport.call__(null, null);
    // TODO If contentLength is -1 (unknown) then percentage won't be correctly calculated.
    final int length = response.contentLength;
    Logger.debug('Content length = $length');
    IOSink sink = IOUtils.getFile(savePath).openWrite();

    if (onProgress == null) {
      await response.pipe(sink);
    } else {
      int received = 0;

      await response.map((List<int> content) {
        received += content.length;
        onProgress(progressData, (received / length) * 100.0);
        return content;
      }).pipe(sink);
    }
  }
}
