/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'log/logger.dart';
import 'utils.dart';

class Numbers {
  static int NEXT_INT = 1;
  static const int NUMBER_RANGE = 10;

  const Numbers._();

  static T getMax<T extends num>(T a, T b) {
    if (a == b) {
      // checks for null as well
      return a;
    }

    a = Utils.toDefault(a, b);
    b = Utils.toDefault(b, a);
    return a.compareTo(b) > 0 ? a : b;
  }

  /// Convert non-English digits to English digits character by character.
  static String toDigitsCharByChar(String text) {
    if (!Utils.isBlank(text)) {
      try {
        final int length = text.length;
        StringBuffer en = StringBuffer();

        for (int i = 0; i < length; i++) {
          String c = text[i];
          // FIXME int.parse() can not convert non-English digits
          en.write(Utils.isDigit(c) ? int.parse(c) : c);
        }

        return en.toString();
      } catch (e, stackTrace) {
        Logger.error(e, stackTrace: stackTrace);
      }
    }

    return text;
  }

  /// Convert the internationalized digits to English digits.
  static String toEnDigits(String number) {
    if (Utils.isBlank(number)) {
      return number;
    }

    String temp = number.trim();

    for (int index = 0; index < temp.length; index++) {
      if (!Utils.isDigit(temp[index])) {
        return number;
      }
    }

    try {
      StringBuffer sb = StringBuffer();

      while (!Utils.isBlank(temp)) {
        int length = temp.length;
        String sub =
            length > NUMBER_RANGE ? temp.substring(0, NUMBER_RANGE) : temp;
        // FIXME num.parse() can not parse non-English digits
        String formatted = num.parse(sub).toString();

        while (formatted.length != sub.length) {
          formatted = "0$formatted";
        }

        sb.write(formatted);
        temp = length > NUMBER_RANGE ? temp.substring(NUMBER_RANGE) : "";
      }

      return sb.toString();
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
    }

    return number;
  }

  static int toZero(int number) {
    return Utils.toDefault(number, 0);
  }

  static bool notZero(int number) {
    return number != null && number != 0;
  }

  static bool belowOne(int number) {
    return number != null && number < 1;
  }

  static bool nullOrBelowOne(int number) {
    return number == null || number < 1;
  }

  static String prefixZero(int number) {
    return number < 10 ? "0$number" : number.toString();
  }

  static int nextInt() {
    for (;;) {
      final int nextNo = NEXT_INT;
      int newNo = nextNo + 1;

      if (newNo > 0x00FFFFFF) {
        // generated numbers have the high byte nonzero; clamp to the range under that
        newNo = 1; // Roll over to 1, not 0.
      }

      if (NEXT_INT == nextNo) {
        NEXT_INT = newNo;
        return nextNo;
      }
    }
  }
}
