/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/code.dart';
import '../../error/illegal_argument_exception.dart';
import '../../structure/model/script.dart';
import '../../ui/discrete/renderer/ui_renderer.dart';
import '../../validation/converter/c_boolean.dart';
import '../../validation/converter/conversion_error.dart';
import '../form_utils.dart';
import '../utils.dart';
import 'script_evaluator.dart';

class ConditionEvaluator {
  const ConditionEvaluator._();

  static bool evalExpr(
      String expression, Map<String, Object> args, UIRenderer renderer) {
    return eval___(Script.inline_(expression), args, renderer);
  }

  static bool eval(String scriptId, UIRenderer renderer) {
    return eval_(scriptId, null, renderer);
  }

  static bool eval_(String scriptId, bool defaultValue, UIRenderer renderer) {
    bool result =
        Utils.isBlank(scriptId) ? defaultValue : CBoolean.parseNoErr(scriptId);

    if (result == null) {
      // if scriptId was true/false, it won't be null.
      Script script = FormUtils.getScript(scriptId, renderer);
      result = script == null
          ? null
          : eval__(script, Code.NOT.starts(scriptId), renderer);
    }

    return Utils.toDefault(result, defaultValue);
  }

  static bool eval__(Script script, bool negate, UIRenderer renderer) {
    bool result = eval___(script, null, renderer);
    return negate && result != null && Script.notInline(script)
        ? !result
        : result;
  }

  static bool eval___(
      Script script, Map<String, Object> args, UIRenderer renderer) {
    final Object result = ScriptEvaluator.eval__(script, null, args, renderer);

    try {
      return CBoolean.convert(result);
    } on ConversionError {
      throw IllegalArgumentException("Result '$result' isn't boolean: $script");
    }
  }
}
