/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import 'package:flutter_js/javascript_runtime.dart' show JavascriptRuntime;

import '../base64.dart';
import '../cipher/cypher.dart';
import '../cipher/md5.dart';

// TODO No function is tested in here
class ELCypher {
  const ELCypher();

  void reg(String id, void Function(dynamic) fn, JavascriptRuntime engine,
      {bool ret = true}) {
    engine.setupBridge(id, fn);
    engine.evaluate(
        "var $id = function() { ${ret ? 'return' : ''} sendMessage('$id', argsToJson.apply(null, arguments)); };");
  }

  void register(JavascriptRuntime engine) {
    reg('encrypt', (args) => encrypt(args[0], args[1]), engine);
    reg('encrypt_', (args) => encrypt_(args[0], args[1], args[2], args[3]),
        engine);
    reg('encrypt__', (args) => encrypt__(args[0], args[1]), engine);
    reg('encrypt___', (args) => encrypt___(args[0], args[1], args[2], args[3]),
        engine);
    reg('decrypt', (args) => decrypt(args[0], args[1]), engine);
    reg('decrypt_', (args) => decrypt_(args[0], args[1], args[2], args[3]),
        engine);
    reg('decrypt__', (args) => decrypt__(args[0], args[1]), engine);
    reg('decrypt___', (args) => decrypt___(args[0], args[1], args[2], args[3]),
        engine);
    reg('hash', (args) => hash(args), engine);
    reg('hash_', (args) => hash_(args[0]), engine);
    reg('digest', (args) => digest(args), engine);
    reg('digest_', (args) => digest_(args[0]), engine);
    reg('fromHex', (args) => fromHex(args[0]), engine);
    reg('toHex', (args) => toHex(args[0]), engine);
    reg('toHex_', (args) => toHex_(args[0]), engine);
    reg('encode', (args) => encode(args[0]), engine);
    reg('decode', (args) => decode(args[0]), engine);
  }

  Uint8List encrypt(List<int> content, String algorithm) =>
      encrypt_(content, algorithm, null, null);

  Uint8List encrypt_(
          List<int> content, String algorithm, String iv, String key) =>
      Cypher.algo(algorithm, iv, key).encryptNoErr_(content);

  Uint8List encrypt__(String text, String algorithm) =>
      encrypt___(text, algorithm, null, null);

  Uint8List encrypt___(String text, String algorithm, String iv, String key) =>
      Cypher.algo(algorithm, iv, key).encryptNoErr(text);

  Uint8List decrypt(List<int> content, String algorithm) =>
      decrypt_(content, algorithm, null, null);

  Uint8List decrypt_(
          List<int> content, String algorithm, String iv, String key) =>
      Cypher.algo(algorithm, iv, key).decryptNoErr_(content);

  String decrypt__(String text, String algorithm) =>
      decrypt___(text, algorithm, null, null);

  String decrypt___(String text, String algorithm, String iv, String key) =>
      Cypher.algo(algorithm, iv, key).decryptNoErr(text);

  String hash(List<dynamic> content) => MD5.hash_(content);

  String hash_(String text) => MD5.hash(text);

  List<int> digest(List<dynamic> content) => MD5.digest_(content);

  List<int> digest_(String text) => MD5.digest(text);

  String fromHex(String text) => MD5.fromHex(text);

  String toHex(List<int> content) => MD5.toHex__(content);

  String toHex_(String text) => MD5.toHex_(text);

  String encode(List<int> content) => Base64.encode_(content);

  Uint8List decode(String content) => Base64.decode(content);
}
