/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../config/config.dart';
import 'date_utils.dart';
import 'log/logger.dart';
import 'log/logger_stack_trace.dart';
import 'utils.dart';

class StopWatch {
  static const Type CLAZZ = StopWatch;
  static final String NAME = CLAZZ.toString();
  static const int THRESHOLD = 5;

  /// Stores the start time when this class is initialized.
  final int startTime;

  StopWatch._() : this.startTime = DateUtils.epoch();

  static StopWatch start() {
    return Config.DEBUG ? StopWatch._() : null;
  }

  static void stop(StopWatch stopWatch) {
    stop_(null, stopWatch);
  }

  static void stop_(Object param, StopWatch stopWatch) {
    if (stopWatch != null) {
      try {
        final int elapsedTime = stopWatch.elapsedTime();

        // ignore anything done within this time
        if (elapsedTime > THRESHOLD) {
          String sParam = Utils.toString_O(param);
          String paramPart = Utils.isBlank(sParam) ? "" : ", Param: $sParam";
          Logger.debug("${getCallee()} took ${elapsedTime}ms$paramPart");
        }
      } catch (e, stackTrace) {
        Logger.error(e, stackTrace: stackTrace);
      }
    }
  }

  static String getCallee() {
    try {
      return LoggerStackTrace.from(StackTrace.current).toString();
    } catch (e, stackTrace) {
      Logger.error(e, stackTrace: stackTrace);
      return "?";
    }
  }

  int getStartTime() {
    return this.startTime;
  }

  /// Gets the elapsed time since the time this class was initialized.
  int elapsedTime() {
    return DateUtils.epoch() - startTime;
  }
}
