/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:collection' show LinkedHashSet;

import '../utils.dart';

// Case Insensitive Linked String Set
class StringSet implements LinkedHashSet<String> {
  final LinkedHashSet<String> set;

  StringSet() : this.set = LinkedHashSet();

  StringSet.list(List<String> list) : this.set = LinkedHashSet() {
    // we can't use LinkedHashSet.from() or LinkedHashSet.of()
    // as stupid Dart doesn't call add() or addAll() for each item (it should)
    addAll(list);
  }

  @override
  bool add(String value) {
    return !Utils.isBlank(value) && set.add(Utils.trimAndToLower(value));
  }

  @override
  void addAll(Iterable<String> elements) {
    if (elements != null) {
      elements.forEach((element) => add(element));
    }
  }

  @override
  bool any(bool Function(String element) test) {
    return set.any(test);
  }

  @override
  Set<R> cast<R>() {
    return set.cast();
  }

  @override
  void clear() {
    set.clear();
  }

  @override
  bool contains(Object value) {
    return value is String && set.contains(Utils.trimAndToLower(value));
  }

  @override
  bool containsAll(Iterable<Object> other) {
    return set.containsAll(other);
  }

  @override
  Set<String> difference(Set<Object> other) {
    return set.difference(other);
  }

  @override
  String elementAt(int index) {
    return set.elementAt(index);
  }

  @override
  bool every(bool Function(String element) test) {
    return set.every(test);
  }

  @override
  Iterable<T> expand<T>(Iterable<T> Function(String element) f) {
    return set.expand(f);
  }

  @override
  String get first => set.first;

  @override
  String firstWhere(bool Function(String element) test,
      {String Function() orElse}) {
    return set.firstWhere(test, orElse: orElse);
  }

  @override
  T fold<T>(
      T initialValue, T Function(T previousValue, String element) combine) {
    return set.fold(initialValue, combine);
  }

  @override
  Iterable<String> followedBy(Iterable<String> other) {
    return set.followedBy(other);
  }

  @override
  void forEach(void Function(String element) action) {
    set.forEach(action);
  }

  @override
  Set<String> intersection(Set<Object> other) {
    return set.intersection(other);
  }

  @override
  bool get isEmpty => set.isEmpty;

  @override
  bool get isNotEmpty => set.isNotEmpty;

  @override
  Iterator<String> get iterator => set.iterator;

  @override
  String join([String separator = ""]) {
    return set.join(separator);
  }

  @override
  String get last => set.last;

  @override
  String lastWhere(bool Function(String element) test,
      {String Function() orElse}) {
    return lastWhere(test, orElse: orElse);
  }

  @override
  int get length => set.length;

  @override
  String lookup(Object object) {
    return set.lookup(object);
  }

  @override
  Iterable<T> map<T>(T Function(String e) f) {
    return set.map(f);
  }

  @override
  String reduce(String Function(String value, String element) combine) {
    return set.reduce(combine);
  }

  @override
  bool remove(Object value) {
    return !Utils.isBlank(value) &&
        set.remove(Utils.trimAndToLower(Utils.toString_O(value)));
  }

  @override
  void removeAll(Iterable<Object> elements) {
    set.removeAll(elements);
  }

  @override
  void removeWhere(bool Function(String element) test) {
    set.removeWhere(test);
  }

  @override
  void retainAll(Iterable<Object> elements) {
    set.retainAll(elements);
  }

  @override
  void retainWhere(bool Function(String element) test) {
    set.retainWhere(test);
  }

  @override
  String get single => set.single;

  @override
  String singleWhere(bool Function(String element) test,
      {String Function() orElse}) {
    return set.singleWhere(test, orElse: orElse);
  }

  @override
  Iterable<String> skip(int count) {
    return set.skip(count);
  }

  @override
  Iterable<String> skipWhile(bool Function(String value) test) {
    return set.skipWhile(test);
  }

  @override
  Iterable<String> take(int count) {
    return set.take(count);
  }

  @override
  Iterable<String> takeWhile(bool Function(String value) test) {
    return set.takeWhile(test);
  }

  @override
  List<String> toList({bool growable = true}) {
    return set.toList(growable: growable);
  }

  @override
  Set<String> toSet() {
    return set.toSet();
  }

  @override
  Set<String> union(Set<String> other) {
    return set.union(other);
  }

  @override
  Iterable<String> where(bool Function(String element) test) {
    return set.where(test);
  }

  @override
  Iterable<T> whereType<T>() {
    return set.whereType();
  }

  String toConcatString() {
    return Utils.toCsv(this);
  }

  @override
  String toString() => toConcatString();
}
