/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../validation/converter/c_number.dart';
import '../utils.dart';

class BoundsUtils {
  static const String COMMA = ',';
  static const String OPEN_BRACKET = '(';
  static const String CLOSE_BRACKET = ')';

  const BoundsUtils._();

  static String getValue(String boundedViewId, String value) {
    if (Utils.isBlank(value)) {
      return null;
    }

    String trimmed = value.trim();
    List<int> bounds = getTextBounds(boundedViewId, trimmed.length);
    return getValue__(trimmed, bounds[0], bounds[1]);
  }

  static String getValue_(
      String boundedViewId, List<String> values, int index) {
    String value = inArrayValue(values, index);

    if (Utils.isBlank(value)) {
      return null;
    }

    value = value.trim();
    List<int> bounds = getTextBounds(boundedViewId, value.length);
    return getValue__(value, bounds[0], bounds[1]);
  }

  static String getViewId(String boundedViewId) {
    if (Utils.isBlank(boundedViewId)) {
      return null;
    }

    String trimmed = boundedViewId.trim();
    int openBracketIndex = trimmed.indexOf(OPEN_BRACKET);

    if (openBracketIndex == Defaults.MISSING) {
      return trimmed;
    }

    if (openBracketIndex < Defaults.MISSING) {
      return null;
    }

    return trimmed.substring(0, openBracketIndex).trim();
  }

  static List<int> getTextBounds(String text, int valueLength) {
    if (Utils.isBlank(text)) {
      return [Defaults.MISSING, Defaults.MISSING];
    }

    int openBraceIndex = text.indexOf(OPEN_BRACKET);

    if (openBraceIndex == Defaults.MISSING) {
      return [Defaults.MISSING, Defaults.MISSING];
    }

    StringBuffer startIndex = StringBuffer();
    StringBuffer endIndex = StringBuffer();
    bool startIndexLocated = false;
    int length = text.length;

    for (int index = openBraceIndex + 1; index < length; index++) {
      final String ch = text[index];

      if (startIndexLocated) {
        if (CLOSE_BRACKET != ch) {
          endIndex.write(ch);
        }
      } else {
        if (COMMA != ch && CLOSE_BRACKET != ch) {
          startIndex.write(ch);
        }
      }

      if (COMMA == ch) {
        startIndexLocated = true;
      }
    }

    int start = CNumber.parseNoErr(startIndex.toString());
    int end = CNumber.parseNoErr(endIndex.toString());
    int lastIndex = valueLength + 1; // inclusive

    if (start == null) {
      start = 1;
    } else if (start < 0) {
      start = lastIndex - (start * Defaults.MISSING);
    }

    if (end == null) {
      end = lastIndex;
    } else if (end < 0) {
      end = lastIndex - (end * Defaults.MISSING);
    }

    return [start, end];
  }

  static String inArrayValue(List<String> values, int index) {
    try {
      return values[index];
    } catch (e) {
      return null;
    }
  }

  static String getValue__(String value, int startIndex, int endIndex) {
    try {
      // We don't want to ignore spaces, check for null only.
      if (value == null) {
        return null;
      }

      // if both indexes are not provided.
      if (startIndex < 1 && endIndex < 1) {
        return value;
      }

      // if end index is provided but it's smaller than start index.
      if (endIndex > 0 && endIndex < startIndex) {
        return null;
      }

      final int length = value.length;

      // if start index is more than the length.
      if (startIndex > length) {
        return null;
      }

      // coerce indexes to fit within the boundary.
      final int start = inBounds(startIndex, length) ? 1 : startIndex;
      final int end = inBounds(endIndex, length) ? length : endIndex;

      // we consider 1st character as one not zero.
      return value.substring(start - 1, end);
    } catch (e) {
      return null;
    }
  }

  static bool inBounds(int index, int length) {
    return index < 1 || index > length;
  }
}
