/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/string_object_map.dart';
import '../../structure/model/field.dart';
import '../../ui/discrete/renderer/features.dart';
import '../../ui/discrete/renderer/ui_renderer.dart';
import '../../ui/discrete/view_state.dart';
import '../form_utils.dart';
import '../stop_watch.dart';
import '../utils.dart';
import 'bounds_utils.dart';
import 'value_utils.dart';

// TODO It should be used by every input component?
class Distributor {
  final UIRenderer renderer;

  const Distributor(this.renderer);

  String perform(String value, String vSrcId) {
    StopWatch stopWatch = StopWatch.start();
    StringObjectMap data = StringObjectMap();
    String srcValue = perform_(value, vSrcId, data);
    ValueUtils.setValues(data, true, renderer);
    StopWatch.stop_(value, stopWatch);
    return srcValue;
  }

  String perform_(String val, String vSrcId, Map<String, Object> data) {
    ViewState view = FormUtils.getView_(vSrcId, renderer);
    Field field = FormUtils.getField(view);
    return field == null || !view.supportsMode(renderer)
        ? val
        : perform__(val, field, {}, data);
  }

  String perform__(String srcValue, Field fSrc, Set<String> predecessors,
      Map<String, Object> data) {
    List<String> distribution =
        fSrc == null ? null : fSrc.getDistribution(renderer);

    if (Utils.isBlank(distribution)) {
      return srcValue;
    }

    final String vSrc = fSrc.getId();
    // split the value using the configured delimiters.
    List<String> values = Utils.split(srcValue, fSrc.getDelimiters());

    // if there is only one value, we simply assign it to the source view.
    if (Utils.isBlank(values) || values.length == 1) {
      return srcValue;
    }

    int vIndex = 0;
    String myValue;

    for (String boundedViewId in distribution) {
      // e.g. target viewId will be 'ICCID' extracted from 'ICCID(-3)'.
      String viewId = BoundsUtils.getViewId(boundedViewId);

      if (predecessors.contains(viewId)) {
        continue;
      }

      ViewState target = FormUtils.getView_(viewId, renderer);

      if (target != null && target.canInput()) {
        if (Utils.equalsIgnoreCase(vSrc, viewId)) {
          myValue = getTargetValue(boundedViewId, values, vIndex, target);
        } else {
          perform___(boundedViewId, values, vIndex, target, predecessors, data);
        }

        predecessors.add(viewId);
      }

      vIndex++;
    }

    return myValue;
  }

  void perform___(String boundedViewId, List<String> values, int vIndex,
      ViewState target, Set<String> predecessors, Map<String, Object> data) {
    // get the value for the target
    String vValue = getTargetValue(boundedViewId, values, vIndex, target);

    // Normally you would want validation for distribution feature but:
    // 1) validation would mark, even at the beginning (may be to change)
    // 2) for now distribution is only for barcode, so no validation needed

    // put the value in the map.
    data[target.getId()] = vValue;

    // distribute the targeted view's value further down the hierarchy.
    perform__(vValue, target.getField(), predecessors, data);
  }

  String getTargetValue(
      String boundedViewId, List<String> values, int vIndex, ViewState target) {
    // value from the source view as per the configured index.
    String vValue = BoundsUtils.getValue_(boundedViewId, values, vIndex);
    // get the mapped value if it's the case.
    return Features.getArrayValueHandler(renderer)
        .getMappedValue(vValue, target);
  }
}
