/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../error/illegal_argument_exception.dart';
import '../../structure/model/field.dart';
import '../../ui/discrete/view_state.dart';

/// Luhn algorithm to generate check digit in ICCID or similar.
class Luhn {
  static const int MODULUS = 10;

  const Luhn._();

  static Object perform(ViewState view) {
    if (view == null) {
      throw IllegalArgumentException("view shouldn't be null.");
    }

    Object value = view.getValue();

    if (value is String && view.canInput()) {
      Field field = view.getField();

      if (field != null) {
        int position = field.getLuhn();

        if (position != null) {
          value = doLuhn(value as String, position);
          view.setValue(value);
        }
      }
    }

    return value;
  }

  static int generate(String text) {
    return (10 - doLuhn_(text, true) % 10) % 10;
  }

  static String doLuhn(String text, int position) {
    if (text.length == position - 1) {
      try {
        text +=
            ((MODULUS - doLuhn_(text, true) % MODULUS) % MODULUS).toString();
      } catch (e) {
        // nothing to do here
      }
    }

    return text;
  }

  static int doLuhn_(String text, bool evenPosition) {
    int sum = 0;

    for (int i = text.length - 1; i >= 0; i--) {
      int number = int.parse(text.substring(i, i + 1));

      if (evenPosition) {
        number *= 2;

        if (number >= MODULUS) {
          number = (number % MODULUS) + 1;
        }
      }

      sum += number;
      evenPosition = !evenPosition;
    }

    return sum;
  }
}
