/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/string_object_map.dart';
import '../../enums/code.dart';
import '../../structure/model/field.dart';
import '../../ui/discrete/renderer/features.dart';
import '../../ui/discrete/renderer/ui_renderer.dart';
import '../../ui/discrete/view_state.dart';
import '../form_utils.dart';
import '../stop_watch.dart';
import '../utils.dart';
import 'bounds_utils.dart';
import 'value_utils.dart';

class Replicator {
  final UIRenderer renderer;

  const Replicator(this.renderer);

  void perform(ViewState view) {
    if (!view.supportsMode(renderer)) {
      return;
    }

    Field field = view.getField();

    if (field != null) {
      StopWatch stopWatch = StopWatch.start();
      StringObjectMap data = StringObjectMap();

      if (perform_(
          view.getValue(), view.getId(), field.getReplicate(renderer), data)) {
        ValueUtils.setValues(data, false, renderer);
      }

      StopWatch.stop_(view, stopWatch);
    }
  }

  bool perform_(Object value, String vSrc, List<String> replicate,
      Map<String, Object> data) {
    if (Utils.isBlank(replicate) ||
        !FormUtils.getView_(vSrc, renderer).supportsMode(renderer)) {
      return false;
    }

    // Set of such views who are higher in the replication hierarchy. This avoids any infinite callback by
    // prohibiting of setting the data from a view lower in the hierarchy to the view upper in the hierarchy.
    Set<String> predecessors = {};
    predecessors.add(vSrc);

    perform__(value, predecessors, replicate, data);
    return true;
  }

  void perform__(Object value, Set<String> predecessors, List<String> replicate,
      Map<String, Object> data) {
    for (String boundedViewId in replicate) {
      final bool setLabel = Code.EFFECT.starts(boundedViewId);
      boundedViewId = setLabel ? Utils.skipFirst(boundedViewId) : boundedViewId;

      // e.g. target viewId will be 'ICCID' extracted from 'ICCID(-3)'.
      String viewId = BoundsUtils.getViewId(boundedViewId);

      // Ignore the view if it was already visited in the hierarchy.
      if (predecessors.contains(viewId)) {
        continue;
      }

      predecessors.add(viewId);
      ViewState target = FormUtils.getView_(viewId, renderer);

      if (target == null) {
        continue; // target view should exist.
      }

      Object rValue = value;

      if (rValue is String) {
        // value from the source view as per the indexes configured.
        rValue = BoundsUtils.getValue(boundedViewId, rValue as String);
        // get the mapped value if it's the case.
        rValue = Features.getArrayValueHandler(renderer)
            .getMappedValue_(rValue as String, target, setLabel);
      }

      if (setLabel ||
          Features.getValidator(renderer).runWithoutMarking(target, rValue)) {
        data[setLabel ? Code.EFFECT.prefix(viewId) : viewId] =
            rValue; // put the value in the map

        Field field = target.getField();

        // replicate the targeted view's value further down the hierarchy.
        if (field != null) {
          List<String> targetViews = field.getReplicate(renderer);

          if (!Utils.isBlank(targetViews)) {
            perform__(value, predecessors, targetViews, data);
          }
        }
      }
    }
  }
}
