/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../form/structure/ordered_map.dart';
import '../enums/view_type.dart';
import '../error/illegal_argument_exception.dart';
import '../structure/ui/view.dart';
import '../ui/discrete/component/behavior/i_enable.dart';
import '../ui/discrete/component/behavior/i_render.dart';
import '../ui/discrete/renderer/params/renderer_params.dart';
import '../ui/discrete/view_state.dart';
import 'structure/string_set.dart';
import 'utils.dart';

class ViewUtils {
  const ViewUtils._();

  static void setEnabled(bool enabled, List<IEnable> components) {
    for (IEnable component in components) {
      if (component != null) {
        component.setEnabled(enabled);
      }
    }
  }

  static void setRendered(bool rendered, List<IRender> components) {
    for (IRender component in components) {
      if (component != null) {
        component.setRendered(rendered);
      }
    }
  }

  static String validate(View view, ViewState vParent,
      OrderedMap<ViewState> allViews, RendererParams params) {
    final String id = view.getId();

    if (Utils.got(id, allViews)) {
      // disallow multiple declaration from same view
      throw IllegalArgumentException("View '$id' is duplicated.");
    }

    if (params.getBaseUI().hasGroup(id)) {
      throw IllegalArgumentException("View '$id' is declared as group too.");
    }

    if (params.getModel().hasScript(id)) {
      throw IllegalArgumentException("View '$id' is declared as script too.");
    }

    ViewType cType = vParent.getType();
    ViewType type = view.getType();

    // If parent view is TabPanel, we must make sure that the child is only a Tab.
    if (ViewType.TABPANEL == cType && ViewType.TAB != type) {
      throw IllegalArgumentException(
          "Can't add '$type' in a ${ViewType.TABPANEL}");
    }

    // If parent view is Table, we must make sure that the child is only a Column.
    if (ViewType.TABLE == cType && !type.isColumn()) {
      throw IllegalArgumentException(
          "Can't add '$type' in a ${ViewType.TABLE}");
    }

    return id;
  }

  static void removeReferences(
      ViewState vSrc, OrderedMap<ViewState> allViews, StringSet inputViews) {
    if (vSrc != null) {
      List<ViewState> children = vSrc.getChildren();

      if (!Utils.isBlank(children)) {
        for (ViewState vChild in children) {
          if (vChild.isContainer()) {
            removeReferences(vChild, allViews, inputViews);
          } else {
            final String viewId = vChild.getId();
            allViews.remove(viewId);
            inputViews.remove(viewId);
          }
        }
      }

      allViews.remove(vSrc.getId());
    }
  }

// FIXME
// static InputStream getResourceStream(String name) {
//     return getResourceStream(name, ViewUtils);
// }

// static InputStream getResourceStream(String name, Type clazz) {
//     // attempt to load the resource from base project
//     InputStream stream = ClassLoader.getSystemResourceAsStream(name);
//     // if not found, look for it in implementation or base library (internally).
//     return stream == null ? clazz.getResourceAsStream(Utils.prefixSlash(name)) : stream;
// }

}
