/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/enum.dart';
import '../../util/enum_utils.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

class CCheckCase extends CAbstract<String> {
  static final ConstraintType TYPE = ConstraintType.CHECKCASE;

  static const String CAMEL_CASE_PATTERN = "([a-z]+[A-Z]+\\w+)+";

  final Case _case;

  CCheckCase() : this.full(Case.UPPER);

  CCheckCase.full(this._case) : super(TYPE) {
    if (_case == null) {
      throw IllegalArgumentException("${Args.CASE} can't be blank!");
    }
  }

  @override
  String isValid(String value) {
    if (Utils.isBlank(value)) {
      return null;
    }

    bool valid = true;

    if (Case.LOWER == _case) {
      valid = value == value.toLowerCase();
    } else if (Case.MIXED == _case) {
      valid = isMixedCase(value);
    } else if (Case.CAMEL == _case) {
      valid = Utils.matches(value.trim(), CAMEL_CASE_PATTERN);
    } else if (Case.UPPER == _case) {
      valid = value == value.toUpperCase();
    }

    return valid ? null : getMsgId();
  }

  bool isMixedCase(String value) {
    if (Utils.isBlank(value)) {
      return false;
    }

    int lowerChars = 0, upperChars = 0;

    for (int i = 0; i < value.length; i++) {
      final String ch = value[i];

      if (Utils.isLowerCase(ch)) {
        lowerChars++;
      } else if (Utils.isUpperCase(ch)) {
        upperChars++;
      }

      if (lowerChars > 0 && upperChars > 0) {
        return true; // no need to check further
      }
    }

    return false;
  }

  @override
  String getMsgId() {
    return _case.msgId();
  }
}

class Args {
  static const String CASE = "case";
}

class Case extends Enum<Case> {
  static const LOWER = Case("LOWER");
  static const MIXED = Case("MIXED");
  static const CAMEL = Case("CAMEL");
  static const UPPER = Case("UPPER");

  const Case(String name) : super(name);

  String msgId() {
    return "validation.${name().toLowerCase()}case.invalid";
  }

  static Case toCase(String _case, bool throwOnMismatch) {
    return EnumUtils.toEnum_(_case, _ENUMS, throwOnMismatch);
  }

  static const List<Case> _ENUMS = [LOWER, MIXED, CAMEL, UPPER];
}
