/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/bean/range/int_range.dart';
import '../../util/utils.dart';
import 'ca_range.dart';

/// Counts all the given characters in a string.
///
/// E.g. When you don't want more than 2 vowels in a name
class CCount extends CARange {
  static final ConstraintType TYPE = ConstraintType.COUNT;

  bool sensitive;
  String chars;

  CCount(IntRange range, bool sensitive, String chars) : super(TYPE, range) {
    if (Utils.isBlank(chars)) {
      throw IllegalArgumentException("${Args.CHARS} can't be blank!");
    }

    this.sensitive = Utils.toFalse(sensitive);
    this.chars = this.sensitive ? chars : Utils.trimAndToLower(chars);
  }

  @override
  int count(String text) {
    String str = sensitive ? Utils.trim(text) : Utils.trimAndToLower(text);
    int count = 0;

    for (int index = 0; index < str.length; index++) {
      final String c = str[index];

      if (chars.contains(c)) {
        count++;
      }
    }

    return count;
  }
}

class Args {
  static const String MIN = CARange.MIN;
  static const String MAX = CARange.MAX;
  static const String SENSITIVE = Defaults.SENSITIVE;
  static const String CHARS = Defaults.CHARS;
}
