/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../util/date_utils.dart';
import '../../util/enum.dart';
import '../../util/enum_utils.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

class CDate extends CAbstract<DateTime> {
  static final ConstraintType TYPE = ConstraintType.DATE;

  Mode mode;
  bool allowToday;
  int years;
  int months;
  int days;

  CDate(Mode mode, bool allowToday, int years, int months, int days)
      : super(TYPE) {
    this.mode = Utils.toDefault(mode, Mode.PAST);
    this.allowToday = Utils.toFalse(allowToday);
    this.years = years;
    this.months = months;
    this.days = days;
  }

  @override
  String isValid(DateTime value) {
    return value == null || isValid_(DateUtils.dateToMillis_(value))
        ? null
        : mode.msgId(Numbers.notZero(years) ||
            Numbers.notZero(months) ||
            Numbers.notZero(days));
  }

  bool isValid_(int givenDate) {
    final bool inPast = Mode.PAST == mode;
    final int thresholdDate = DateUtils.dateToMillis_(
        DateUtils.alterToday_(inPast, years, months, days));

    if (allowToday) {
      return inPast ? givenDate <= thresholdDate : givenDate >= thresholdDate;
    }

    return inPast ? givenDate < thresholdDate : givenDate > thresholdDate;
  }

  @override
  List<Object> getMsgArgs() {
    return [
      Numbers.toZero(years),
      Numbers.toZero(months),
      Numbers.toZero(days)
    ];
  }
}

class Args {
  /// mode determine if dates in the past or in the future only are valid
  static const String MODE = Defaults.MODE;

  static const String YEARS = "years";
  static const String MONTHS = "months";
  static const String DAYS = "days";

  /// Allow today's date for past or future mode
  static const String ALLOW_TODAY = "allowToday";
}

class Mode extends Enum<Mode> {
  static const PAST = Mode("PAST");
  static const FUTURE = Mode("FUTURE");

  const Mode(String name) : super(name);

  String msgId(bool dateProvided) {
    return "validation.${name().toLowerCase()}${dateProvided ? ".date" : ""}.invalid";
  }

  static Mode toMode(String mode, bool throwOnMismatch) {
    return EnumUtils.toEnum_(mode, _ENUMS, throwOnMismatch);
  }

  static const List<Mode> _ENUMS = [PAST, FUTURE];
}
