/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

/// Checks if a text contains minimum X distinct characters.
///
/// For example, if X = 2, then '111' and '222' is invalid.
class CDistinct extends CAbstract<String> {
  static final ConstraintType TYPE = ConstraintType.DISTINCT;

  int min;
  bool sensitive;
  String ignore;

  CDistinct(int min, bool sensitive, String ignore) : super(TYPE) {
    if (Numbers.belowOne(min)) {
      throw IllegalArgumentException("${Args.MIN} should be a natural number.");
    }

    this.min = Utils.toDefault(min, 2);
    this.sensitive = Utils.toFalse(sensitive);
    this.ignore = toLowerCase(ignore);
  }

  @override
  String isValid(String value) {
    if (Utils.isBlank(value)) {
      return null;
    }

    String str = value.trim();
    str = sensitive ? str : str.toLowerCase();
    Set<String> uniqueCharacters = {};

    for (int index = 0; index < str.length; index++) {
      final String c = str[index];

      // ignore the character if it's part of given text.
      if (ignore.indexOf(c) == Defaults.MISSING) {
        uniqueCharacters.add(c);

        if (uniqueCharacters.length >= min) {
          break; // no need to continue since we already have satisfied the minimum requirement.
        }
      }
    }

    return uniqueCharacters.length >= min ? null : getMsgId();
  }

  @override
  List<Object> getMsgArgs() {
    return [min];
  }

  String toLowerCase(String ignore) {
    String ignoreChars = Utils.toBlank(ignore);
    return sensitive ? ignoreChars : ignoreChars.toLowerCase();
  }
}

class Args {
  static const String MIN = Defaults.MIN;
  static const String SENSITIVE = Defaults.SENSITIVE;
  static const String IGNORE = "ignore";
}
