/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

/// Prevents user to input the identical value in multiple fields for n times.
class CDuplicate extends CAbstract<Object> {
  static final ConstraintType TYPE = ConstraintType.DUPLICATE;

  final List<Object> values;
  final int max;

  /// Create a new Duplicate constraint
  ///
  /// @param values
  ///        list of values to compare
  /// @param max
  ///        maximum allowed number of occurrences for the identical value.
  CDuplicate(this.values, this.max) : super(TYPE);

  /// Compare the value of the calling view against values in the list. Returns an error message id if provided value
  /// is duplicated more than allowed. Uses 'equals' to compare objects.
  @override
  String isValid(Object value) {
    if (value == null) {
      return null;
    }

    if (value is String) {
      // if value is String, should be preprocessed to be case insensitive
      final String sValue = value;

      if (Utils.isBlank(sValue)) {
        return null;
      }

      value = Utils.trimAndToLower(sValue);
    }

    int count = 0;

    for (Object refValue in values) {
      if (refValue != null) {
        if (refValue is String) {
          String sRefValue = refValue;

          if (Utils.isBlank(sRefValue)) {
            continue;
          }

          refValue = Utils.trimAndToLower(sRefValue);
        }

        if (value == refValue) {
          count++;
        }

        if (count > max - 1) {
          // '> max - 1' because the calling view counts as well
          return getMsgId();
        }
      }
    }

    return null;
  }

  @override
  List<Object> getMsgArgs() {
    return [max];
  }
}

class Args {
  static const String FIELDS = Defaults.FIELDS;
  static const String MAX = Defaults.MAX;
}
