/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

class CExclude extends CAbstract<String> {
  static final ConstraintType TYPE = ConstraintType.EXCLUDE;

  bool sensitive;
  String chars;
  int position; // TODO check for multiple positions (int[])?

  CExclude(bool sensitive, String chars, int position) : super(TYPE) {
    if (chars == null) {
      // don't use Utils.isBlank() here
      throw IllegalArgumentException("${Args.CHARS} is required.");
    }

    if (Numbers.nullOrBelowOne(position)) {
      throw IllegalArgumentException("${Args.POSITION} is required.");
    }

    this.sensitive = Utils.toFalse(sensitive);
    this.chars = this.sensitive ? chars : chars.toLowerCase();
    this.position = position;
  }

  @override
  String isValid(String value) {
    if (Utils.isBlank(value)) {
      return null; // if null or empty, pass the validation
    }

    final String tValue = value.trim();
    bool valid = true;

    if (position <= tValue.length) {
      // if position is out of bounds, pass the validation
      String character = tValue[position - 1];
      character = sensitive ? character : character.toLowerCase();

      for (int i = 0; i < chars.length; i++) {
        if (chars[i] == character) {
          valid = false;
          break;
        }
      }
    }

    return valid ? null : getMsgId();
  }

  @override
  List<Object> getMsgArgs() {
    return [chars, position];
  }
}

class Args {
  static const String SENSITIVE = Defaults.SENSITIVE;
  static const String CHARS = Defaults.CHARS;
  static const String POSITION = "position";
}
