/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import '../../../form/structure/string_object_map.dart';
import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../ui/discrete/form_controller.dart';
import '../../util/bean/range/int_range.dart';
import '../../util/form_utils.dart';
import '../../util/io_utils.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

class CFile extends CAbstract<Uint8List> {
  static final ConstraintType TYPE = ConstraintType.FILE;

  static const String MSG = "validation.file.size";

  final IntRange size; // in KBs

  CFile(this.size) : super(TYPE) {
    if (size != null &&
        (Numbers.belowOne(size.min) || Numbers.belowOne(size.max))) {
      throw IllegalArgumentException(
          "${Args.MIN} and ${Args.MAX} should be a natural numbers.");
    }
  }

  @override
  String isValid(Uint8List value) {
    return size == null ||
            Utils.isBlank(value) ||
            size.isInRange(getSize(value))
        ? null
        : MSG;
  }

  int getSize(Uint8List value) {
    return IOUtils.getSize__(value);
  }

  @override
  List<Object> getMsgArgs() {
    return getMsgArgs_(size);
  }

  static List<Object> getMsgArgs_(IntRange size) {
    final bool verifySize = size != null;

    return [
      Utils.toDefault(verifySize ? size.min : null, 0),
      Utils.toDefault(verifySize ? size.max : null, Defaults.NA)
    ];
  }

  static IntRange getSize_(String viewId, FormController controller) {
    StringObjectMap args =
        FormUtils.getConstraintArgs(viewId, TYPE, controller);
    return IntRange.get(Args.MIN, Args.MAX, args);
  }
}

class Args {
  static const String EXT = Defaults.EXT;
  static const String MIN = Defaults.MIN; // in KBs
  static const String MAX = Defaults.MAX; // in KBs
  static const String AUTO_COMPRESS = "autoCompress";
}
