/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/ordered_map.dart';
import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../ui/discrete/view_state.dart';
import '../../util/numbers.dart';
import '../../util/structure/string_set.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';
import 'c_mandatory.dart';

/// Checks if value of at least 'n' number of given fields is not null.
class CGroupMandatory extends CAbstract<Object> {
  static final ConstraintType TYPE = ConstraintType.GROUPMANDATORY;

  static const int MIN_COUNT = 1;

  List<Object> values;
  int count;

  CGroupMandatory(String fields, int count, OrderedMap<ViewState> allViews)
      : super(TYPE) {
    StringSet fieldIds = Utils.splitByComma(fields);

    if (Utils.isBlank(fieldIds)) {
      throw IllegalArgumentException("${Args.FIELDS} is mandatory.");
    }

    this.count = Numbers.nullOrBelowOne(count) ? MIN_COUNT : count;
    this.values = [];

    for (String fieldId in fieldIds) {
      ViewState view = allViews[fieldId];

      if (view == null || !view.canInput()) {
        throw IllegalArgumentException("Invalid view '$fieldId'");
      }

      values.add(view.getValue());
    }
  }

  @override
  String isValid(Object value) {
    CMandatory mandatory = CMandatory();
    // if given value is provided, count begins with 1, else with 0.
    int notNullCount = CAbstract.check(value, mandatory) ? 1 : 0;

    if (notNullCount < count) {
      for (Object refValue in values) {
        if (CAbstract.check(refValue, mandatory)) {
          notNullCount++;

          if (enoughMandatory(notNullCount)) {
            break;
          }
        }
      }
    }

    return enoughMandatory(notNullCount) ? null : getMsgId();
  }

  bool enoughMandatory(int notNullCount) {
    return notNullCount >= count;
  }

  @override
  List<Object> getMsgArgs() {
    return [count];
  }
}

class Args {
  static const String FIELDS = Defaults.FIELDS; // comma-separated field ids
  static const String COUNT =
      "count"; // how many to be considered mandatory among all?

}
