/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../ui/discrete/util/language/languages.dart';
import '../../util/bean/range/int_range.dart';
import '../../util/utils.dart';
import 'ca_range.dart';

class CLength extends CARange {
  static final ConstraintType TYPE = ConstraintType.LENGTH;

  final String languageId;

  CLength(IntRange range, this.languageId) : super(TYPE, range);

  @override
  int count(String text) {
    if (Utils.isBlank(languageId)) {
      return text.trim().length;
    }

    Language language = Languages.get(languageId);
    int count = 0;

    if (language != null) {
      String regex = "[${language.letters}${language.digits}]";

      for (int index = 0; index < text.length; index++) {
        final String c = text[index];

        if (Utils.matches(c, regex)) {
          count++;
        }
      }
    }

    return count;
  }

  @override
  List<Object> getMsgArgs() {
    return [range.min, range.max, languageId];
  }
}

class Args {
  static const String MIN = CARange.MIN;
  static const String MAX = CARange.MAX;
  static const String LANGUAGE = Defaults.LANGUAGE;
}
