/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

class CRepetition extends CAbstract<String> {
  static final ConstraintType TYPE = ConstraintType.REPETITION;

  int max;
  bool sensitive;
  String chars;

  CRepetition(int max, bool sensitive, String chars) : super(TYPE) {
    if (Numbers.belowOne(max)) {
      throw IllegalArgumentException("${Args.MAX} should be a natural number.");
    }

    this.max = Utils.toDefault(max, 3);
    this.sensitive = Utils.toFalse(sensitive);
    this.chars = this.sensitive ? chars : toLower(chars);
  }

  @override
  String isValid(String value) {
    if (Utils.isBlank(value)) {
      return null;
    }

    String str = value.trim();
    str = sensitive ? str : toLower(str);
    final bool everyCharCounts =
        isEmpty(chars); // don't use Utils.isBlank() here
    bool valid = true;
    String repeatedChar;
    int count = 0;

    for (int index = 0; index < str.length; index++) {
      final String c = str[index];

      if (!everyCharCounts && !chars.contains(c)) {
        repeatedChar = null;
        count = 0;
        continue;
      }

      // TODO Check for Character.isLetter().
      if (repeatedChar != null && repeatedChar == c) {
        count++;

        if (count >= max) {
          valid = false;
          break;
        }
      } else {
        repeatedChar = c;
        count = 1;
      }
    }

    return valid ? null : getMsgId();
  }

  String toLower(String chars) {
    return isEmpty(chars)
        ? null
        : chars.toLowerCase(); // don't use Utils.isBlank() here
  }

  bool isEmpty(String chars) {
    return chars == null || chars.length < 1;
  }

  @override
  List<Object> getMsgArgs() {
    return [max];
  }
}

class Args {
  static const String MAX = Defaults.MAX;
  static const String SENSITIVE = Defaults.SENSITIVE;
  static const String CHARS = Defaults.CHARS;
}
