/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../ui/discrete/renderer/ui_renderer.dart';
import '../../util/script/condition_evaluator.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

class CScript extends CAbstract<Object> {
  static final ConstraintType TYPE = ConstraintType.SCRIPT;

  static const String VALUE = Defaults.VALUE;

  String expression;
  bool ignoreIfNull;
  UIRenderer renderer;

  CScript(String expression, bool ignoreIfNull, UIRenderer renderer)
      : super(TYPE) {
    if (Utils.isBlank(expression)) {
      throw IllegalArgumentException("${Args.EXPR} is required.");
    }

    if (!Utils.contains(expression, [VALUE])) {
      throw IllegalArgumentException(
          "${Args.EXPR} doesn't use the provided value.");
    }

    this.expression = expression;
    this.ignoreIfNull = Utils.toFalse(ignoreIfNull);
    this.renderer = renderer;
  }

  @override
  String isValid(Object value) {
    return (ignoreIfNull && Utils.isBlank(value)) || eval(value)
        ? null
        : getMsgId();
  }

  bool eval(Object value) {
    Map<String, Object> args = {VALUE: value};

    return Utils.toTrue(
        ConditionEvaluator.evalExpr(expression, args, renderer));
  }
}

class Args {
  static const String EXPR = "expr";
  static const String IGNORE_IF_NULL = "ignoreIfNull";
}
