/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../form/structure/ordered_map.dart';
import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../ui/discrete/component/table/table.dart';
import '../../ui/discrete/view_state.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import 'c_abstract.dart';

/// Checks if a field's value is 'n' times duplicated in the specified column of a table.
class CTableDuplicate extends CAbstract<Object> {
  static final ConstraintType TYPE = ConstraintType.TABLEDUPLICATE;

  ViewState vTable;
  int columnIndex;
  int max;

  CTableDuplicate(
      String tableId, int columnIndex, int max, OrderedMap<ViewState> allViews)
      : super(TYPE) {
    this.vTable = allViews[tableId];

    if (vTable == null) {
      throw IllegalArgumentException("Table view not found.");
    }

    if (Numbers.nullOrBelowOne(columnIndex)) {
      throw IllegalArgumentException("${Args.COLUMN} is required.");
    }

    this.columnIndex = columnIndex - 1; // configuration is 1-based
    this.max = Numbers.nullOrBelowOne(max) ? 1 : max;
  }

  @override
  String isValid(Object value) {
    if (value == null || (value is String && Utils.isBlank(value))) {
      return null;
    }

    Table table = vTable.getComponent() as Table;
    List<List<String>> rows = table.getVal() as List<List<String>>;

    if (Utils.isBlank(rows)) {
      return null;
    }

    final int selectedRowIndex = table.getSelectedRowIndex();
    final String sValue = value.toString();

    int count = 0;
    int rowIndex = Defaults.MISSING;

    for (List<String> row in rows) {
      // in edit mode, we shouldn't consider the row being updated.
      if (++rowIndex == selectedRowIndex) {
        continue;
      }

      try {
        if (Utils.equalsIgnoreCase(sValue, row[columnIndex])) {
          count++;

          if (count >= max) {
            break; // no need to check further
          }
        }
      } catch (e) {
        throw IllegalArgumentException(
            "${Args.COLUMN} should in the columns range.");
      }
    }

    return count < max ? null : getMsgId();
  }
}

class Args {
  static const String TABLE = "table"; // table id
  static const String COLUMN = "column"; // column-index (1 being the 1st)
  static const String MAX = Defaults.MAX;
}
