/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/constraint_type.dart';
import '../../util/bean/range/int_range.dart';
import '../../util/utils.dart';
import 'ca_range.dart';

class CWord extends CARange {
  static final ConstraintType TYPE = ConstraintType.WORD;

  CWord(IntRange range) : super(TYPE, range);

  @override
  int count(String text) {
    final int length = text.length;
    final int eol = length - 1; // last index
    bool word = false;
    int count = 0;

    for (int i = 0; i < length; i++) {
      final String c = text[i];
      final bool letter = Utils.isLetter(c);

      if (letter && i != eol) {
        // char is a letter, word = true.
        word = true;
      } else if (!letter && c != '\'' && word) {
        // +1 if char isn't a letter and there were letters before
        word = false;
        count++;
      } else if (letter && i == eol) {
        // last word not ending with a non letter, it wouldn't count without this
        count++;
      }
    }

    return count;
  }
}

class Args {
  static const String MIN = CARange.MIN;
  static const String MAX = CARange.MAX;
}
