/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/constraint_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/bean/range/int_range.dart';
import '../../util/numbers.dart';
import '../../util/utils.dart';
import '../converter/c_string.dart';
import 'c_abstract.dart';

abstract class CARange extends CAbstract<Object> {
  static const String MIN = Defaults.MIN;
  static const String MAX = Defaults.MAX;

  final IntRange range;

  CARange(ConstraintType constraint, this.range) : super(constraint) {
    if (range == null) {
      throw IllegalArgumentException("Either $MIN or $MAX is required.");
    }

    if (Numbers.belowOne(range.min) || Numbers.belowOne(range.max)) {
      throw IllegalArgumentException(
          "$MIN and $MAX should be natural numbers.");
    }
  }

  @override
  String isValid(Object value) {
    final String text = CString.convert(value);
    return Utils.isBlank(text) ? null : check(text);
  }

  String check(String text) {
    final int count = this.count(text);

    if (range.min == null) {
      return count <= range.max ? null : "validation.$constraint.max.invalid";
    }

    if (range.max == null) {
      return count >= range.min ? null : "validation.$constraint.min.invalid";
    }

    return count >= range.min && count <= range.max ? null : getMsgId();
  }

  int count(String text);

  @override
  String getMsgId() {
    return range.minEqualsMax()
        ? "validation.$constraint.equal.invalid"
        : super.getMsgId();
  }

  @override
  List<Object> getMsgArgs() {
    return [range.min, range.max];
  }
}
