/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import '../../../../form/structure/string_object_map.dart';
import '../../../enums/constraint_type.dart';
import '../../../ui/discrete/form_controller.dart';
import '../../../util/form_utils.dart';
import '../c_abstract.dart';

class CImageMatcher extends CAbstract<Uint8List> {
  static final ConstraintType TYPE = ConstraintType.IMAGEMATCHER;

  CImageMatcher() : super(TYPE);

  @override
  String isValid(Uint8List value) {
    return null; // matching is done live during the image capturing processing, nothing to verify here.
  }
}

class Args {
// effects only the component not the constraint
  static const String APP_MODE = "app.mode"; // mode of the app to run in?
  static const String PIXEL_THRESHOLD =
      "pixel.threshold"; // brightest grey scaled pixel we will accept (0 - 255)
  static const String ASPECT_RATIO =
      "aspect.ratio"; // Frame width/height in float
  static const String CAMERA_SIZE = "camera.size"; // camera/canvas size
  static const String TEMPLATE_ZONES = "template.zones";
  static const String TEMPLATE_ZONES_ANDROID = "template.zones.android";
}

class UIParams {
  final String
      appMode; // 1 = Production, 2 = Templates Generation, 3 = Test Mode
  final String pixelThreshold; // default = 50
  final String aspectRatio; // for Android only
  final String cameraSize; // default = 640x480 (not for Android)
  // left%, top%, width%, height%, template, matches, scaleFactor, minWidth, minHeight, maxWidth, maxHeight,
  // minNeighbors, flags
  final String templateZones; // separate zones by semi-colon (not for Android)
  // left%, top%, width%, height%, template, matches, scaleFactor, minWidth, minHeight, maxWidth, maxHeight,
  // minNeighbors, flags
  final String
      templateZonesAndroid; // separate zones by semi-colon (for Android only)

  const UIParams._(this.appMode, this.pixelThreshold, this.aspectRatio,
      this.cameraSize, this.templateZones, this.templateZonesAndroid);

  static UIParams get(String viewId, FormController controller) {
    StringObjectMap args =
        FormUtils.getConstraintArgs(viewId, CImageMatcher.TYPE, controller);
    return UIParams._(
        args.getAsString(Args.APP_MODE),
        args.getAsString(Args.PIXEL_THRESHOLD),
        args.getAsString(Args.ASPECT_RATIO),
        args.getAsString(Args.CAMERA_SIZE),
        args.getAsString(Args.TEMPLATE_ZONES),
        args.getAsString(Args.TEMPLATE_ZONES_ANDROID));
  }
}
