/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import '../../../../form/structure/string_object_map.dart';
import '../../../config/defaults.dart';
import '../../../enums/constraint_type.dart';
import '../../../ui/discrete/form_controller.dart';
import '../../../util/form_utils.dart';
import '../../converter/c_number.dart';
import '../c_abstract.dart';

class CSmartScan extends CAbstract<Uint8List> {
  static final ConstraintType TYPE = ConstraintType.SMARTSCAN;

  CSmartScan() : super(TYPE);

  @override
  String isValid(Uint8List value) {
    return null; // scanning is done live during the image capturing processing, nothing to verify here.
  }
}

class Args {
  // effects only the component not the constraint
  static const String WIDTH = Defaults.WIDTH;
  static const String HEIGHT = Defaults.HEIGHT;
  static const String MARGIN_LEFT = "margin.left";
  static const String MARGIN_RIGHT = "margin.right";
  static const String MARGIN_TOP = "margin.top";
  static const String MARGIN_BOTTOM = "margin.bottom";
}

class UIParams {
  final int width;
  final int height;
  final int marginLeft;
  final int marginRight;
  final int marginTop;
  final int marginBottom;

  const UIParams._(this.width, this.height, this.marginLeft, this.marginRight,
      this.marginTop, this.marginBottom);

  static UIParams get(String viewId, FormController controller) {
    StringObjectMap args =
        FormUtils.getConstraintArgs(viewId, CSmartScan.TYPE, controller);
    return UIParams._(
        CNumber.convertNoErr(args.getAsString(Args.WIDTH)),
        CNumber.convertNoErr(args.getAsString(Args.HEIGHT)),
        CNumber.convertNoErr(args.getAsString(Args.MARGIN_LEFT)),
        CNumber.convertNoErr(args.getAsString(Args.MARGIN_RIGHT)),
        CNumber.convertNoErr(args.getAsString(Args.MARGIN_TOP)),
        CNumber.convertNoErr(args.getAsString(Args.MARGIN_BOTTOM)));
  }
}
