/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../../config/defaults.dart';
import '../../../enums/constraint_type.dart';
import '../../../error/illegal_argument_exception.dart';
import '../../../util/numbers.dart';
import '../../../util/utils.dart';
import '../c_abstract.dart';

class CPattern extends CAbstract<String> {
  static final ConstraintType TYPE = ConstraintType.PATTERN;

  static const String ALPHANUMERIC = "a-zA-Z0-9";

  String regex;
  bool enDigits;
  bool find;

  RegExp pattern;

  CPattern(String regex) : this.regex(null, regex, null);

  CPattern.type(ConstraintType type) : this.regex(type, null, type.regex());

  CPattern.regex(ConstraintType type, String regex, String defaultRegex)
      : this.full(type, regex, defaultRegex, null, null, null);

  CPattern.full(ConstraintType type, String regex, String defaultRegex,
      bool enDigits, bool find, bool sensitive)
      : super(Utils.toDefault(type, TYPE)) {
    this.regex = Utils.toDefault(
        regex, defaultRegex); // if regex is blank, use defaultRegex.

    if (Utils.isBlank(this.regex)) {
      // if regex is blank, we can't continue
      throw IllegalArgumentException("${Args.REGEX} can't be blank!");
    }

    this.pattern = RegExp(this.regex, caseSensitive: Utils.toTrue(sensitive));
    this.enDigits = Utils.toFalse(enDigits);
    this.find = Utils.toFalse(find);
  }

  @override
  String isValid(String value) {
    if (Utils.isBlank(value)) {
      return null;
    }

    String sValue = value.trim();

    if (enDigits) {
      sValue = Numbers.toDigitsCharByChar(sValue);
    }

    // TODO How to do pattern.find()?
    if (find ? pattern.hasMatch(sValue) : pattern.hasMatch(sValue)) {
      return null;
    }

    return find ? "validation.pattern.find.invalid" : getMsgId();
  }

  String getRegex() {
    return this.regex;
  }

  @override
  List<Object> getMsgArgs() {
    return [regex, enDigits];
  }
}

class Args {
  static const String REGEX = Defaults.REGEX;
  static const String EN_DIGITS = "enDigits";
  static const String FIND = "find"; // false by default
  static const String SENSITIVE = Defaults.SENSITIVE; // true by default

}
