/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../enums/field_type.dart';
import '../error/illegal_argument_exception.dart';
import 'converter/c_binary.dart';
import 'converter/c_boolean.dart';
import 'converter/c_byte.dart';
import 'converter/c_character.dart';
import 'converter/c_date.dart';
import 'converter/c_date_time.dart';
import 'converter/c_decimal.dart';
import 'converter/c_float.dart';
import 'converter/c_long.dart';
import 'converter/c_number.dart';
import 'converter/c_short.dart';
import 'converter/c_string.dart';
import 'converter/c_time.dart';
import 'converter/conversion_error.dart';

class Converter {
  const Converter._();

  static Object convertNoErr(FieldType type, Object value) {
    try {
      return convert(type, value);
    } on ConversionError {
      return value;
    }
  }

  static Object convert(FieldType type, Object value) {
    if (value == null || value is List) {
      return value;
    }

    if (FieldType.BYTE == type) {
      return CByte.convert(value);
    } else if (FieldType.SHORT == type) {
      return CShort.convert(value);
    } else if (FieldType.NUMBER == type) {
      return CNumber.convert(value);
    } else if (FieldType.LONG == type) {
      return CLong.convert(value);
    } else if (FieldType.FLOAT == type) {
      return CFloat.convert(value);
    } else if (FieldType.DECIMAL == type) {
      return CDecimal.convert(value);
    } else if (FieldType.CHARACTER == type) {
      return CCharacter.convert(value);
    } else if (FieldType.STRING == type) {
      return CString.convert(value);
    } else if (FieldType.BOOLEAN == type) {
      return CBoolean.convert(value);
    } else if (FieldType.DATE == type) {
      return CDate.convert(value);
    } else if (FieldType.TIME == type) {
      return CTime.convert(value);
    } else if (FieldType.DATETIME == type) {
      return CDateTime.convert(value);
    } else if (FieldType.BINARY == type) {
      return CBinary.convert(value);
    } else if (FieldType.LIST == type) {
      return value;
    }

    throw IllegalArgumentException("Field Type '$type' isn't supported!");
  }
}
