/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import '../../enums/field_type.dart';
import '../../util/log/logger.dart';
import '../../util/utils.dart';
import 'conversion_error.dart';

class CBinary {
  const CBinary._();

  static bool checkType(Object object) {
    return object is List<int> || object is Uint8List;
  }

  static bool isBinary(Object object) {
    return checkType(object);
  }

  static List<int> convertNoErr(Object value) {
    try {
      return convert(value);
    } on ConversionError {
      return null;
    }
  }

  // TODO Test this for ACamera/AFingerprint/AFilepicker.
  static List<int> convert(Object value) {
    if (CBinary.isBinary(value)) {
      return value;
    } else if (value is String) {
      try {
        return Utils.toBytes_(value, true);
      } catch (e, stackTrace) {
        Logger.error(e, stackTrace: stackTrace);
      }
    }

    throw ConversionError(FieldType.BINARY);
  }
}
