/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/field_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/utils.dart';
import 'c_number.dart';
import 'conversion_error.dart';

class CBoolean {
  static const bool YES = true;
  static const bool NO = false;

  const CBoolean._();

  static bool checkType(Object object) {
    return object is bool;
  }

  static bool isBoolean(Object object) {
    return checkType(object);
  }

  static bool coercelyConvert(Object value) {
    return Utils.toFalse(convertNoErr(value));
  }

  static bool convertNoErr(Object value) {
    try {
      return convert(value);
    } on ConversionError {
      return null;
    }
  }

  static bool convert(Object value) {
    if (value is bool) {
      return value;
    } else if (value is num) {
      final int number = CNumber.toPrimitive(value);

      if (Utils.match_(number, [0, 1])) {
        return number == 1;
      }
    } else if (value is String) {
      try {
        return parse(value);
      } on IllegalArgumentException {
        // nothing to do here
      }
    }

    throw ConversionError(FieldType.BOOLEAN);
  }

  static bool parseNoErr(String value) {
    try {
      return parse(value);
    } on IllegalArgumentException {
      return null;
    }
  }

  static bool parse(String value) {
    if (Utils.isBlank(value)) {
      return null;
    }

    if (Utils.match(value, ["true", "yes", "y", "1"])) {
      return YES;
    }

    if (Utils.match(value, ["false", "no", "n", "0"])) {
      return NO;
    }

    throw IllegalArgumentException("$value can't be parsed to Boolean.");
  }

  static bool toPrimitive(Object value) {
    return value as bool;
  }
}
