/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/field_type.dart';
import '../../error/illegal_argument_exception.dart';
import '../../util/utils.dart';
import 'c_number.dart';
import 'c_string.dart';
import 'conversion_error.dart';

class CCharacter {
  const CCharacter._();

  static bool checkType(Object object) {
    return CString.checkType(object);
  }

  static String convertNoErr(Object value) {
    try {
      return convert(value);
    } on ConversionError {
      return null;
    }
  }

  static String convert(Object value) {
    if (value is String) {
      try {
        return parse(value);
      } on IllegalArgumentException {
        // nothing to do here
      }
    } else if (value is num) {
      return String.fromCharCode(CNumber.toPrimitive(value));
    }

    throw ConversionError(FieldType.CHARACTER);
  }

  static String parseNoErr(String value) {
    try {
      return parse(value);
    } on IllegalArgumentException {
      return null;
    }
  }

  static String parse(String value) {
    final String tValue = Utils.trim(value);

    if (Utils.isBlank(tValue)) {
      return null;
    } else if (tValue.length == 1) {
      return tValue[0];
    }

    throw IllegalArgumentException("$value can't be parsed to Character.");
  }

  static String toPrimitive(Object value) {
    return value.toString()[0];
  }
}
