/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../config/defaults.dart';
import '../../enums/field_type.dart';
import '../../util/date_utils.dart';
import '../../util/utils.dart';
import 'c_number.dart';
import 'conversion_error.dart';

class CDate {
  const CDate._();

  static bool checkType(Object object) {
    return object is DateTime;
  }

  static DateTime convertNoErr(Object value) {
    try {
      return convert(value);
    } on ConversionError {
      return null;
    }
  }

  static DateTime convertNoErr_(Object value, String format) {
    try {
      return convert_(value, format);
    } on ConversionError {
      return null;
    }
  }

  static DateTime convert(Object value) {
    return convert_(value, Defaults.DATE_FORMAT);
  }

  static DateTime convert_(Object value, String format) {
    return convert__(value, format, FieldType.DATE);
  }

  static DateTime convert__(Object value, String format, FieldType fieldType) {
    if (value is DateTime) {
      return value;
    } else if (value is int) {
      return DateUtils.millisToDate(value);
    } else if (value is String) {
      try {
        return parse(value, format);
      } catch (e) {
        // nothing to do here
      }
    }

    throw ConversionError(fieldType);
  }

  static DateTime parseNoErr(String sDate, String pattern) {
    try {
      return parse(sDate, pattern);
    } catch (e) {
      return null;
    }
  }

  static DateTime parse(String sDate, String pattern) {
    final int milliseconds =
        CNumber.parseNoErr(sDate); // long value disguised as String?

    if (milliseconds != null && milliseconds > CNumber.MAX_VALUE) {
      return DateUtils.millisToDate(milliseconds);
    }

    return parseSimple(sDate, pattern);
  }

  static DateTime parseSimple(String sDate, String pattern) {
    final String tDate = Utils.trim(sDate);
    // 'XX' in the date string is coerced to 1st of day/month/year
    return Utils.anyBlank([tDate, pattern])
        ? null
        : DateUtils.getFormatter(pattern)
            .parseLoose(tDate.replaceAll("(?i)XX", "01"));
  }

  static DateTime parseSimpleNoErr(String sDate, String pattern) {
    try {
      return parseSimple(sDate, pattern);
    } catch (e) {
      return null;
    }
  }

  static DateTime parseWithMany(String sDate, List<String> patterns) {
    for (String pattern in patterns) {
      DateTime date = parseSimpleNoErr(sDate, pattern);

      if (date != null) {
        return date;
      }
    }

    return null;
  }
}
