/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../../enums/field_type.dart';
import '../../util/utils.dart';
import 'c_decimal.dart';
import 'conversion_error.dart';

class CFloat {
  const CFloat._();

  static bool checkType(Object object) {
    return CDecimal.checkType(object);
  }

  static double convertNoErr(Object value) {
    try {
      return convert(value);
    } on ConversionError {
      return null;
    }
  }

  static double convert(Object value) {
    if (value is double) {
      return value;
    } else if (value is num) {
      return toPrimitive(value);
    } else if (value is String) {
      try {
        return parse(value);
      } catch (e) {
        // nothing to do here
      }
    }

    throw ConversionError(FieldType.FLOAT);
  }

  static double parseNoErr(String value) {
    try {
      return parse(value);
    } catch (e) {
      return null;
    }
  }

  static double parse(String value) {
    final String tValue = Utils.trim(value);
    return Utils.isBlank(tValue) ? null : double.parse(tValue);
  }

  static double toPrimitive(Object value) {
    return (value as num).toDouble();
  }
}
