/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import '../ui/discrete/component/behavior/i_err.dart';
import '../ui/discrete/renderer/features.dart';
import '../ui/discrete/renderer/ui_renderer.dart';
import '../ui/discrete/view_state.dart';
import '../util/log/logger.dart';
import '../util/utils.dart';

class ErrorMarker {
  const ErrorMarker._();

  static bool clear(ViewState view) {
    return mark(view, null, null, null);
  }

  static bool mark(
      ViewState view, UIRenderer renderer, String i18nMsgId, String bundleMsgId,
      [Object args]) {
    bool valid = Utils.isBlank(
        bundleMsgId); // Only bundleMsgId tells if the view was valid (NOT i18nMsgId)
    String msg = ""; // means it has no error

    ErrMsg errMsg = view.getErrMsg();
    errMsg.clear();

    if (!valid) {
      msg = renderer.getMsg(i18nMsgId, bundleMsgId, args);

      if (!Utils.isBlank(msg)) {
        // set error message id in ViewState
        errMsg.set(Utils.toDefault(i18nMsgId, bundleMsgId), args);
      }
    }

    if (msg == null) {
      // unknown message (not to be confused with blank message)
      Logger.warn(
          "Undefined message where i18nMsgId is '$i18nMsgId' and bundleMsgId is '$bundleMsgId'");
      msg = "???";
    }

    if (view.toastError) {
      Features.getAlert_(renderer).error(msg);
    } else if (view.canErr()) {
      (view.getComponent() as IErr).setErrMsg(msg);
    }

    return valid;
  }
}
