/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import 'package:mt_formalyze/com/mt/formalyze/config/config.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/base64.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/io_utils.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/stop_watch.dart';

void main() {
  testInputSmallerThan76Bytes();
  testInputLargerThan76Bytes();
  testEncodeDecode();
  testImage();
}

void testInputSmallerThan76Bytes() {
  String content = "2VBjuj43YtnaJrYYTEq87AdJXbbkkvcZjyPTCyTibW6ibqeDG98P3c";
  String result = Base64.encode__(content.codeUnits, true);
  assert(result ==
      "MlZCanVqNDNZdG5hSnJZWVRFcTg3QWRKWGJia2t2Y1pqeVBUQ3lUaWJXNmlicWVERzk4UDNj");
}

void testInputLargerThan76Bytes() {
  String content =
      "2VBjuj43YtnaJrYYTEq87adjxbbkkvczjyptcytibw6ibqeDG98P3c2VBjuj43YtnaJrYYTEq87adjxbbkkvczjyPTCyTibW6ibqeDG98p3c";
  String result = Base64.encode__(content.codeUnits, false);
  assert(result ==
      "MlZCanVqNDNZdG5hSnJZWVRFcTg3YWRqeGJia2t2Y3pqeXB0Y3l0aWJ3NmlicWVERzk4UDNjMlZCanVqNDNZdG5hSnJZWVRFcTg3YWRqeGJia2t2Y3pqeVBUQ3lUaWJXNmlicWVERzk4cDNj");
}

void testEncodeDecode() {
  String string = "Dart";
  String encoded = Base64.encode(string);
  String decoded = Base64.decodeToString(encoded.codeUnits);

  assert(encoded == "RGFydA==");
  assert(string == decoded);
}

void testImage() {
  Config.DEBUG = true;

  Uint8List content = IOUtils.readFileNoErr(
      "D:/DevEnv/workspaceAndroid/mt_formalyze/test/resources/Twinkle.jpg");

  StopWatch stopWatch = StopWatch.start();
  String base64 = Base64.encode_(content);
  StopWatch.stop(stopWatch);

  stopWatch = StopWatch.start();
  Base64.decode(base64);
  StopWatch.stop(stopWatch);
}
