/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import 'package:mt_formalyze/com/mt/form/structure/string_object_map.dart';
import 'package:mt_formalyze/com/mt/formalyze/enums/dir.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/bean_utils.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/date_utils.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/enum_utils.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/log/logger.dart';
import 'package:mt_formalyze/com/mt/formalyze/validation/converter/c_binary.dart';
import 'package:mt_formalyze/com/mt/formalyze/validation/converter/c_boolean.dart';
import 'package:mt_formalyze/com/mt/formalyze/validation/converter/c_date.dart';
import 'package:mt_formalyze/com/mt/formalyze/validation/converter/c_decimal.dart';
import 'package:mt_formalyze/com/mt/formalyze/validation/converter/c_number.dart';
import 'package:mt_formalyze/com/mt/formalyze/validation/converter/c_string.dart';

void main() {
  Logger.debug('=CBinary================');
  {
    Logger.debug(CBinary.checkType(null));
    Logger.debug(CBinary.checkType(['a', 'b', 'c']));

    Logger.debug(CBinary.checkType([1, 2, 3]));
    Logger.debug(CBinary.checkType(Uint8List(2)));
    Logger.debug(CBinary.checkType(Uint8List.fromList([1, 2, 3])));
  }
  Logger.debug('=CBoolean================');
  {
    Logger.debug(CBoolean.checkType(null));
    Logger.debug(CBoolean.checkType("1"));
    Logger.debug(CBoolean.checkType(1));
    Logger.debug(CBoolean.checkType(1.2));

    Logger.debug(CBoolean.checkType(true));
  }
  Logger.debug('=CDate================');
  {
    Logger.debug(CDate.checkType(null));
    Logger.debug(CDate.checkType("1"));
    Logger.debug(CDate.checkType(1));
    Logger.debug(CDate.checkType(1.2));
    Logger.debug(CDate.checkType(true));

    Logger.debug(CDate.checkType(DateUtils.now()));
  }
  Logger.debug('=CDecimal================');
  {
    Logger.debug(CDecimal.checkType(null));
    Logger.debug(CDecimal.checkType("1"));
    Logger.debug(CDecimal.checkType(1));

    Logger.debug(CDecimal.checkType(1.2));
  }
  Logger.debug('=CNumber================');
  {
    Logger.debug(CNumber.checkType(null));
    Logger.debug(CNumber.checkType("1"));
    Logger.debug(CNumber.checkType(1.2));

    Logger.debug(CNumber.checkType(1));
  }
  Logger.debug('=CString================');
  {
    Logger.debug(CString.checkType(null));
    Logger.debug(CString.checkType(1));
    Logger.debug(CString.checkType(1.2));

    Logger.debug(CString.checkType("1"));
    Logger.debug(CString.checkType('a'));
  }
  Logger.debug('=Enum================');
  {
    Logger.debug(EnumUtils.isEnum(null));
    Logger.debug(EnumUtils.isEnum("1"));
    Logger.debug(EnumUtils.isEnum(1));
    Logger.debug(EnumUtils.isEnum(1.2));
    Logger.debug(EnumUtils.isEnum(true));
    Logger.debug(EnumUtils.isEnum('CodeUnit.Dart'));

    Logger.debug(EnumUtils.isEnum(Dir.LTR));
    Logger.debug(EnumUtils.isEnum(CodeUnit.Dart));
  }
  Logger.debug('=Map================');
  {
    Logger.debug(BeanUtils.isMap(null));
    Logger.debug(BeanUtils.isMap("1"));
    Logger.debug(BeanUtils.isMap(1));
    Logger.debug(BeanUtils.isMap(1.2));
    Logger.debug(BeanUtils.isMap(true));
    Logger.debug(BeanUtils.isMap(Dir.LTR));
    Logger.debug(BeanUtils.isMap(CodeUnit.Dart));
    Logger.debug(BeanUtils.isMap([1, 2, 3]));

    Logger.debug(BeanUtils.isMap({}));
    Logger.debug(BeanUtils.isMap({'a': 1}));
    Logger.debug(BeanUtils.isMap(StringObjectMap.fromMap({'x': 2})));
  }
  Logger.debug('=List================');
  {
    Logger.debug(BeanUtils.isList(null));
    Logger.debug(BeanUtils.isList("1"));
    Logger.debug(BeanUtils.isList(1));
    Logger.debug(BeanUtils.isList(1.2));
    Logger.debug(BeanUtils.isList(true));
    Logger.debug(BeanUtils.isList(Dir.LTR));
    Logger.debug(BeanUtils.isList(CodeUnit.Dart));
    Logger.debug(BeanUtils.isList({}));
    Logger.debug(BeanUtils.isList({'a': 1}));
    Logger.debug(BeanUtils.isList(StringObjectMap.fromMap({'x': 2})));

    Logger.debug(BeanUtils.isList([1, 2, 3]));
  }
  Logger.debug('=ComplexType================');
  {
    Logger.debug(BeanUtils.isComplexType(null));
    Logger.debug(BeanUtils.isComplexType("1"));
    Logger.debug(BeanUtils.isComplexType(1));
    Logger.debug(BeanUtils.isComplexType(1.2));
    Logger.debug(BeanUtils.isComplexType(true));

    Logger.debug(BeanUtils.isComplexType(Dir.LTR));
    Logger.debug(BeanUtils.isComplexType(CodeUnit.Dart));
    Logger.debug(BeanUtils.isComplexType({}));
    Logger.debug(BeanUtils.isComplexType({'a': 1}));
    Logger.debug(BeanUtils.isComplexType(StringObjectMap.fromMap({'x': 2})));
    Logger.debug(BeanUtils.isComplexType([1, 2, 3]));
    Logger.debug(BeanUtils.isComplexType(Object()));
  }
}

enum CodeUnit { Dart, Java }
