/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'package:mt_formalyze/com/mt/formalyze/error/illegal_argument_exception.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/date_utils.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/log/logger.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/utils.dart';

Future<void> main() async {
  // 1. Setup our logger for each log level
  Logger.debug =
      (o, {StackTrace stackTrace}) => log('DEBUG', o, stackTrace: stackTrace);
  Logger.info =
      (o, {StackTrace stackTrace}) => log('INFO', o, stackTrace: stackTrace);
  Logger.warn =
      (o, {StackTrace stackTrace}) => log('WARN', o, stackTrace: stackTrace);
  Logger.error =
      (o, {StackTrace stackTrace}) => log('ERROR', o, stackTrace: stackTrace);

  // 2. Test simple messages without stackTrace
  Logger.debug('Meow');
  Logger.info('Baa');
  Logger.warn('Woof');
  Logger.error('Ouch');

  // 3. Test exceptions
  try {
    double z;
    z++;
    Logger.info(z);
  } catch (e, stackTrace) {
    Logger.error(e, stackTrace: stackTrace);
  }

  Logger.info('2 / 2 = ${await divide(2, 2)}');

  try {
    await divide(2, 0);
  } on IllegalArgumentException catch (e, trace) {
    Logger.error(e.details(), stackTrace: trace);
  }
}

void log(String level, Object o, {StackTrace stackTrace}) => print(
    'CMS | $level | ${DateUtils.now()} | $o | ${Utils.getStackTrace(stackTrace)}');

Future<double> divide(int x, int y) async {
  await validate(y);
  return x / y;
}

Future<void> validate(int num) async {
  if (num == 0) throw IllegalArgumentException('Divide by zero');
}

extension AsyncException on IllegalArgumentException {
  String details() => 'Extension: ${toString()}';
}
