/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'package:flutter_js/javascript_runtime.dart' show JavascriptRuntime;
import 'package:mt_formalyze/com/mt/formalyze/config/config.dart';
import 'package:mt_formalyze/com/mt/formalyze/ui/discrete/action/data/data_comparator.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/log/logger.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/script/script_evaluator.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/utils.dart';

// NOTE: We can not run this without setting up flutter_js with your OS.
// Alternatively, please use main_js.dart in mt_form_flutter_demo project.
void main() {
  Config.DEBUG = true;
  JavascriptRuntime engine = ScriptEvaluator.getEngine(null);
  registerFunctions(engine);

  {
    Logger.debug(ScriptEvaluator.execute("""
        // String s = resolve('?PLATFORM').toString();
        // print(s.toUpperCase());
        bool isMobile = eql(resolve('?PLATFORM'),'Android');// ? 'Mobile' : 'Web'; // woof
        print(isMobile);
        // bool x = val('accounttype') == 'Personal';
        // return x;
      """, null, engine));
  }

  {
    ScriptEvaluator.execute("""
        print(random('meow'));
        print(random('woof'));
    
        var value = "Forgot";
    
        if (val('rememberMe')) {
          value = 'rememberMe';
        }
    
        setVal('msisdn', value);
      """, null, engine);
  }

  {
    Object value = ScriptEvaluator.execute(r'''
        var title = args[0];
        var a = args[1];
        var b = args[2] / 2;
        var c = args[3] * 3;
        
        if (c > 120) {
          c = 120;
        }
        
        var str = 'variables> a: $a ; b: $b ; c: $c' ;
        var sumAB = a + b ;
        var sumABC = a + b + c;
        
        print(str);
        print(title);
        print(sumAB);
        print(sumABC);
        hello(5);
        print('UUID from CMS ${uuid()}');
        return sumABC;
      ''', {
      'args': ['Sums', 10, 30, 50]
    }, engine);
    Logger.debug("Result from 'fn0' script: $value");
  }

  {
    var value = ScriptEvaluator.execute(
        "return 'UUID from CMS \${uuid()}';", {}, engine);
    Logger.debug("Result from 'fn1' script: $value");
  }

  {
    // FIXME Bad state: Variable student not pointing to an object instance: {type: Student, value: Student{name: RT, age: 9}}
    // var name = student.getName();
    ScriptEvaluator.execute("print('Value received: \$student');",
        {'student': Student("RT", 9)}, engine);
  }

  {
    Logger.debug('Cat: ${ScriptEvaluator.execute("'Meow';", null, engine)}');
    Logger.debug(
        'Cow: ${ScriptEvaluator.execute("return 'Moo';", null, engine)}');
  }

  engine.dispose();
}

void registerFunctions(JavascriptRuntime engine) {
  String fnName = 'hello';
  engine.setupBridge(fnName, (args) => Logger.debug('Hello World: ${args[0]}'));
  engine.evaluate(
      "var $fnName = function(num) { sendMessage('$fnName', JSON.stringify([num])); };");

  fnName = 'uuid';
  engine.setupBridge(fnName, (args) => Utils.getUUID());
  engine.evaluate(
      "var $fnName = function() { return sendMessage('$fnName', ''); };");

  fnName = 'length';
  engine.setupBridge(fnName, (args) => args[0].length);
  engine.evaluate(
      "var $fnName = function(value) { return sendMessage('$fnName', JSON.stringify([value])); };");

  fnName = 'val';
  engine.setupBridge(fnName, (args) => !true);
  engine.evaluate(
      "var $fnName = function(viewId) { return sendMessage('$fnName', JSON.stringify([viewId])); };");

  fnName = 'setVal';
  engine.setupBridge(
      fnName, (args) => Logger.debug('setVal(${args[0]}, ${args[1]})'));
  engine.evaluate(
      "var $fnName = function(viewId, value) { sendMessage('$fnName', JSON.stringify([viewId, value])); };");

  fnName = 'random';
  engine.setupBridge(fnName, (args) async => await random(args[0]));
  engine.evaluate(
      "var $fnName = function(viewId) { return sendMessage('$fnName', JSON.stringify([viewId])); };");

  fnName = 'eql';
  engine.setupBridge(fnName, (args) => eql(args[0], args[1]));
  engine.evaluate(
      "var $fnName = function(obj1, obj2) { return sendMessage('$fnName', JSON.stringify([obj1, obj2])); };");

  fnName = 'resolve';
  engine.setupBridge(fnName, (args) async => await resolve(args[0]));
  engine.evaluate(
      "var $fnName = function(value) { return sendMessage('$fnName', JSON.stringify([value])); };");
}

Future<Object> random(String viewId) async => '$viewId = ${Utils.getUUID()}';

bool eql(Object obj1, Object obj2) => !neql(obj1, obj2);

bool neql(Object obj1, Object obj2) => DataComparator.isChanged(obj1, obj2);

Future<Object> resolve(String value) async => 'Android';

class Student {
  final String name;
  final int age;

  const Student(this.name, this.age);

  String getName() => name;

  @override
  String toString() => 'Student{name: $name, age: $age}';
}
