/*
 * ----------------------------------------------------------------------------
 * Copyright © 2009 by Mobile-Technologies Limited. All rights reserved. All intellectual property rights in and/or in
 * the computer program and its related documentation and technology are the sole Mobile-Technologies Limited property.
 * This computer program is under Mobile-Technologies Limited copyright and cannot be in whole or in part reproduced,
 * sublicensed, leased, sold or used in any form or by any means, including without limitation graphic, electronic,
 * mechanical, photocopying, recording, taping or information storage and retrieval systems without Mobile-Technologies
 * Limited prior written consent. The downloading, exporting or reexporting of this computer program or any related
 * documentation or technology is subject to any export rules, including US regulations.
 * ----------------------------------------------------------------------------
 */

import 'dart:typed_data' show Uint8List;

import 'package:mt_formalyze/com/mt/formalyze/config/config.dart';
import 'package:mt_formalyze/com/mt/formalyze/enums/code.dart';
import 'package:mt_formalyze/com/mt/formalyze/ui/discrete/action/handler/rmi/mpos_http_transport_se.dart';
import 'package:mt_formalyze/com/mt/formalyze/ui/discrete/action/handler/rmi/mpos_https_transport_se.dart';
import 'package:mt_formalyze/com/mt/formalyze/ui/discrete/action/handler/rmi/transport.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/io_utils.dart';
import 'package:mt_formalyze/com/mt/formalyze/util/log/logger.dart';

Future<void> main() async {
  Config.DEBUG = true;
  const int timeout = 12000;

  await test('http://192.168.8.34:8081/CredentialWebService?wsdl', timeout,
      null, request, null, null);
  await test('https://192.168.8.34:9081/CredentialWebService?wsdl', timeout,
      null, request, null, null);

  await test(
      'https://192.168.8.34:18449/nifi-api/system-diagnostics',
      timeout,
      MposHttpTransportSE.REQ_METHOD_GET,
      null,
      IOUtils.readFile(
          'E:/Desktop/NiFi/Certificates/CN=tanveer.ulislam_OU=NIFI.p12'),
      'gZ5OiCRsLF152VjE7npN2xlVC6t1DHLLr7+olS0RH3M');
}

Future<void> test(String url, int timeout, String method, String request,
    Uint8List certificate, String keyPass) async {
  Logger.debug('===========================');
  try {
    Transport transport = url.startsWith(Code.HTTPS.code)
        ? MposHttpsTransportSE(url, timeout, certificate, keyPass,
            method: method)
        : MposHttpTransportSE(url, timeout);
    transport.debug = Config.DEBUG;
    await transport.call(request);
  } catch (e, stackTrace) {
    Logger.error(e, stackTrace: stackTrace);
  } finally {
    Logger.debug('===========================');
  }
}

const request = '''
<v:Envelope
	xmlns:i="http://www.w3.org/2001/XMLSchema-instance"
	xmlns:d="http://www.w3.org/2001/XMLSchema"
	xmlns:c="http://schemas.xmlsoap.org/soap/encoding/"
	xmlns:v="http://schemas.xmlsoap.org/soap/envelope/">
	<v:Header>
		<n0:info xmlns:n0="http://mobile-technologies.com">
			<offlineLoginFlag>false</offlineLoginFlag>
			<operationName>getUserInfo</operationName>
			<instanceId>800001</instanceId>
		</n0:info>
	</v:Header>
	<v:Body>
		<n1:getUserInfo
			xmlns:n1="http://mobile-technologies.com/eai/isl/">
			<userInfoReq i:type=":">
				<userName i:type="d:string">wcminzer</userName>
				<idCompany i:type="d:int">1</idCompany>
				<instanceId i:type="d:int">800001</instanceId>
			</userInfoReq>
			<instanceId i:type="d:int">800001</instanceId>
			<offlineLoginFlag i:type="d:boolean">false</offlineLoginFlag>
			<operationName i:type="d:string">getUserInfo</operationName>
		</n1:getUserInfo>
	</v:Body>
</v:Envelope>
''';
